<?php

namespace MetodikaTI\Http\Controllers\Back\DiasFestivos;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\DiaFestivo;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\DiaFestivo\CreateDiaFestivoRequest;
use MetodikaTI\Http\Requests\Back\DiaFestivo\EditDiaFestivoRequest;
use MetodikaTI\Library\URI;

class DiasFestivosController extends Controller
{


    public function index()
    {

        $data = DiaFestivo::get();
        return view('back.dias-festivos.index', array('data' => $data, 'permitions' => URI::checkPermitions()));
    }


    public function create()
    {
        return view('back.dias-festivos.create');
    }


    public function store(CreateDiaFestivoRequest $request)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        $data = new DiaFestivo();
        $data->name = $request->name;
        $data->date = Carbon::createFromFormat("d/m/Y", $request->date)->format("Y-m-d");

        if ($data->save()) {

            DB::commit();

            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('dias_festivos_index');

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function edit($id)
    {
        $data = DiaFestivo::find(base64_decode($id));

        return view('back.dias-festivos.edit', array("data" => $data));
    }


    public function update(EditDiaFestivoRequest $request, $id)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        $data = DiaFestivo::find(base64_decode($id));
        $data->name = $request->name;
        $data->date = Carbon::createFromFormat("d/m/Y", $request->date)->format("Y-m-d");

        if ($data->save()) {

            DB::commit();

            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('dias_festivos_index');

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function destroy($id)
    {

        $response = array(
            'status' => true,
            'message' => 'El usuario ha sido eliminado correctamente.',
            'url' => route('dias_festivos_index'),
        );

        $data = DiaFestivo::find(base64_decode($id));

        if ($data != null) {
            if ($data->delete()) {

                $response = [
                    'message' => 'Se ha eliminado el registro correctamente.',
                    'status' => true,
                    'url' => route('dias_festivos_index')
                ];

            }
        } else {
            $response = [
                'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                'status' => false
            ];
        }

        return response()->json($response);
    }


}
