<?php

namespace MetodikaTI\Http\Controllers\Back\Liga;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\Handicap;
use MetodikaTI\HandicapEachCategory;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\Handicap\CreateHandicapRequest;
use MetodikaTI\League;

class HandicapController extends Controller
{

    public function create($league_id)
    {

        $league = League::find(base64_decode($league_id));
        $handycap = Handicap::where("league_id", base64_decode($league_id))->first();

        //Si ya existe este catalogo en BD, lo redireccionamos a edit
        if ($handycap != null) {
            return redirect(route('handicap.edit', $league_id));
        } else {
            return view('back.league.handycap.create', array("league" => $league));
        }

    }


    public function store(CreateHandicapRequest $request, $id)
    {

        //dd($request->all());

        $response = array('status' => false);

        DB::beginTransaction();

        $handycap = new Handicap();
        $handycap->league_id = $id;
        $handycap->how_many_games_to_get_new_handicap = $request->how_many_games_to_get_new_handicap;
        $handycap->weeks_to_complete_games = $request->weeks_to_complete_games;
        $handycap->news_players_how_many_games_play_on_first_week = $request->news_players_how_many_games_play_on_first_week;
        $handycap->dummy_by_team = $request->dummy_by_team;

        if ($handycap->save()) {

            //Almacenamos los nombres de los equipos
            foreach ($request->configuration_by_category as $team_categories_id => $value) {

                $handicap_each_categories = new HandicapEachCategory();
                $handicap_each_categories->team_category_id = $team_categories_id;

                //Normal
                $handicap_each_categories->base_normal = $request->configuration_by_category[$team_categories_id]["base_normal"];
                $handicap_each_categories->male_normal = $request->configuration_by_category[$team_categories_id]["male_normal"];
                $handicap_each_categories->male_normal_tope = $request->configuration_by_category[$team_categories_id]["male_normal_tope"];
                $handicap_each_categories->female_normal = $request->configuration_by_category[$team_categories_id]["female_normal"];
                $handicap_each_categories->female_normal_tope = $request->configuration_by_category[$team_categories_id]["female_normal_tope"];
                $handicap_each_categories->juvenil_normal = $request->configuration_by_category[$team_categories_id]["juvenil_normal"];
                $handicap_each_categories->juvenil_normal_tope = $request->configuration_by_category[$team_categories_id]["juvenil_normal_tope"];
                $handicap_each_categories->especial_normal = $request->configuration_by_category[$team_categories_id]["especial_normal"];
                $handicap_each_categories->especial_normal_tope = $request->configuration_by_category[$team_categories_id]["especial_normal_tope"];

                //Dummy
                $handicap_each_categories->base_dummy = $request->configuration_by_category[$team_categories_id]["base_dummy"];
                $handicap_each_categories->male_limit_dummy = $request->configuration_by_category[$team_categories_id]["male_limit_dummy"];
                $handicap_each_categories->female_limit_dummy = $request->configuration_by_category[$team_categories_id]["female_limit_dummy"];
                $handicap_each_categories->juvenil_limit_dummy = $request->configuration_by_category[$team_categories_id]["juvenil_limit_dummy"];
                $handicap_each_categories->especial_limit_dummy = $request->configuration_by_category[$team_categories_id]["especial_limit_dummy"];

                $handicap_each_categories->save();
            }


            DB::commit();
            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('jugadores_create', base64_encode($id));

        } else {

            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }

        return response()->json($response);
    }


    public function edit($id)
    {

        $league = League::find(base64_decode($id));
        $handycap = Handicap::where("league_id", base64_decode($id))->first();

        return view('back.league.handycap.edit', array("league" => $league, "handycap" => $handycap));
    }


    public function update(CreateHandicapRequest $request, $id)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        
        $handycap = Handicap::where("league_id", $id)->first();
        $handycap->how_many_games_to_get_new_handicap = $request->how_many_games_to_get_new_handicap;
        $handycap->weeks_to_complete_games = $request->weeks_to_complete_games;
        $handycap->news_players_how_many_games_play_on_first_week = $request->news_players_how_many_games_play_on_first_week;
        $handycap->dummy_by_team = $request->dummy_by_team;
        $handycap->retroactivo = $request->retroactivo == "on" ? 1 : 0;

        error_log("retroactivo ==>". $handycap->retroactivo);
        
        if ($handycap->save()) {
            
            //Almacenamos los nombres de los equipos
            foreach ($request->configuration_by_category as $team_categories_id => $value) {
                
                $handicap_each_categories = HandicapEachCategory::where("team_category_id", $team_categories_id)->first();
                
                // error_log( "recibe data". $request->configuration_by_category[$team_categories_id]["hombre_prom_cond_esp"] );

                if ($handicap_each_categories == null) {
                    $handicap_each_categories = new HandicapEachCategory();
                    $handicap_each_categories->team_category_id = $team_categories_id;
                }

                //Normal
                $handicap_each_categories->base_normal = $request->configuration_by_category[$team_categories_id]["base_normal"];
                $handicap_each_categories->male_normal = $request->configuration_by_category[$team_categories_id]["male_normal"];
                $handicap_each_categories->male_normal_tope = $request->configuration_by_category[$team_categories_id]["male_normal_tope"];
                $handicap_each_categories->female_normal = $request->configuration_by_category[$team_categories_id]["female_normal"];
                $handicap_each_categories->female_normal_tope = $request->configuration_by_category[$team_categories_id]["female_normal_tope"];
                $handicap_each_categories->juvenil_normal = $request->configuration_by_category[$team_categories_id]["juvenil_normal"];
                $handicap_each_categories->juvenil_normal_tope = $request->configuration_by_category[$team_categories_id]["juvenil_normal_tope"];
                $handicap_each_categories->especial_normal = $request->configuration_by_category[$team_categories_id]["especial_normal"];
                $handicap_each_categories->especial_normal_tope = $request->configuration_by_category[$team_categories_id]["especial_normal_tope"];

                //Dummy
                $handicap_each_categories->base_dummy = $request->configuration_by_category[$team_categories_id]["base_dummy"];
                $handicap_each_categories->male_limit_dummy = $request->configuration_by_category[$team_categories_id]["male_limit_dummy"];
                $handicap_each_categories->female_limit_dummy = $request->configuration_by_category[$team_categories_id]["female_limit_dummy"];
                $handicap_each_categories->juvenil_limit_dummy = $request->configuration_by_category[$team_categories_id]["juvenil_limit_dummy"];
                $handicap_each_categories->especial_limit_dummy = $request->configuration_by_category[$team_categories_id]["especial_limit_dummy"];

                //CONDICION ESPECIAL                             

                $handicap_each_categories->hombre_prom_cond_esp           = $request->configuration_by_category[$team_categories_id]["hombre_prom_cond_esp"];
                $handicap_each_categories->hombre_condicion_prom_cond_esp = $request->configuration_by_category[$team_categories_id]["hombre_condicion_prom_cond_esp"];
                $handicap_each_categories->hombre_hc_cond_esp             = $request->configuration_by_category[$team_categories_id]["hombre_hc_cond_esp"];         
                $handicap_each_categories->hombre_aplica_cond_esp         = $request->aplica_hombre;
    
                $handicap_each_categories->mujer_prom_cond_esp            = $request->configuration_by_category[$team_categories_id]["mujer_prom_cond_esp"];
                $handicap_each_categories->mujer_condicion_prom_cond_esp  = $request->configuration_by_category[$team_categories_id]["mujer_condicion_prom_cond_esp"];
                $handicap_each_categories->mujer_hc_cond_esp              = $request->configuration_by_category[$team_categories_id]["mujer_hc_cond_esp"];
                $handicap_each_categories->mujer_aplica_cond_esp          = $request->aplica_mujer;

                $handicap_each_categories->juvenil_prom_cond_esp           = $request->configuration_by_category[$team_categories_id]["juvenil_prom_cond_esp"];
                $handicap_each_categories->juvenil_condicion_prom_cond_esp = $request->configuration_by_category[$team_categories_id]["juvenil_condicion_prom_cond_esp"];
                $handicap_each_categories->juvenil_hc_cond_esp             = $request->configuration_by_category[$team_categories_id]["juvenil_hc_cond_esp"];
                $handicap_each_categories->juvenil_aplica_cond_esp         = $request->aplica_juvenil;

                $handicap_each_categories->especial_prom_cond_esp           = $request->configuration_by_category[$team_categories_id]["especial_prom_cond_esp"];
                $handicap_each_categories->especial_condicion_prom_cond_esp = $request->configuration_by_category[$team_categories_id]["especial_condicion_prom_cond_esp"];
                $handicap_each_categories->especial_hd_cond_esp             = $request->configuration_by_category[$team_categories_id]["especial_hd_cond_esp"];
                $handicap_each_categories->especial_aplica_cond_esp         = $request->aplica_especial;

                $handicap_each_categories->save();
            }


            DB::commit();
            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('handicap.edit', base64_encode($id));

        } else {

            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }

        return response()->json($response);
    }


}