<?php

namespace MetodikaTI\Http\Controllers\Back\Liga;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\CategoryRule;
use MetodikaTI\DiaFestivo;
use MetodikaTI\FrecuencyCustomLeague;
use MetodikaTI\Handicap;
use MetodikaTI\HandicapEachCategory;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\League\LeaguePrincipalInformationRequest;
use MetodikaTI\League;
use MetodikaTI\LeagueContact;
use MetodikaTI\Library\URI;
use MetodikaTI\TeamAndCategorySpecification;

class LigaController extends Controller
{


    public function index()
    {
        $data = League::get();
        
        return view('back.league.principal_information.index', array('data' => $data, 'permitions' => URI::checkPermitions()));
    }


    public function create()
    {

        $dias_festivos = DiaFestivo::pluck("name", "date")->toArray();

        return view('back.league.principal_information.create', compact("dias_festivos"));
    }


    public function store(LeaguePrincipalInformationRequest $request)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        /******************************************** INFORMACION DE LA LIGA **************************************************/
        $data = new League();
        $data->date_league = ($request->date_league != "") ? Carbon::createFromFormat("d/m/Y", $request->date_league)->format("Y-m-d") : "";
        $data->city = $request->city;
        $data->state = $request->state;
        $data->country = $request->country;
        $data->league_name = $request->league_name;
        $data->place = $request->place;
        $data->honorific_name = $request->honorific_name;
        $data->date_start_league = ($request->date_start_league != "") ? Carbon::createFromFormat("d/m/Y", $request->date_start_league)->format("Y-m-d") : "";
        $data->date_end_league = ($request->date_end_league != "") ? Carbon::createFromFormat("d/m/Y", $request->date_end_league)->format("Y-m-d") : "";
        $data->hour_start = $request->hour_start . ":00";
        $data->frecuency = $request->frecuency;
        $data->holidays_date = $request->add_dias_festivos;

        //Dias disponibles
        $days_available = "";
        $days_available .= (isset($request->lunes)) ? "1," : "0,";
        $days_available .= (isset($request->martes)) ? "1," : "0,";
        $days_available .= (isset($request->miercoles)) ? "1," : "0,";
        $days_available .= (isset($request->jueves)) ? "1," : "0,";
        $days_available .= (isset($request->viernes)) ? "1," : "0,";
        $days_available .= (isset($request->sabado)) ? "1," : "0,";
        $days_available .= (isset($request->domingo)) ? "1" : "0";

        $data->days_available = $days_available;

        if ($data->save()) {

            DB::commit();

            //Subimos el reglamento
            if (!is_dir("assets/rules/" . $data->id)) {
                mkdir("assets/rules/" . $data->id, 0755);
            }

            if ($request->hasFile("rules_path")) {
                $file_name = md5(date("d-m-Y H:i:s")) . "." . $request->file("rules_path")->getClientOriginalExtension();
                $request->file("rules_path")->move("assets/rules/" . $data->id, $file_name);
                $data->rules_path = "assets/rules/" . $data->id . "/" . $file_name;
                $data->save();
            }


            //En caso de que tenga un logo de la liga,lo subimos
            if ($request->hasFile("league_logo")) {

                //Subimos el reglamento
                if (!is_dir("assets/league/" . $data->id)) {
                    mkdir("assets/league/" . $data->id, 0755);
                }

                $file_name = md5(date("d-m-Y H:i:s")) . "_league_logo_." . $request->file("league_logo")->getClientOriginalExtension();
                $request->file("league_logo")->move("assets/league/" . $data->id, $file_name);
                $data->league_logo = "assets/league/" . $data->id . "/" . $file_name;
                $data->save();

            }


            /******************************************** CONTACTOS **************************************************/
            foreach ($request->role as $key => $value) {
                if ($request->nip[$key] != "" && $request->role[$key] != "" && $request->name_complete[$key] != "" && $request->cellphone[$key] != "" && $request->email[$key] != "") {
                    $league_contacts = new LeagueContact();
                    $league_contacts->league_id = $data->id;
                    $league_contacts->role = $request->role[$key];
                    $league_contacts->name_complete = $request->name_complete[$key];
                    $league_contacts->cellphone = $request->cellphone[$key];
                    $league_contacts->email = $request->email[$key];
                    $league_contacts->nip = $request->nip[$key];
                    error_log( "NIPPP =>". $request->nip[$key] );
                    $league_contacts->save();
                }
            }


            /******************************************** DIAS DISPONIBLES **************************************************/
            //En caso de que exista una frecuenta de intervalo de fechas customisable, ingresaremos las fechas disponibles en la tabla "frecuency_custom_leagues"
            if ($request->frecuency == "intervalo_fechas") {

                $frecuency_custom_leagues = array();
                foreach (explode(",", $request->interval_dates) as $date_available) {
                    $custom_date = array();
                    $custom_date["league_id"] = $data->id;
                    $custom_date["date_available"] = Carbon::createFromFormat("d/m/Y", $date_available)->format("Y-m-d");
                    $frecuency_custom_leagues[] = $custom_date;
                }

                FrecuencyCustomLeague::insert($frecuency_custom_leagues);

            } else if ($request->frecuency == "semana") {
                $data->frecuency_week = $request->cada_semana;
                $data->save();
            }


            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('player_types_create', base64_encode($data->id));

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function edit($id)
    {

        $dias_festivos = DiaFestivo::pluck("name", "date")->toArray();        

        $league = League::find(base64_decode($id));

        return view('back.league.principal_information.edit', array("league" => $league, "dias_festivos" => $dias_festivos));
    }


    public function update(LeaguePrincipalInformationRequest $request, $id)
    {

        //dd($request->all());

        $response = array('status' => false);

        DB::beginTransaction();

        /******************************************** INFORMACION DE LA LIGA **************************************************/
        $data = League::find($id);
        $data->date_league = ($request->date_league != "") ? Carbon::createFromFormat("d/m/Y", $request->date_league)->format("Y-m-d") : "";
        $data->city = $request->city;
        $data->state = $request->state;
        $data->country = $request->country;
        $data->league_name = $request->league_name;
        $data->place = $request->place;
        $data->honorific_name = $request->honorific_name;
        $data->date_start_league = ($request->date_start_league != "") ? Carbon::createFromFormat("d/m/Y", $request->date_start_league)->format("Y-m-d") : "";
        $data->date_end_league = ($request->date_end_league != "") ? Carbon::createFromFormat("d/m/Y", $request->date_end_league)->format("Y-m-d") : "";
        $data->hour_start = $request->hour_start . ":00";
        $data->frecuency = $request->frecuency;
        $data->frecuency_week = 0;
        $data->holidays_date = $request->add_dias_festivos;

        //Dias disponibles
        $days_available = "";
        $days_available .= (isset($request->lunes)) ? "1," : "0,";
        $days_available .= (isset($request->martes)) ? "1," : "0,";
        $days_available .= (isset($request->miercoles)) ? "1," : "0,";
        $days_available .= (isset($request->jueves)) ? "1," : "0,";
        $days_available .= (isset($request->viernes)) ? "1," : "0,";
        $days_available .= (isset($request->sabado)) ? "1," : "0,";
        $days_available .= (isset($request->domingo)) ? "1" : "0";

        $data->days_available = $days_available;

        if ($data->save()) {

            DB::commit();

            if ($request->hasFile("rules_path")) {

                //Subimos el reglamento
                if (!is_dir("assets/rules/" . $data->id)) {
                    mkdir("assets/rules/" . $data->id, 0755);
                }

                if ($request->hasFile("rules_path")) {
                    $file_name = md5(date("d-m-Y H:i:s")) . "." . $request->file("rules_path")->getClientOriginalExtension();
                    $request->file("rules_path")->move("assets/rules/" . $data->id, $file_name);

                    //Eliminamos el archivo anterior
                    if (is_file($data->rules_path)) {
                        unlink($data->rules_path);
                    }

                    //Subimos el archivo nuevo
                    $data->rules_path = "assets/rules/" . $data->id . "/" . $file_name;
                    $data->save();
                }

            }


            //En caso de que tenga un logo de la liga,lo subimos
            if ($request->hasFile("league_logo")) {

                //Subimos el reglamento
                if (!is_dir("assets/league/" . $data->id)) {
                    mkdir("assets/league/" . $data->id, 0755);
                }

                //Eliminamos la imagen anterior
                if (file_exists($data->league_logo)) {
                    unlink($data->league_logo);
                }

                $file_name = md5(date("d-m-Y H:i:s")) . "_league_logo_." . $request->file("league_logo")->getClientOriginalExtension();
                $request->file("league_logo")->move("assets/league/" . $data->id, $file_name);
                $data->league_logo = "assets/league/" . $data->id . "/" . $file_name;
                $data->save();

            }


            /******************************************** CONTACTOS **************************************************/
            //Primero eliminamos todos los contactos y los volvemos a agregar
            LeagueContact::where("league_id", $data->id)->delete();

            foreach ($request->role as $key => $value) {
                if ($request->role[$key] != "" && $request->name_complete[$key] != "" && $request->cellphone[$key] != "" && $request->email[$key] != "") {
                    $league_contacts = new LeagueContact();
                    $league_contacts->league_id = $data->id;
                    $league_contacts->role = $request->role[$key];
                    $league_contacts->name_complete = $request->name_complete[$key];
                    $league_contacts->cellphone = $request->cellphone[$key];
                    $league_contacts->email = $request->email[$key];
                    $league_contacts->nip = $request->nip[$key];
                    $league_contacts->save();
                }
            }


            /******************************************** DIAS DISPONIBLES **************************************************/
            //Primero eliminamos todas las fechas customizables y las volvemos a agregar
            FrecuencyCustomLeague::where("league_id", $data->id)->delete();

            //En caso de que exista una frecuenta de intervalo de fechas customisable, ingresaremos las fechas disponibles en la tabla "frecuency_custom_leagues"
            if ($request->frecuency == "intervalo_fechas") {

                $frecuency_custom_leagues = array();
                foreach (explode(",", $request->interval_dates) as $date_available) {
                    $custom_date = array();
                    $custom_date["league_id"] = $data->id;
                    $custom_date["date_available"] = Carbon::createFromFormat("d/m/Y", $date_available)->format("Y-m-d");
                    $frecuency_custom_leagues[] = $custom_date;
                }

                FrecuencyCustomLeague::insert($frecuency_custom_leagues);

            } else if ($request->frecuency == "semana") {
                $data->frecuency_week = $request->cada_semana;
                $data->save();
            }


            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('league.edit', base64_encode($data->id));

        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);
    }


    public function delete_contact($contact_id)
    {

        LeagueContact::find($contact_id)->delete();

        return 1;
    }


    public function destroy($id)
    {

        $response = array(
            'status' => true,
            'message' => 'El usuario ha sido eliminado correctamente.',
            'url' => route('league_index'), //route('colors_index')
        );

        $data = League::find(base64_decode($id));

        if ($data != null) {

            //Eliminamos el archivo en caso de tenerlo
            if (file_exists($data->rules_path)) {
                unlink($data->rules_path);
            }

            if ($data->delete()) {

                //Eliminamos todos los derivados
                LeagueContact::where("league_id", base64_decode($id))->delete();
                FrecuencyCustomLeague::where("league_id", base64_decode($id))->delete();

                $response = [
                    'message' => 'Se ha eliminado el registro correctamente.',
                    'status' => true,
                    'url' => route('league_index')
                ];
            }

        } else {
            $response = [
                'message' => 'No hemos podido eliminar el registro, intentalo nuevamente.',
                'status' => false
            ];
        }

        return response()->json($response);
    }


    public function check_if_route_is_enabled(Request $request)
    {

        //league_id

        //
        //
        //
        //
        //
        //
        //
        //

        $response = array();
        $response["error"] = true;
        $response["message"] = "";

        $option_action = $request->option_action;

        //Primeramente tiene que dar de alta la informacion principal
        if ($request->league_id == null) {

            $response["error"] = true;
            $response["message"] = "Ingresa la información de la sección 'Información principal' para continuar.";

        } else {

            $league = League::find(base64_decode($request->league_id));

            if ($league == null) {

                $response["error"] = true;
                $response["message"] = "Ingresa la información de la sección 'Información principal' para continuar.";

            } else if ($option_action == "informacion_principal") {

                $response["error"] = false;
                $response["message"] = "";

            } else if ($option_action == "tipo_jugadores") {

                $response["error"] = false;
                $response["message"] = "";

            } else if ($option_action == "equipos") {

                $response["error"] = false;
                $response["message"] = "";

            } else if ($option_action == "reglas_categorias" || $option_action == "handicap" || $option_action == "premiacion") {


                //Revisamos que halla agregado las categorias
                $team_and_category_specifications = TeamAndCategorySpecification::where("league_id", $league->id);

                if ($team_and_category_specifications->count() > 0) {
                    $response["error"] = false;
                    $response["message"] = "";
                } else {
                    $response["error"] = true;
                    $response["message"] = "Ingresa la información de la sección 'Equipos' para continuar.";
                }


            } else if ($option_action == "jugadores") {

                $category_rule = CategoryRule::where("league_id", $league->id)->first();

                if ($category_rule != null) {
                    $response["error"] = false;
                    $response["message"] = "";
                } else {
                    $response["error"] = true;
                    $response["message"] = "Ingresa la información de la sección 'Reglas de categorías' para continuar.";
                }

            } else if ($option_action == "rol_juegos") {

                $team_and_category_specifications = TeamAndCategorySpecification::where("league_id", $league->id);
                $category_rule = CategoryRule::where("league_id", $league->id)->first();

                if ($team_and_category_specifications->count() > 0 && $category_rule != null) {
                    $response["error"] = false;
                    $response["message"] = "";
                } else {
                    $response["error"] = true;
                    $response["message"] = "Ingresa la información de todas las secciónes para continuar.";
                }

            }

        }

        return response()->json($response);
    }





}
















