<?php

namespace MetodikaTI\Http\Controllers\Back\Liga;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\CategoryRule;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\PlayerInEachCategory\CreatePlayerInEachCategoryRequest;
use MetodikaTI\Http\Requests\Back\Players\CreatePlayerRequest;
use MetodikaTI\League;
use MetodikaTI\Player;
use MetodikaTI\PlayerInTeamEachCategory;
use MetodikaTI\PlayerRules;
use MetodikaTI\PlayerType;
use MetodikaTI\TeamAndCategorySpecification;
use MetodikaTI\TeamCategory;
use MetodikaTI\TeamEachCategory;

class PlayerController extends Controller
{


    public function create($league_id)
    {

        $league = League::find(base64_decode($league_id));
        $player_rules = PlayerRules::where("league_id", $league->id)->first();
        $players_obj = Player::where("name", "!=", "DUMMY")->orderBy("nickname", "ASC")->get();

        //Generamos el array para los jugadores
        $players = array();
        foreach ($players_obj as $key => $value) {

            if ($value->nickname != "") {
                $players[$value->id] = $value->nickname;
            } else {
                $players[$value->id] = $value->name . " " . $value->last_name . " " . $value->second_last_name;
            }

        }
        

        //Si ya existen las reglas de las categorias
        if ($player_rules != null) {
            return redirect(route('jugadores.edit', $league_id));
        } else {
            return view('back.league.players_in_league.create', array("league" => $league, "players" => $players));
        }

    }


    public function store(CreatePlayerInEachCategoryRequest $request, $id)
    {


        $response = array('status' => false);

        $league = League::find($id);
        $team_and_category_specifications = TeamAndCategorySpecification::where("league_id", $id)->first();
        $team_categories = TeamCategory::where("team_and_category_specifications_id", $team_and_category_specifications->id)->get();
        $player_types = PlayerType::where("league_id", $id)->first();

        DB::beginTransaction();

        //Primero insertamos las reglas de jugadores
        $player_rules = new PlayerRules();
        $player_rules->league_id = $id;
        $player_rules->players_start_with_zero_handicap = $request->players_start_with_zero_handicap;
        $player_rules->limit_date_to_register_player = Carbon::createFromFormat("d/m/Y", $request->limit_date_to_register_player)->format("Y-m-d");

        if ($request->can_players_change_between_teams == 1) {

            $player_rules->can_players_change_between_teams = 1;
            $player_rules->limit_date = Carbon::createFromFormat("d/m/Y", $request->limit_date)->format("Y-m-d");

            if ($request->transfer_player_with_handicap == 0) {
                $player_rules->transfer_player_with_handicap = 0;
                $player_rules->how_many_games_needs_to_play_to_recover_handicap = $request->how_many_games_needs_to_play_to_recover_handicap;
            }

        } else {

            $player_rules->can_players_change_between_teams = 0;
            $player_rules->limit_date = null;
            $player_rules->transfer_player_with_handicap = 1;
            $player_rules->how_many_games_needs_to_play_to_recover_handicap = 0;

        }

        if ($player_rules->save()) {

            //Ahora insertamos a los jugadores
            foreach ($request->jugadores as $team_each_categories_id => $players) {
                foreach ($players as $key_player => $player_id) {
                    if ($player_id != "") {
                        $player_in_team_each_categories = new PlayerInTeamEachCategory();
                        $player_in_team_each_categories->team_each_categories_id = $team_each_categories_id;
                        $player_in_team_each_categories->player_id = $player_id;
                        $player_in_team_each_categories->player_handy_cap_in_tournament = $request->jugador_handy_cap[$team_each_categories_id][$key_player];
                        $player_in_team_each_categories->save();
                    }
                }
            }

            DB::commit();


            /*********** Actualización: ***********/
            //Verificaremos si involucran equipos DUMMY para insertar sus jugadores de forma automatica, esto servirá al momento de agregar el puntaje

            //Primero obtenemos todos los jugadores dummy que tenemos registrados en caso de que tengamos que asignarlos
            $players_dummys = Player::where("name", "DUMMY")->get();

            //Verificamos por categoria si contiene un equipo dummy para asignar jugadores
            foreach ($team_categories as $team_category) {

                $has_dummy_team = TeamEachCategory::where("team_category_id", $team_category->id)->where("team_id", 39)->first();

                //Si contiene un equipo dummy, asignamos los jugadores
                if ($has_dummy_team != null) {

                    $dummy_players_already_registered = PlayerInTeamEachCategory::where("team_each_categories_id", $has_dummy_team->id)->count();

                    for ($i = 0; $i < ($player_types->type_game - $dummy_players_already_registered); $i++) {
                        $player_in_team_each_categories = new PlayerInTeamEachCategory();
                        $player_in_team_each_categories->team_each_categories_id = $has_dummy_team->id;
                        $player_in_team_each_categories->player_id = $players_dummys[$i]->id;
                        $player_in_team_each_categories->player_handy_cap_in_tournament = 0;
                        $player_in_team_each_categories->save();
                    }

                }

            }


            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('premiacion_create', base64_encode($id));
        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);

    }


    public function edit($league_id)
    {

        $league = League::find(base64_decode($league_id));
        $player_rules = PlayerRules::where("league_id", $league->id)->first();
        $players_obj = Player::where("name", "!=", "DUMMY")->orderBy("nickname", "ASC")->get();

        //Generamos el array para los jugadores
        $players = array();
        foreach ($players_obj as $key => $value) {
            
            if ($value->nickname != "") {
                $players[$value->id] = $value->nickname . " / " . $value->name . " " . $value->last_name . " " . $value->second_last_name;
            } else {
                $players[$value->id] = $value->name . " " . $value->last_name . " " . $value->second_last_name;
            }
            
        }        

        return view('back.league.players_in_league.edit', array("league" => $league, "players" => $players, "player_rules" => $player_rules));

    }


    public function update(CreatePlayerInEachCategoryRequest $request, $id)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        $league = League::find($id);
        $team_and_category_specifications = TeamAndCategorySpecification::where("league_id", $id)->first();
        $team_categories = TeamCategory::where("team_and_category_specifications_id", $team_and_category_specifications->id)->get();
        $player_types = PlayerType::where("league_id", $id)->first();


        //Primero insertamos las reglas de jugadores
        $player_rules = PlayerRules::where("league_id", $id)->first();
        $player_rules->league_id = $id;
        $player_rules->players_start_with_zero_handicap = $request->players_start_with_zero_handicap;
        $player_rules->limit_date_to_register_player = Carbon::createFromFormat("d/m/Y", $request->limit_date_to_register_player)->format("Y-m-d");

        if ($request->can_players_change_between_teams == 1) {

            $player_rules->can_players_change_between_teams = 1;
            $player_rules->limit_date = Carbon::createFromFormat("d/m/Y", $request->limit_date)->format("Y-m-d");

            if ($request->transfer_player_with_handicap == 0) {
                $player_rules->transfer_player_with_handicap = 0;
                $player_rules->how_many_games_needs_to_play_to_recover_handicap = $request->how_many_games_needs_to_play_to_recover_handicap;
            } else {
                $player_rules->transfer_player_with_handicap = 1;
                $player_rules->how_many_games_needs_to_play_to_recover_handicap = 0;
            }

        } else {

            $player_rules->can_players_change_between_teams = 0;
            $player_rules->limit_date = null;
            $player_rules->transfer_player_with_handicap = 0;
            $player_rules->how_many_games_needs_to_play_to_recover_handicap = 0;

        }

        if ($player_rules->save()) {

            //Primero actualizamos a los jugadores que ya teniamos
            if ($request->current_jugador != null) {
                foreach ($request->current_jugador as $player_in_team_each_categories_id => $player_id) {

                    //Si player_id es diferente de null es por que hubo una edicion, si no es por que no se selecciono nada
                    if ($player_id != null) {
                        $player_in_team_each_categories = PlayerInTeamEachCategory::find($player_in_team_each_categories_id);
                        $player_in_team_each_categories->player_id = $player_id;
                        $player_in_team_each_categories->player_handy_cap_in_tournament = $request->current_jugador_handy_cap[$player_in_team_each_categories_id];
                        $player_in_team_each_categories->save();
                    } else {
                        PlayerInTeamEachCategory::where("id", $player_in_team_each_categories_id)->delete();
                    }
                }
            }

            //Ahora insertamos a los jugadores nuevos
            if ($request->nuevos_jugador != null) {
                foreach ($request->nuevos_jugador as $team_each_categories_id => $players) {
                    //dd($team_each_categories_id, $players);
                    foreach ($players as $key_player => $player_id) {
                        if ($player_id != "") {
                            //dd($team_each_categories_id, $key_player);
                            $player_in_team_each_categories = new PlayerInTeamEachCategory();
                            $player_in_team_each_categories->team_each_categories_id = $team_each_categories_id;
                            $player_in_team_each_categories->player_id = $player_id;
                            $player_in_team_each_categories->player_handy_cap_in_tournament = $request->nuevos_jugador_handy_cap[$team_each_categories_id][$key_player];
                            $player_in_team_each_categories->save();
                        }
                    }
                }
            }

            DB::commit();


            /*********** Actualización: ***********/
            //Verificaremos si involucran equipos DUMMY para insertar sus jugadores de forma automatica, esto servirá al momento de agregar el puntaje

            //Primero obtenemos todos los jugadores dummy que tenemos registrados en caso de que tengamos que asignarlos
            $players_dummys = Player::where("name", "DUMMY")->get();

            //Verificamos por categoria si contiene un equipo dummy para asignar jugadores
            foreach ($team_categories as $team_category) {

                $has_dummy_team = TeamEachCategory::where("team_category_id", $team_category->id)->where("team_id", 39)->first();

                //Si contiene un equipo dummy, asignamos los jugadores
                if ($has_dummy_team != null) {

                    $dummy_players_already_registered = PlayerInTeamEachCategory::where("team_each_categories_id", $has_dummy_team->id)->count();

                    for ($i = 0; $i < ($player_types->type_game - $dummy_players_already_registered); $i++) {
                        $player_in_team_each_categories = new PlayerInTeamEachCategory();
                        $player_in_team_each_categories->team_each_categories_id = $has_dummy_team->id;
                        $player_in_team_each_categories->player_id = $players_dummys[$i]->id;
                        $player_in_team_each_categories->player_handy_cap_in_tournament = 0;
                        $player_in_team_each_categories->save();
                    }

                }

            }


            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('jugadores.edit', base64_encode($id));
        } else {
            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";
        }

        return response()->json($response);


    }


}
