<?php

namespace MetodikaTI\Http\Controllers\Back\Liga;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\Back\PlayersType\CreatePlayersTypeRequest;
use MetodikaTI\League;
use MetodikaTI\PlayerType;

class PlayersTypeController extends Controller
{

    public function create($league_id)
    {

        $league = League::find(base64_decode($league_id));
        $player_type = PlayerType::where("league_id", base64_decode($league_id))->first();

        //Si ya existe este catalogo en BD, lo redireccionamos a edit
        if ($player_type != null) {
            return redirect(route('player_types.edit', $league_id));
        } else {
            return view('back.league.players_type.create', array("league" => $league));
        }

    }


    public function store(CreatePlayersTypeRequest $request, $id)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        $player_types = new PlayerType();

        //Tipo de jugadores
        $player_types->league_id = $id;
        $player_types->type_game = $request->type_game;

        //Jugadores por genero y/o condición
        $player_types->male_major_than_year = $request->male_major_than_year;
        $player_types->women_major_than_year = $request->women_major_than_year;

        $player_types->young_gender_male = ($request->young_gender_permited == "male" || $request->young_gender_permited == "ambos") ? 1 : 0;
        $player_types->young_gender_female = ($request->young_gender_permited == "female" || $request->young_gender_permited == "ambos") ? 1 : 0;
        $player_types->young_less_than_year = $request->young_less_than_year;

        $player_types->special_gender_male = ($request->special_gender_permited == "male") ? 1 : 0;
        $player_types->special_gender_female = ($request->special_gender_permited == "female") ? 1 : 0;
        $player_types->special_condition = $request->special_condition;

        //Formato y puntos de liga
        $player_types->can_young_and_special_play_any_gender = $request->can_young_and_special_play_any_gender;
        $player_types->limit_date_to_be_edited = Carbon::createFromFormat("d/m/Y", $request->limit_date_to_be_edited)->format("Y-m-d");

        $player_types->player_type_enable_male = (isset($request->player_type_enable_male)) ? 1 : 0;
        $player_types->player_type_enable_female = (isset($request->player_type_enable_female)) ? 1 : 0;
        $player_types->player_type_enable_mix = (isset($request->player_type_enable_mix)) ? 1 : 0;
        $player_types->player_type_enable_young = (isset($request->player_type_enable_young)) ? 1 : 0;
        $player_types->player_type_enable_special = (isset($request->player_type_enable_special)) ? 1 : 0;


        //Si esta seleccionado la opcion de mixta, entonces pueden existir reglas especiales para la integración de equipos
        if (isset($request->player_type_enable_mix)) {

            $player_types->there_are_special_rule_with_the_teams = $request->there_are_special_rule_with_the_teams;

            if ($request->there_are_special_rule_with_the_teams == 1) {
                $player_types->number_of_male = $request->number_of_male;
                $player_types->number_of_female = $request->number_of_female;
                $player_types->number_of_young = $request->number_of_young;
                $player_types->number_of_special = $request->number_of_special;
            } else {
                $player_types->number_of_male = 0;
                $player_types->number_of_female = 0;
                $player_types->number_of_young = 0;
                $player_types->number_of_special = 0;
            }

        } else {

            //Si no esta seleccionado la opcion de mixta, no hay reglas especiales para la integracion de equipos
            $player_types->there_are_special_rule_with_the_teams = 0;
            $player_types->number_of_male = 0;
            $player_types->number_of_female = 0;
            $player_types->number_of_young = 0;
            $player_types->number_of_special = 0;

        }


        //Puntaje
        $player_types->number_game_by_session = $request->number_game_by_session;
        $player_types->point_by_win_serie = $request->point_by_win_serie;
        $player_types->point_by_puntuality = $request->point_by_puntuality;
        $player_types->point_by_game = $request->point_by_game;


        if ($player_types->save()) {

            DB::commit();
            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('teams_create', base64_encode($id));

        } else {

            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }

        return response()->json($response);
    }


    public function edit($id)
    {

        $league = League::find(base64_decode($id));
        $player_type = PlayerType::where("league_id", base64_decode($id))->first();

        return view('back.league.players_type.edit', array("player_type" => $player_type, "league" => $league));
    }


    public function update(CreatePlayersTypeRequest $request, $id)
    {

        $response = array('status' => false);

        DB::beginTransaction();

        $player_types = PlayerType::where("league_id", $id)->first();

        //Tipo de jugadores
        $player_types->type_game = $request->type_game;

        //Jugadores por genero y/o condición
        $player_types->male_major_than_year = $request->male_major_than_year;
        $player_types->women_major_than_year = $request->women_major_than_year;

        $player_types->young_gender_male = ($request->young_gender_permited   == "male" || $request->young_gender_permited == "ambos") ? 1 : 0;
        $player_types->young_gender_female = ($request->young_gender_permited == "female" || $request->young_gender_permited == "ambos") ? 1 : 0;
        $player_types->young_less_than_year = $request->young_less_than_year;

        $player_types->special_gender_male = ($request->special_gender_permited == "male") ? 1 : 0;
        $player_types->special_gender_female = ($request->special_gender_permited == "female") ? 1 : 0;
        $player_types->special_condition = $request->special_condition;

        //Formato y puntos de liga
        $player_types->can_young_and_special_play_any_gender = $request->can_young_and_special_play_any_gender;
        $player_types->limit_date_to_be_edited = Carbon::createFromFormat("d/m/Y", $request->limit_date_to_be_edited)->format("Y-m-d");

        $player_types->player_type_enable_male = (isset($request->player_type_enable_male)) ? 1 : 0;
        $player_types->player_type_enable_female = (isset($request->player_type_enable_female)) ? 1 : 0;
        $player_types->player_type_enable_mix = (isset($request->player_type_enable_mix)) ? 1 : 0;
        $player_types->player_type_enable_young = (isset($request->player_type_enable_young)) ? 1 : 0;
        $player_types->player_type_enable_special = (isset($request->player_type_enable_special)) ? 1 : 0;
        
        // penalty_pins

        error_log("player_type_enable_mix => ".$request->player_type_enable_mix);

        //Si esta seleccionado la opcion de mixta, entonces pueden existir reglas especiales para la integración de equipos
        if (isset($request->player_type_enable_mix)) {

            $player_types->there_are_special_rule_with_the_teams = $request->there_are_special_rule_with_the_teams;

            if ($request->there_are_special_rule_with_the_teams == 1) {
                $player_types->number_of_male = $request->number_of_male;
                $player_types->number_of_female = $request->number_of_female;
                $player_types->number_of_young = $request->number_of_young;
                $player_types->number_of_special = $request->number_of_special;

                $player_types->penalty_pins = $request->penalty_pins;

            } else {
                $player_types->number_of_male = 0;
                $player_types->number_of_female = 0;
                $player_types->number_of_young = 0;
                $player_types->number_of_special = 0;
                $player_types->penalty_pins = 0;                
            }

        } else {

            //Si no esta seleccionado la opcion de mixta, no hay reglas especiales para la integracion de equipos
            $player_types->there_are_special_rule_with_the_teams = 0;
            $player_types->number_of_male = 0;
            $player_types->number_of_female = 0;
            $player_types->number_of_young = 0;
            $player_types->number_of_special = 0;
            $player_types->penalty_pins = 0;  

        }


        //Puntaje
        $player_types->number_game_by_session = $request->number_game_by_session;
        $player_types->point_by_win_serie = $request->point_by_win_serie;
        $player_types->point_by_puntuality = $request->point_by_puntuality;
        $player_types->point_by_game = $request->point_by_game;


        if ($player_types->save()) {

            DB::commit();
            $response['status'] = true;
            $response["message"] = "Los datos se han almacenado correctamente.";
            $response['url'] = route('player_types.edit', base64_encode($id));

        } else {

            DB::rollBack();
            $response['status'] = false;
            $response["message"] = "No hemos podido almacenar los datos, intentalo nuevamente.";

        }

        return response()->json($response);
    }


}
