<?php

namespace MetodikaTI\Http\Requests\Back\PlayersType;

use Illuminate\Foundation\Http\FormRequest;

class CreatePlayersTypeRequest extends FormRequest
{
    /**
     * Determine if the user is authorized to make this request.
     *
     * @return bool
     */
    public function authorize()
    {
        return true;
    }

    /**
     * Get the validation rules that apply to the request.
     *
     * @return array
     */
    public function rules()
    {

        $rules = array();

        $rules["type_game"] = "required";
        $rules["male_major_than_year"] = "required";
        $rules["women_major_than_year"] = "required";
        $rules["young_gender_permited"] = "required";
        $rules["young_less_than_year"] = "required";
        $rules["special_gender_permited"] = "required";
        $rules["special_condition"] = "required";
        $rules["can_young_and_special_play_any_gender"] = "required";
        $rules["limit_date_to_be_edited"] = "required";

        if ($this->player_type_enable_male == "" && $this->player_type_enable_female == "" && $this->player_type_enable_mix == "" && $this->player_type_enable_young == "" && $this->player_type_enable_special == "") {
            $rules["league_format"] = "required";
        }

        //Si se selecciono la opcion de mixa en formato de liga, verificamos si hay reglas especiales para validar
        if ($this->player_type_enable_mix != "") {
            if ($this->there_are_special_rule_with_the_teams == "1") {

                //Si se selecciono la opcion de mixta y se coloco como "si" a la pregunta de '¿Tienes alguna regla especial en la integración de los equipos?', verificamos que
                // la cantidad de jugadores por genero sea igual a lo seleccionado en '¿Cómo se juega?'
                $number_of_male = ($this->number_of_male != null) ? intval($this->number_of_male) : 0;
                $number_of_female = ($this->number_of_female != null) ? intval($this->number_of_female) : 0;
                $number_of_young = ($this->number_of_young != null) ? intval($this->number_of_young) : 0;
                $number_of_special = ($this->number_of_special != null) ? intval($this->number_of_special) : 0;

                $total_rule_players = $number_of_male + $number_of_female + $number_of_young + $number_of_special;
                
                if ($total_rule_players != intval($this->type_game)) {
                    $rules["invalid_total_players_vs_type_game"] = "required";
                }

            }
        }

        $rules["number_game_by_session"] = "required";
        $rules["point_by_win_serie"] = "required";
        $rules["point_by_puntuality"] = "required";
        $rules["point_by_game"] = "required";


        return $rules;
    }


    public function messages()
    {
        return [
            "type_game.required" => "Selecciona como se jugará la liga.",
            "male_major_than_year.required" => "Ingresa la edad de la categoría hombres permitidos.",
            "women_major_than_year.required" => "Ingresa la edad de la categoría mujeres permitidos.",
            "young_gender_permited.required" => "Selecciona el genero permitido para la categoría juveniles",
            "young_less_than_year.required" => "Ingresa la edad de la categoría juveniles permitidos",
            "special_gender_permited.required" => "Selecciona el genero permitido para la categoría especiales.",
            "special_condition.required" => "Ingresa la condición para la categoría especiales.",

            "can_young_and_special_play_any_gender.required" => "Selecciona si es permitido que los jugadores juveniles y especiales puedan jugar en cualquier genero.",
            "limit_date_to_be_edited.required" => "El campo 'Fecha final para ser editable' es requerido.",

            "league_format.required" => "El campo 'Formato de la liga' es requerido.",

            "number_of_male.required" => "Ingresa la cantidad de jugadores para la categoría hombres.",
            "number_of_female.required" => "Ingresa la cantidad de jugadores para la categoría mujeres.",
            "number_of_young.required" => "Ingresa la cantidad de jugadores para la categoría juveniles.",
            "number_of_special.required" => "Ingresa la cantidad de jugadores para la categoría especiales.",

            "invalid_total_players_vs_type_game.required" => "La cantidad total de jugadores por genero debe de coincidir por como se jugará la liga.",

            "number_game_by_session.required" => "El campo 'Juegos por sesión' es requerido.",
            "point_by_win_serie.required" => "El campo 'Puntos por serie ganada' es requerido.",
            "point_by_puntuality.required" => "El campo 'Puntos por puntialidad' es requerido.",
            "point_by_game.required" => "El campo 'Punto por juego' es requerido.",
        ];
    }

}
