<?php namespace MetodikaTI\Library;

use Auth;
use Conekta\Conekta;
use Faker\Test\Provider\Collection;
use GuzzleHttp\Psr7\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Mail;
use MetodikaTI\CategoryRule;
use MetodikaTI\DegreeFrame;
use MetodikaTI\DegreeFrameConfiguration;
use MetodikaTI\DiplomaFrame;
use MetodikaTI\DiplomaFrameConfiguration;
use MetodikaTI\Extra;
use MetodikaTI\Glass;
use MetodikaTI\League;
use MetodikaTI\Order;
use MetodikaTI\Player;
use MetodikaTI\PortaretratoFrame;
use MetodikaTI\PortaretratoFrameConfiguration;
use MetodikaTI\Reservation;
use MetodikaTI\Rol;
use MetodikaTI\RolTournament;
use MetodikaTI\RolTournamentSession;
use MetodikaTI\RolTournamentSessionPoint;
use MetodikaTI\RolTournamentSessionPointsTotal;
use MetodikaTI\SystemModule;
use MetodikaTI\TeamAndCategorySpecification;
use MetodikaTI\University;
use MetodikaTI\User;
use MetodikaTI\HandicapEachCategory;
use MetodikaTI\TeamsAddedInCurrentTournament;
use MetodikaTI\TeamCategory;

use Permission;
use Session;

use function _\each;

class Pastora
{


    public static function in_relation($object, $field_id, $field)
    {
        if (!is_null($object)) {
            if (!is_null($object->$field_id)) {
                if (!is_null($object->$field_id->$field)) {
                    return true;
                }
            }
        }
        return false;
    }

    /**
     * Metodo que devuelve verdadero si una propiedad se encuentra en un arreglo
     *
     * @return Boolean
     */
    public static function in_object($value, $object)
    {
        if (is_object($object)) {
            foreach ($object as $key => $item) {
                if ($value == $key) return true;
            }
        }
        return false;
    }

    /**
     * Metodo que devuelve los permisos que tiene el usuario
     *
     * @return JSON
     */
    public static function userProfile()
    {

        return User::find(Auth::user()->id)->userProfile->permits;
    }

    /**
     * Metodo que convierte un Json a Array
     *
     * @return Array
     */
    public static function jsonToArray($json)
    {
        $json = (array)json_decode($json);
        $response = [];
        foreach ($json as $key => $value) {
            $response[(int)$key] = (int)$value;
        }
        return $response;
    }

    /**
     * This method convert the array to object
     *
     * @param $array array to convert to object
     * @return mixed return the array on object
     */
    public static function arrayToObject($array)
    {
        return json_decode(json_encode($array));
    }


    /**
     *
     *
     */
    public function strongMeter($password)
    {
        $rank = 0;
        $rank += (strlen(trim($password)) > 7) ? 5 : 0;
        $rank += (strlen(trim($password)) > 11 && strlen(trim($password)) < 16) ? 10 : 0;
        $rank += (preg_match("/([A-Z])/", $password)) ? 10 : 0;
        $rank += (preg_match("/([0-9])/", $password)) ? 10 : 0;
    }

    /**
     * Metodo que construye el arbol de modulos del sistema
     *
     * @var $parent integer. ID del papa del módulo
     *
     * @return array
     */
    public static function moduleTree($parent = 0)
    {

        $response = [];

        foreach (SystemModule::where('parent', '=', $parent)->orderBy('order')->get() as $module) {

            $response[] = [
                'id' => $module->id,
                'name' => $module->name,
                'url' => $module->url,
                'icon' => $module->icon,
                'parent_as_child' => $module->parent_as_child,
                'child' => self::moduleTree($module->id)
            ];

        }

        return $response;

    }


    /**
     *
     *
     */
    public static function cleanPhone($phone)
    {
        $mapper = [
            '/',
            '(',
            ')',
            '-',
            ' '
        ];

        foreach ($mapper as $key => $value) {

            $phone = str_replace($value, "", $phone);

        }

        return $phone;
    }

    public static function randomPassword()
    {
        $alphabet = 'abcdefghijkmnpqrstuvwxyz1234567890';
        $pass = array(); //remember to declare $pass as an array
        $alphaLength = strlen($alphabet) - 1; //put the length -1 in cache
        for ($i = 0; $i < 3; $i++) {
            $n = rand(0, $alphaLength);
            $pass[] = $alphabet[$n];
        }

        return "beonline_" . implode($pass); //turn the array into a string
    }

    public function getRandomWord($len = 5)
    {
        $word = array_merge(range('0', '9'), range('A', 'Z'));
        shuffle($word);
        return substr(implode($word), 0, $len);
    }


    //PAYMENTS CONEKTA
    public static function create_customer($name, $email, $phone, $token_id)
    {

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";


        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");
        \Conekta\Conekta::setLocale('es');

        try {

            $customer = \Conekta\Customer::create(
                array(
                    "name" => $name,
                    "email" => $email,
                    "phone" => $phone,
                )//customer
            );

            $response["error"] = false;
            $response["message"] = "";
            $response["customer"] = $customer;

        } catch (\Conekta\ProccessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function add_card($customer_id, $token_id)
    {

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";


        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");
        \Conekta\Conekta::setLocale('es');

        try {

            $customer = \Conekta\Customer::find($customer_id);
            $source = $customer->createPaymentSource(array(
                'token_id' => $token_id,
                'type' => 'card'
            ));

            $response["error"] = false;
            $response["message"] = "";
            $response["customer"] = $customer;

        } catch (\Conekta\ProccessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function delete_card($customer_id, $card_id)
    {

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";


        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");
        \Conekta\Conekta::setLocale('es');

        try {

            $customer = \Conekta\Customer::find($customer_id);
            for ($i = 0; $i < count($customer->payment_sources); $i++) {
                if ($customer->payment_sources[$i]["id"] == $card_id) {
                    $source = $customer->payment_sources[$i]->delete();
                    $i = count($customer->payment_sources) + 1;
                }
            }

            $response["error"] = false;
            $response["message"] = "";
            $response["customer"] = $source;

        } catch (\Conekta\ProccessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function get_user_from_conekta($customer_id)
    {

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";


        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");
        \Conekta\Conekta::setLocale('es');

        try {

            $customer = \Conekta\Customer::find($customer_id);

            $response["error"] = false;
            $response["message"] = "";
            $response["customer"] = $customer;

        } catch (\Conekta\ProccessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function make_payment($concepto, $cantidad_cobrar, $app_user, $card_id, $address_to_send)
    {

        $cantidad_cobrar = ($cantidad_cobrar * 100);
        $cantidad_cobrar = explode(".", $cantidad_cobrar);

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";

        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");

        try {
            $order = \Conekta\Order::create(
                array(
                    "line_items" => array(
                        array(
                            "name" => $concepto,
                            "unit_price" => $cantidad_cobrar[0],
                            "quantity" => 1
                        )
                    ),
                    "currency" => "MXN",
                    "customer_info" => array(
                        "customer_id" => $app_user->customer_id
                    ), //customer_info
                    "charges" => array(
                        array(
                            "payment_method" => array(
                                "type" => "card",
                                "payment_source_id" => $card_id,
                            )
                        )
                    ),
                    "shipping_lines" => array(
                        array(
                            "amount" => 0,
                            "carrier" => "Fedex",
                            "method" => "Terrestre",
                            "tracking_number" => "----",
                            "object" => "shipping_line",
                            "id" => "",
                            "parent_id" => ""
                        )
                    ),
                    "shipping_contact" => array(
                        "id" => "",
                        "object" => "shipping_contact",
                        "created_at" => "",
                        "parent_id" => "",
                        "receiver" => $app_user->name,
                        "phone" => $app_user->phone,
                        "between_streets" => $address_to_send->address,
                        "address" => array(
                            "street1" => $address_to_send->address,
                            "city" => $address_to_send->city,
                            "state" => $address_to_send->state,
                            "postal_code" => $address_to_send->cp,
                            "country" => "mx",
                            "residential" => true,
                            "object" => "shipping_address"
                        )
                    )
                )
            );


            $response["error"] = false;
            $response["message"] = "";
            $response["order"] = $order;

        } catch (\Conekta\ProcessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function make_payment_oxxo($concepto, $cantidad_cobrar, $app_user, $address_to_send)
    {

        $cantidad_cobrar = ($cantidad_cobrar * 100);
        $cantidad_cobrar = explode(".", $cantidad_cobrar);

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";

        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");

        try {
            $order = \Conekta\Order::create(
                array(
                    "line_items" => array(
                        array(
                            "name" => $concepto,
                            "unit_price" => $cantidad_cobrar[0],
                            "quantity" => 1
                        )
                    ),
                    "currency" => "MXN",
                    "customer_info" => array(
                        "customer_id" => $app_user->customer_id
                    ), //customer_info
                    "charges" => array(
                        array(
                            "payment_method" => array(
                                "type" => "oxxo_cash"
                            )
                        )
                    ),
                    "shipping_lines" => array(
                        array(
                            "amount" => 0,
                            "carrier" => "Fedex",
                            "method" => "Terrestre",
                            "tracking_number" => "----",
                            "object" => "shipping_line",
                            "id" => "",
                            "parent_id" => ""
                        )
                    ),
                    "shipping_contact" => array(
                        "id" => "",
                        "object" => "shipping_contact",
                        "created_at" => "",
                        "parent_id" => "",
                        "receiver" => $app_user->name,
                        "phone" => $app_user->phone,
                        "between_streets" => $address_to_send->address,
                        "address" => array(
                            "street1" => $address_to_send->address,
                            "city" => $address_to_send->city,
                            "state" => $address_to_send->state,
                            "postal_code" => $address_to_send->cp,
                            "country" => "mx",
                            "residential" => true,
                            "object" => "shipping_address"
                        )
                    )
                )
            );


            $response["error"] = false;
            $response["message"] = "";
            $response["order"] = $order;

        } catch (\Conekta\ProcessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function make_payment_spei($concepto, $cantidad_cobrar, $app_user, $address_to_send)
    {

        $cantidad_cobrar = ($cantidad_cobrar * 100);
        $cantidad_cobrar = explode(".", $cantidad_cobrar);

        $response = array();
        $response["error"] = false;
        $response["message"] = "";
        $response["customer"] = "";

        \Conekta\Conekta::setApiKey(env("CONEKTA_PRIVATE_KEY"));
        \Conekta\Conekta::setApiVersion("2.0.0");

        try {
            $order = \Conekta\Order::create(
                array(
                    "line_items" => array(
                        array(
                            "name" => $concepto,
                            "unit_price" => $cantidad_cobrar[0],
                            "quantity" => 1
                        )
                    ),
                    "currency" => "MXN",
                    "customer_info" => array(
                        "customer_id" => $app_user->customer_id
                    ), //customer_info
                    "charges" => array(
                        array(
                            "payment_method" => array(
                                "type" => "spei"
                            )
                        )
                    ),
                    "shipping_lines" => array(
                        array(
                            "amount" => 0,
                            "carrier" => "Fedex",
                            "method" => "Terrestre",
                            "tracking_number" => "----",
                            "object" => "shipping_line",
                            "id" => "",
                            "parent_id" => ""
                        )
                    ),
                    "shipping_contact" => array(
                        "id" => "",
                        "object" => "shipping_contact",
                        "created_at" => "",
                        "parent_id" => "",
                        "receiver" => $app_user->name,
                        "phone" => $app_user->phone,
                        "between_streets" => $address_to_send->address,
                        "address" => array(
                            "street1" => $address_to_send->address,
                            "city" => $address_to_send->city,
                            "state" => $address_to_send->state,
                            "postal_code" => $address_to_send->cp,
                            "country" => "mx",
                            "residential" => true,
                            "object" => "shipping_address"
                        )
                    )
                )
            );


            $response["error"] = false;
            $response["message"] = "";
            $response["order"] = $order;

        } catch (\Conekta\ProcessingError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\ParameterValidationError $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        } catch (\Conekta\Handler $error) {
            $response["error"] = true;
            $response["message"] = $error->message;
        }

        return $response;
    }


    public static function make_sku()
    {

        $reservation = null;
        $sku = "";

        do {

            $sku = chr(rand(65, 90)) . chr(rand(65, 90)) . chr(rand(65, 90)) . "-" . rand(100, 999);
            $reservation = Order::where("order_id", $sku)->count();

        } while ($reservation >= 1);

        return $sku;
    }


    public static function sendPushNotification($notification)
    {
        // error_log( "notification =>". print_r($notification["RESPONSE"]["TEXTO_NOTIFICACION"], true) );
        
        $API_ACCESS_KEY = env('SERVER_KEY_FIREBASE');

        $singleID = $notification["RESPONSE"]["UUID"];//$notification->RESPONSE["UUID"];

        $instructions = array();
        $instructions["TEXTO_NOTIFICACION"] = $notification["RESPONSE"]["TEXTO_NOTIFICACION"];//$notification->RESPONSE["TEXTO_NOTIFICACION"];

        $fcmMsg = array(
            'body'  => $notification["RESPONSE"]["TEXTO_NOTIFICACION"],//$notification->RESPONSE["TEXTO_NOTIFICACION"],
            'title' => $notification["RESPONSE"]["TITLE"],//$notification->RESPONSE["TITLE"],
            'subtitle' => "BeOnline",
            'sound' => "default",
        );

        $fcmFields = array(
            'to' => $singleID,
            'priority' => 'high',
            'data' => $instructions,
            'notification' => $fcmMsg
        );

        $headers = array(
            'Authorization: key=' . $API_ACCESS_KEY,
            'Content-Type: application/json'
        );

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');
        curl_setopt($ch, CURLOPT_POST, true);
        curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($fcmFields));
        $result = curl_exec($ch);
        curl_close($ch);

        $result = json_decode($result);

        return $result;
    }


    public static function autocomplete_array($obj, $colum)
    {

        $obj = $obj->groupBy($colum)->get()->toArray();
        $tags = "";
        foreach ($obj as $item) {
            $tags .= $item[$colum] . ",";
        }

        return $tags;
    }


    public static function format_product_from_carrito($product)
    {

        $product_formated = array();
        $price = array();
        $price["maria_luisa_precio_original"] = 0;
        $price["maria_luisa_descuento"] = 0;
        $price["maria_luisa_precio_final"] = 0;

        $price["vidrio_precio_original"] = 0;
        $price["vidrio_descuento"] = 0;
        $price["vidrio_precio_final"] = 0;

        $price["extra_precio_original"] = 0;
        $price["extra_descuento"] = 0;
        $price["extra_precio_final"] = 0;

        $price["frame_precio_original"] = 0;
        $price["frame_descuento"] = 0;
        $price["frame_precio_final"] = 0;

        $price["unique_price"] = 0;

        $price["gran_total"] = 0;

        if ($product["frame_type"] == "titulo") {

            $degree_frame_configurations = DegreeFrameConfiguration::where("id", intval($product["configuration_id"]))->first();
            $degree_frame = DegreeFrame::where("id", intval($product["frame_id"]))->first();
            $glass = Glass::where("id", intval($product["glass_id"]))->first();

            if ($product["extra_id"] != "") {
                $extra = Extra::find(intval($product["extra_id"]))->first();
            } else {
                $extra = null;
            }

            $product_formated["rowid"] = $product["rowid"];
            $product_formated["frame"] = $degree_frame->name;
            $product_formated["tipo"] = "Titulo";
            $product_formated["position"] = ($product["position"] == "horizontal") ? "Horizontal" : "Vertical";
            $product_formated["quantity"] = intval($product["quantity"]);
            $product_formated["color"] = $degree_frame_configurations->get_color["name"];
            $product_formated["height"] = $degree_frame->height;
            $product_formated["width"] = $degree_frame->width;
            $product_formated["height_finish"] = $degree_frame->height_finish;
            $product_formated["width_finish"] = $degree_frame->width_finish;


            if ($degree_frame_configurations->image_url_with_maria_luisa == null) {
                $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_color;
                $product_formated["tipo_maria_luisa"] = "Sin María Luisa";
            } else {
                $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_maria_luisa;
                $product_formated["tipo_maria_luisa"] = $degree_frame_configurations->get_maria_luisa["name"];
            }

            $product_formated["tipo_vidrio"] = $glass->name;


            if ($extra != null) {
                $product_formated["extra"] = $extra->name;
            } else {
                $product_formated["extra"] = "";
            }


            //Calculamos el precio
            //Precio Maria Luisa
            if ($degree_frame_configurations->maria_luisa_id != null) {
                $price["maria_luisa_precio_original"] = $degree_frame_configurations->get_maria_luisa["price"];
                $price["maria_luisa_descuento"] = $degree_frame_configurations->get_maria_luisa["descount"];
                $price["maria_luisa_precio_final"] = $degree_frame_configurations->get_maria_luisa["price"] - $degree_frame_configurations->get_maria_luisa["descount"];
            }

            //Precio vidrio
            if ($glass != null) {
                $price["vidrio_precio_original"] = $glass->price;
                $price["vidrio_descuento"] = $glass->descount;
                $price["vidrio_precio_final"] = $glass->price - $glass->descount;
            }

            //Precio extra
            if ($extra != null) {
                $price["extra_precio_original"] = $extra->price;
                $price["extra_descuento"] = $extra->descount;
                $price["extra_precio_final"] = $extra->price - $glass->descount;
            }

            //Precio extra
            if ($degree_frame != null) {
                $price["frame_precio_original"] = $degree_frame->price;
                $price["frame_descuento"] = $degree_frame->discount;
                $price["frame_precio_final"] = $degree_frame->price - $degree_frame->discount;
            }

            $price["unique_price"] = $price["maria_luisa_precio_final"] + $price["vidrio_precio_final"] + $price["extra_precio_final"] + $price["frame_precio_final"];

            $price["gran_total"] = $price["unique_price"] * $product["quantity"];

        } else if ($product["frame_type"] == "diploma") {

            $degree_frame_configurations = DiplomaFrameConfiguration::where("id", intval($product["configuration_id"]))->first();
            $degree_frame = DiplomaFrame::where("id", intval($product["frame_id"]))->first();
            $glass = Glass::where("id", intval($product["glass_id"]))->first();

            if ($product["extra_id"] != "") {
                $extra = Extra::find(intval($product["extra_id"]))->first();
            } else {
                $extra = null;
            }

            $product_formated["rowid"] = $product["rowid"];
            $product_formated["frame"] = $degree_frame->name;
            $product_formated["tipo"] = "Diploma";
            $product_formated["position"] = ($product["position"] == "horizontal") ? "Horizontal" : "Vertical";
            $product_formated["quantity"] = intval($product["quantity"]);
            $product_formated["color"] = $degree_frame_configurations->get_color["name"];
            $product_formated["height"] = $degree_frame->height;
            $product_formated["width"] = $degree_frame->width;


            if ($product["position"] == "horizontal") {
                if ($degree_frame_configurations->image_url_with_maria_luisa != null) {
                    $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_maria_luisa;
                    $product_formated["tipo_maria_luisa"] = $degree_frame_configurations->get_maria_luisa["name"];
                } else {
                    $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_color;
                    $product_formated["tipo_maria_luisa"] = "Sin María Luisa";
                }
            } else {
                if ($degree_frame_configurations->image_url_with_maria_luisa_vertical != null) {
                    $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_maria_luisa_vertical;
                    $product_formated["tipo_maria_luisa"] = $degree_frame_configurations->get_maria_luisa["name"];
                } else {
                    $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_color_vertical;
                    $product_formated["tipo_maria_luisa"] = "Sin María Luisa";
                }
            }

            $product_formated["tipo_vidrio"] = $glass->name;

            if ($extra != null) {
                $product_formated["extra"] = $extra->name;
            } else {
                $product_formated["extra"] = "";
            }

            //Calculamos el precio
            //Precio Maria Luisa
            if ($degree_frame_configurations->maria_luisa_id != null) {
                $price["maria_luisa_precio_original"] = $degree_frame_configurations->get_maria_luisa["price"];
                $price["maria_luisa_descuento"] = $degree_frame_configurations->get_maria_luisa["descount"];
                $price["maria_luisa_precio_final"] = $degree_frame_configurations->get_maria_luisa["price"] - $degree_frame_configurations->get_maria_luisa["descount"];
            }

            //Precio vidrio
            if ($glass != null) {
                $price["vidrio_precio_original"] = $glass->price;
                $price["vidrio_descuento"] = $glass->descount;
                $price["vidrio_precio_final"] = $glass->price - $glass->descount;
            }

            //Precio extra
            if ($extra != null) {
                $price["extra_precio_original"] = $extra->price;
                $price["extra_descuento"] = $extra->descount;
                $price["extra_precio_final"] = $extra->price - $glass->descount;
            }

            //Precio extra
            if ($degree_frame != null) {
                $price["frame_precio_original"] = $degree_frame->price;
                $price["frame_descuento"] = $degree_frame->discount;
                $price["frame_precio_final"] = $degree_frame->price - $degree_frame->discount;
            }

            $price["unique_price"] = $price["maria_luisa_precio_final"] + $price["vidrio_precio_final"] + $price["extra_precio_final"] + $price["frame_precio_final"];

            $price["gran_total"] = $price["unique_price"] * $product["quantity"];

        } else if ($product["frame_type"] == "portaretratos") {

            $degree_frame_configurations = PortaretratoFrameConfiguration::where("id", intval($product["configuration_id"]))->first();
            $degree_frame = PortaretratoFrame::where("id", intval($product["frame_id"]))->first();
            //$glass = Glass::where("id", intval($product["glass_id"]))->first();

            $extra = null;

            $product_formated["rowid"] = $product["rowid"];
            $product_formated["frame"] = $degree_frame->name;
            $product_formated["tipo"] = "Portaretrato";
            $product_formated["position"] = ($product["position"] == "horizontal") ? "Horizontal" : "Vertical";
            $product_formated["quantity"] = intval($product["quantity"]);
            $product_formated["color"] = "";
            $product_formated["height"] = $degree_frame->height;
            $product_formated["width"] = $degree_frame->width;


            if ($degree_frame_configurations->image_url_with_maria_luisa == null) {
                $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_color;
                $product_formated["tipo_maria_luisa"] = "Sin María Luisa";
            } else {
                $product_formated["image_frame"] = $degree_frame_configurations->image_url_with_maria_luisa;
                $product_formated["tipo_maria_luisa"] = $degree_frame_configurations->get_maria_luisa["name"];
            }

            $product_formated["tipo_vidrio"] = "";
            $product_formated["extra"] = "";


            //Calculamos el precio
            //Precio Maria Luisa
            if ($degree_frame_configurations->maria_luisa_id != 0) {
                $price["maria_luisa_precio_original"] = $degree_frame_configurations->get_maria_luisa["price"];
                $price["maria_luisa_descuento"] = $degree_frame_configurations->get_maria_luisa["descount"];
                $price["maria_luisa_precio_final"] = $degree_frame_configurations->get_maria_luisa["price"] - $degree_frame_configurations->get_maria_luisa["descount"];
            }

            //Precio vidrio **No lleva

            //Precio extra **No lleva

            //Precio marco
            if ($degree_frame != null) {
                $price["frame_precio_original"] = $degree_frame->price;
                $price["frame_descuento"] = $degree_frame->discount;
                $price["frame_precio_final"] = $degree_frame->price - $degree_frame->discount;
            }

            $price["unique_price"] = $price["maria_luisa_precio_final"] + $price["vidrio_precio_final"] + $price["extra_precio_final"] + $price["frame_precio_final"];

            $price["gran_total"] = $price["unique_price"] * $product["quantity"];
        }

        $product_formated["prices"] = $price;

        return $product_formated;
    }


    public static function shipping_cost($products)
    {

        $quantity_frames = 0;
        $total_packages = 0;
        foreach ($products as $product) {
            $quantity_frames += $product["quantity"];
        }

        //Cada envio es por 3 marcos
        $total_packages = intval($quantity_frames / 3);

        //En caso de que la cantidad de marcos no sea multiplo de 3
        $total_packages += (($quantity_frames % 3) > 0) ? 1 : 0;

        return ($total_packages * 100);
    }


    public static function reload_players($league_id)
    {
        $team_and_category_specifications = TeamAndCategorySpecification::where("league_id", $league_id)->first();
//        $category_rules = CategoryRule::where("league_id", $league_id)->first();
//
//        if ($team_and_category_specifications != null && $category_rules != null) {
//
//            //Ya se ingreso la configuracion inicial de los equipos, vamos a crear registros vacios para cada jugador
//            foreach ($team_and_category_specifications->get_categories as $team_categories) {
//
//                foreach ($team_categories->get_team as $team) {
//
//
//                    //Para cada equipo verificamos si vamos agregar o eliminar jugadores
//                    $total_players = Player::where("team_category_id", $team->id)->count();
//
//                    if ($category_rules->maximum_players_by_team > $total_players) {          //si es mayor, vamos a agregar
//
//                        $total_agregar = $category_rules->maximum_players_by_team - $total_players;
//                        //Insertamos los jugadores vacios para cada equipo
//                        for ($i = 0; $i < $total_agregar; $i++) {
//                            $players = new Player();
//                            $players->team_category_id = $team->id;
//                            $players->gender = "male";
//                            $players->birthday = "0000-00-00";
//                            $players->handycap = 0;
//                            $players->save();
//                        }
//
//                    } else if ($category_rules->maximum_players_by_team < $total_players) {   //si es menor, vamos a eliminar
//
//                        $total_eliminar = $total_players - $category_rules->maximum_players_by_team;
//                        Player::where("team_category_id", $team->id)->orderBy("id", "DESC")->take($total_eliminar)->delete();
//
//                    }
//
//
//                }
//
//            }
//
//        }
//

    }


    public static function slugify($text)
    {
        // Strip html tags
        $text = strip_tags($text);
        // Replace non letter or digits by -
        $text = preg_replace('~[^\pL\d]+~u', '-', $text);
        // Transliterate
        setlocale(LC_ALL, 'en_US.utf8');
        $text = iconv('utf-8', 'us-ascii//TRANSLIT', $text);
        // Remove unwanted characters
        $text = preg_replace('~[^-\w]+~', '', $text);
        // Trim
        $text = trim($text, '-');
        // Remove duplicate -
        $text = preg_replace('~-+~', '-', $text);
        // Lowercase
        $text = strtolower($text);
        // Check if it is empty
        if (empty($text)) {
            return 'n-a';
        }
        // Return result
        return $text;
    }
    
    private function recalcular_handycap( $league_id, $category_id, $week, $team_id, $player_id, $handicap_each_categories,$player_rules, $handicaps  ){
        
        $rol = Rol::where("league_id", $league_id)->first();

        //Un jugador nuevo lo determinaremos si el handycap inicial es null.
        // $is_player_new = (!is_null($player_in_team_each_category->player_handy_cap_in_tournament)) ? false : true;        

        $cumple_primera_condicion = false;
        $cumple_segunda_condicion = false;


        $semana_uno_equipo_nuevo = false;

        $teams_added_team_new = TeamsAddedInCurrentTournament::where("league_id", $league_id)->where("type",1)->where("team_id",$team_id)->first(); 

        if( $teams_added_team_new ){
        
            if( ($teams_added_team_new->week-1) == $week ){   
                $semana_uno_equipo_nuevo = true;               
            }
            
            //SE SE CONSULTA UNA SEMANAS ATRAS DE LA SEMANA QUE INGRESO SE REGRESA 0                    
            if( ($teams_added_team_new->week - 1) > $week ){
                return 0;
            }

            //VERIFICAMOS QUE EQUIPO ES EL NUEVO EN LE TORNEO PARA CAMBIAR SU SEMANA INICIAL
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ($teams_added_team_new->week-1) )->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();                              
            

        }else{
            //revisa la primer semana de los equipos normales
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();                  
        }


        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournaments->id)->where("team_id", $team_id)->pluck("id")->toArray();
        $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_id)->count();
        
        error_log( print_r( $rol_tournament_sessions, true ) );

        if ($rol_tournament_session_points >= $handicaps->news_players_how_many_games_play_on_first_week) {
            $cumple_primera_condicion = true;
        }

        if( $semana_uno_equipo_nuevo ){            
            //Sacamos cuantos juegos realizo el jugador desde la primer semana el equipo nuevo
            $semanas_torneo_equipo_nuevo = $this->semanas_torneo_equipo_nuevo( $handicaps->weeks_to_complete_games, $week, $teams_added_team_new->week );                    
            // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ">", ($teams_added_team_new->week-1))->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();            
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereIn("iterator_week", $semanas_torneo_equipo_nuevo)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                    
        }else{
            
            $semanas_torneo = [];            
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("type", "!=", "ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<",$week)->where("suspended", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->whereRaw("(first_team_suspended = " . 0 . " OR second_team_suspended = " . 0 . ")")->pluck("id")->toArray();            
        
        } 


        foreach ($rol_tournaments as $key => $value) {
            
            $arr = array_slice($rol_tournaments, $key, $handicaps->weeks_to_complete_games);
            $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $arr)->where("team_id", $team_id)->pluck("id")->toArray();
            $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_id)->count();                     
        
            if ($rol_tournament_session_points >= $handicaps->how_many_games_to_get_new_handicap) {            
                $cumple_segunda_condicion = true;    
                break;        
            }
        }


        //Primero revisamos si es la primera semana
        if ($week == 0 || $semana_uno_equipo_nuevo) {
           
            //Cuando es la semana 1, solamente verificaremos si colocamos el handycap inicial o en 0 dependiendo de lo que se configure en "Configuración de jugadores", campo " ¿Jugadores inician con handicap en "0"?"
            if ($player_rules->players_start_with_zero_handicap == 1) {
                return 0; //Jugadores inician con handycap en 0
            } else {                

                //Si es un jugador nuevo, verificamos si genero un handycap, si no genero le regresamos 0 ya que ira generando su handycap conforme se le vaya asignando puntaje
                if ($is_player_new) {

                    if ($cumple_primera_condicion) {

                        //Para la semana 1 (0) al menos para jugadores nuevos si generaremos su handycap de forma dinamica. Para jugadores viejos no generaremos su handycap.
                        //Para generar el nuevo handycap tenemos una formula que desglosaremos
                        //Suma de todos los Pinos tirados anteriores a la semana actual/cantidad de juegos jugados
                        
                        if( $semana_uno_equipo_nuevo ){                            
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", ($teams_added_team_new->week-1))->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        }else{                    
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        }                                            

                        $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();

                        $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->avg("points");
                        $base = $handicap_each_categories->base_normal;
                        
                        return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories);

                    } else {
                        return 0;
                    }

                } else {
                    if( $player->id == 75 ){
                        // error_log("ROL TORNAMET OK");
                        // error_log( $player_in_team_each_category );
                    }
                    //Jugadores inician con handycap inicial indicado en "Jugadores", campo "HAndyCap inicial"
                    return $player_in_team_each_category->player_handy_cap_in_tournament;
                }

            }

        } else {            
                        
            
            /**** GENERAR NUEVO HANDYCAP ****/
            //Revisamos si cumple al menos una condición, si la cumple generamos su nuevo handycap.
            //Si no la cumple pierde su handycap y le colocamos el handycap en 0
            if ($cumple_primera_condicion || $cumple_segunda_condicion) {               
                
                
                //SE AÑADE ES TABLA PARA EL CONTROL DE EQUIPOS AÑADIDOS EN UN JUGO EN CURSOR
                //EN ESTE CASO ES LA SUSTITUCION DEL DUMMY
                //CUENTA A PARTIR DE LA SEMANA EN QUE SE AÑADIO EL EQUIPO Y SUS JUGADORES
                

                // $teams_added_in_current_tournament =  TeamsAddedInCurrentTournament::where("team_id", $team_id)->where("league_id", $league_id)->where("week", $week)->first();
                       
                
                //Para generar el nuevo handycap tenemos una formula que desglosaremos
                //Suma de todos los Pinos tirados anteriores a la semana actual/cantidad de juegos jugados   
                
                //JUEGOS SUSPENDIDOS INDIVIDUALES                
                $teams_added_partido = TeamsAddedInCurrentTournament::where("team_id", $team_id)->where("type",2)->where("league_id", $league_id)->where("week", "<" ,$week)->get();

                //REVISA SI HAY NUEVO EQUIPO SOLO DEBE SER UNO POR AHORA
                $teams_added_team_new = TeamsAddedInCurrentTournament::where("league_id", $league_id)->where("type",1)->where("team_id",$team_id)->first();                
                
                //SE VALIDA SI HAY JUEGOS SUSPENDIDOS
                //SI HAY JUEGO SUSPENDIDOS SE REALIZAN OTRAS CONSULTAS YA QUE HAY QUE OMITIR ESA SEMANA EN EL HANDICAP
                
                // //SE REVISA SI LA SEMANA ANTERIOR ES UNA RONDA DE POSICIONES
                // $rol_tournaments_prev = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", $week-1)->first();
                                
                
                if( count( $teams_added_partido ) > 0 ){                                           
                    
                    // error_log( print_r($teams_added_team_new, true) );
                    // error_log("Equipoo new ==> ". print_r($teams_added_partido, true));
        
                    //REVISA SI HAY UN NUEVO EQUIPO
                    if( $teams_added_team_new ){  
                                                                                                
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        
                    }else{
                                                
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                        
                    }
                   
                }else{    
                                                          
                    if( $teams_added_team_new ){

                       
                         
                        //SI LA SEMANA REGISTRADA ES MENOR A LA ACTUAL COMIENZA A CONTAR A PARTIR DE AHI                                                
                        if( ($teams_added_team_new->week-1) < $week ){                                                        
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->whereBetween("iterator_week",[ ($teams_added_team_new->week-1), ($week-1)])->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                                                                      

                        }else{                            
                            // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                       
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->whereBetween("iterator_week",[ ($teams_added_team_new->week - 1), ($week-1)])->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                                                                
                        }                                                

                    }else{                                                                                                                      
                        
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                                                     
                        
                    }                                         
                    
                } 
                                                                                       
                $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();
                $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_id)->avg("points");

                $base = $handicap_each_categories->base_normal;                               
                
                
                // VALIDA EN EQUIPOS AGREGADOS EN TORNEO EN CURSO QUE SI EXISTE UN JUGADOR Y NO HA TENIDO NUNGUN SCORE
                // TENDRA UN HANDUCAP DE 0
                if( $teams_added_team_new ){

                    $r_t_s_p = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_id)->count();                    

                    if( $r_t_s_p == 0 ){                        
                        return 0;
                    }

                }   
                
                $player = Player::find( $player_id );
                
                return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories, $team_id);
                

            } else {
                return 0;
            }

        }
        
        // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("type", "!=", "ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", ">",$week)->where("suspended", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->whereRaw("(first_team_suspended = " . 0 . " OR second_team_suspended = " . 0 . ")")->pluck("id")->toArray();            
        // // error_log( $player_id );
        // $new_week = $week;
        // foreach ($rol_tournaments as $key => $value) {
        //     // error_log( $value ); 
        //     // $new_week++;           
        //     // $session  = RolTournamentSession::where("rol_tournament_id", $value)->where("team_id", $team_id)->pluck("id")->toArray();
        //     // $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $session)->where("player_id", $player_id)->avg("points");
        //     // error_log( count($session). "SEM => ". $new_week );

        //    //hay que revisar la configuración de la liga 
        // }

        // $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();
        // error_log( print_r( $rol_tournament_sessions, true ) );

    }

    public static function calcular_ronda_de_posiciones($rol_id, $current_iterator_week, $next_iterator_week, $category_id)
    {        

        $rol = Rol::find($rol_id);
        $league = League::find($rol->league_id);        
        
        //Revisamos si los partidos de la semana en curso ya finalizaron para ver si la siguiente semana será ronda de posiciones
        $total_rol_tournaments = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $current_iterator_week)->where("category_id", $category_id)->where("type", "!=", "festivo")->whereRaw("(first_team_total is null AND second_team_total is null)")->count();

        // error_log("calcula ronda de posiciones ==>". $total_rol_tournaments);
        
        //Revisamos, si no hay más puntajes por agregar es por que ya se llenaron todos los puntajes de la semana para cada partido.
        //Por lo que revisaremos si la siguiente semana es de ronda de posiciones
        if ($total_rol_tournaments <= 0) {

            $next_rol_tournaments = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $next_iterator_week)->where("category_id", $category_id)->first();

            if ($next_rol_tournaments != null) {
                
                error_log("total_rol_tournaments ====>>>". $next_rol_tournaments);
                //Siempre revisamos que la siguiente semana sea ronda de posiciones fisica o virtual, hay casos que se junta un día festivo con la ronda de posiciones.
                //por ejemplo si la siguiente semana es festivo y luego la que le sigue es ronda de posiciones, tenemos que brincarnos el día festivo
                if ($next_rol_tournaments->type == "festivo") {

                    self::calcular_ronda_de_posiciones($rol_id, $current_iterator_week, ($next_iterator_week + 1), $category_id);

                } else {

                    //Si la siguiente semana de juegos es ronda de posiciones, vamos a reordenar la rivalidad entre los equipos dependiendo de las posiciones de la semana actual,
                    //en este caso jugará el primero contra el segundo, el tercero contra el cuarto y asi sucesivamente.
                    if ($next_rol_tournaments->type == "ronda_posiciones_virtual" || $next_rol_tournaments->type == "ronda_posiciones" || $next_rol_tournaments->type == "ronda_final") {
                        // error_log("ronda de posiciones virtual ========>");
                        self::get_recalcular_ronda_de_posiciones($rol_id, $current_iterator_week, $next_iterator_week, $category_id, $next_rol_tournaments->type);
                    }

                }

            }

            //REVISAMOS SI EL PARTIDO ANTERIOR ES UNA RONDA DE POSICIONES VIRTUAL PARA CALIFICAR
            $prev_rol_tournaments = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $current_iterator_week - 1)->where("category_id", $category_id)->first();
            
            // error_log("SE GUARDA EN RONDA DE POSICIONES ==>". $prev_rol_tournaments);
            // error_log( "prev_rol_tournaments => ". print_r( $prev_rol_tournaments, true ) );
            if( $prev_rol_tournaments ){

                // error_log( "RONDA VIRTUAL ANTERIOR". $prev_rol_tournaments->type );
                if ($prev_rol_tournaments->type == "ronda_posiciones_virtual"){
                    
                    //Obtenemos la semana ANTERIOR que es ronda de posiciones virtual para irla iterando
                    $rol_tournaments = RolTournament::where("rol_id", $rol_id)->where("category_id", $category_id)->where("iterator_week", $current_iterator_week - 1)->orderBy("id", "ASC")->get();                                    

                    //Primero eliminamos toda la información que tenga la ronda de posiciones virtual para no tener datos basura, mas abajo se agregará dicha información
                    foreach ($rol_tournaments as $rol_tournament) {
                        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournament->id);
                        RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions->pluck("id")->toArray())->delete();
                        $rol_tournament_sessions->delete();
                    }   
                    
                    $handicap_each_categories = HandicapEachCategory::where("team_category_id", $category_id)->first();
                            
                    $puntos_por_seria_ganada = $league->get_player_types->point_by_win_serie;        
                    $total_confi_liga =  $league->get_player_types->number_game_by_session * $handicap_each_categories->base_dummy;
                    
                    foreach ($rol_tournaments as $rol_tournament) {
                        
                        for ($i = 0; $i < 2; $i++) {
    
                            if ($i == 0) {
                                $team_id = $rol_tournament->first_team_id;
                            } else {
                                $team_id = $rol_tournament->second_team_id;
                            }
                            
                            //Obtenemos la equivalencia de roul_tournament para la semana actual de ronda de posiciones
                            $rol_tournament_from_copy = RolTournament::where("rol_id", $rol_id)->where("category_id", $category_id)->where("iterator_week", $current_iterator_week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();
                            
                            //OBTENEMOS JUEGOS SUSPENDIDOS INDIVIDUALES
                            if( $rol_tournament_from_copy->first_team_suspended == 1 || $rol_tournament_from_copy->first_team_suspended == 0 ){
                                $rol_t = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $current_iterator_week - 1)->where("category_id", $category_id)->whereRaw("(first_team_id = " . $rol_tournament_from_copy->first_team_id . " OR second_team_id = " . $rol_tournament_from_copy->first_team_id . ")")->first();

                                if( $rol_t->first_team_id == $rol_tournament_from_copy->first_team_id ){
                                    RolTournament::where("id", $rol_t->id)->update(["first_team_suspended" => $rol_tournament_from_copy->first_team_suspended]);                                     
                                }

                                if( $rol_t->second_team_id == $rol_tournament_from_copy->first_team_id ){
                                    RolTournament::where("id", $rol_t->id)->update(["second_team_suspended" => $rol_tournament_from_copy->first_team_suspended]);
                                }
                                
                            }

                            if( $rol_tournament_from_copy->second_team_suspended == 1 || $rol_tournament_from_copy->second_team_suspended == 0 ){
                                
                                $rol_t = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $current_iterator_week - 1)->where("category_id", $category_id)->whereRaw("(first_team_id = " . $rol_tournament_from_copy->second_team_id . " OR second_team_id = " . $rol_tournament_from_copy->second_team_id . ")")->first();                                                         

                                if( $rol_t->first_team_id == $rol_tournament_from_copy->second_team_id ){                                    
                                    RolTournament::where("id", $rol_t->id)->update(["first_team_suspended" => $rol_tournament_from_copy->second_team_suspended]);                          
                                }

                                if( $rol_t->second_team_id == $rol_tournament_from_copy->second_team_id ){
                                    RolTournament::where("id", $rol_t->id)->update(["second_team_suspended" => $rol_tournament_from_copy->second_team_suspended]);
                                }

                            }
                                                        
                            //se puede programar mensaje en case de que no exista el equipo en la ronda anterior 
                            //para realizar el cambio en rol de juegos
                            
                            //Copiamos rol_tournament_sessions para la semana de posiciones
                            $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournament_from_copy->id)->where("team_id", $team_id)->get();
                                                        
                            foreach ($rol_tournament_sessions as $rol_tournament_session) {
                                
                                error_log( $rol_tournament_session );
                                $rol_tournament_sessions_insert = new RolTournamentSession();
                                $rol_tournament_sessions_insert->rol_tournament_id = $rol_tournament->id;
                                $rol_tournament_sessions_insert->team_id = $team_id;
                                $rol_tournament_sessions_insert->session_number = $rol_tournament_session->session_number;
                                $rol_tournament_sessions_insert->in_process_to_validate = 1;
                                $rol_tournament_sessions_insert->validate = 1;
                                //se añadio 13/12/21
                                $rol_tournament_sessions_insert->sum_puntaje = $rol_tournament_session->sum_puntaje;
                                $rol_tournament_sessions_insert->sum_handycap = $rol_tournament_session->sum_handycap;
                                $rol_tournament_sessions_insert->sum_puntaje_handycap = $rol_tournament_session->sum_puntaje_handycap;
                                $rol_tournament_sessions_insert->save();
                                
                                //Por cada rol_tournament_session vamos a obtener los puntajes ingresados
                                $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->get();                                
        
                                foreach ($rol_tournament_session_points as $rol_tournament_session_point) {                                 
                                    $rol_tournament_session_points_insert = new RolTournamentSessionPoint();
                                    $rol_tournament_session_points_insert->rol_tournament_session_id = $rol_tournament_sessions_insert->id;
                                    $rol_tournament_session_points_insert->player_id = $rol_tournament_session_point->player_id;
                                    $rol_tournament_session_points_insert->handycap_calculated = $rol_tournament_session_point->handycap_calculated;
                                    $rol_tournament_session_points_insert->points = $rol_tournament_session_point->points;
                                    $rol_tournament_session_points_insert->save();
                                    
                                }                             
                                
                            }
                                
                        }
        
                    }                

                    //despues de guardar determina quien gana o pierde
                    foreach ($rol_tournaments as $rol_tournament) {
                        
                        if( $rol_tournament->first_team_suspended == 1 || $rol_tournament->second_team_suspended == 1 ){                                                    
                            
                            //SI EL SUSPENDIDO ES PRIMERO
                            if( $rol_tournament->first_team_suspended == 1 ){                                
                                
                                $rol_t = RolTournament::where("id", $rol_tournament->id)->first();                                                                                                
                                
                                if ( $total_confi_liga > $rol_tournament->second_team_total) {
                                    // $rol_tournaments->first_team_win_serie = $puntos_por_seria_ganada;
                                    $rol_t->second_team_win_serie = 0;
                                    // RolTournament::where("id", $rol_tournament->id)->update(["second_team_win_serie" => 0]);                          
                                    
                                } else if ($total_confi_liga < $rol_tournament->second_team_total) {
                                    // $rol_tournaments->first_team_win_serie = 0;                                    
                                    $rol_t->second_team_win_serie = 1;
                                    // RolTournament::where("id", $rol_tournament->id)->update(["second_team_win_serie" => 1]);                          
                                    
                                } else {                                    
                                    $rol_t->first_team_win_serie = 0;
                                    $rol_t->second_team_win_serie = 0;
                                }

                                $rol_t->first_team_win_serie = 0;
                    
                                $rol_t->save();
                            }

                            if( $rol_tournament->second_team_suspended == 1 ){

                                $rol_t = RolTournament::where("id", $rol_tournament->id)->first();                                

                                if ( $total_confi_liga > $rol_tournament->first_team_total) {                
                                    // $rol_tournaments->first_team_win_serie = $puntos_por_seria_ganada;
                                    $rol_t->first_team_win_serie  = 0;
                                    $rol_t->second_team_win_serie = 0;
                                } else if ($total_confi_liga < $rol_tournament->first_team_total) {
                                    // $rol_tournaments->first_team_win_serie = 0;
                                    
                                    $rol_t->second_team_win_serie = 0;
                                    $rol_t->first_team_win_serie  = 1;
                                } else {
                                    
                                    $rol_t->first_team_win_serie  = 0;
                                    $rol_t->second_team_win_serie = 0;
                                }

                                $rol_t->second_team_win_serie = 0;
                                
                                $rol_t->save();                                

                            }                            

                        }else{

                            //gana el primer equipo y pierde el segundo
                            if( $rol_tournament->first_team_total > $rol_tournament->second_team_total ){
                                $r_tournament = RolTournament::where("id",$rol_tournament->id)->update(["first_team_win_serie" => 1, "second_team_win_serie" => 0 ]);
                                
                                //gana el segundo equipo y pierde el primero
                            }else if( $rol_tournament->first_team_total < $rol_tournament->second_team_total ){
                                // error_log( $rol_tournament->first_team_total. ">". $rol_tournament->second_team_total );
                                $r_tournament = RolTournament::where("id",$rol_tournament->id)->update(["first_team_win_serie" => 0, "second_team_win_serie" => 1 ]);
                            }else{
    
                            }

                        }

                        for ($serie = 1; $serie <= $league->get_player_types["number_game_by_session"]; $serie++) {
                            // error_log( $serie );
                                                    

                            $win_first_team = 0;
                            $win_second_team = 0;

                            $rol_tournament_sessions_first_team  = RolTournamentSession::where("rol_tournament_id", $rol_tournament->id)->where("team_id", $rol_tournament->first_team_id)->where("session_number", $serie)->first();
                            $rol_tournament_sessions_second_team = RolTournamentSession::where("rol_tournament_id", $rol_tournament->id)->where("team_id", $rol_tournament->second_team_id)->where("session_number", $serie)->first();

                            $rol_tournament_sessions_points_first_team  = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_first_team->id)->get();
                            $rol_tournament_sessions_points_second_team = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_second_team->id)->get();

                            $suma_puntaje_first_team    = 0;
                            $suma_puntaje_first_team_hc = 0;
                            $handycap_first             = 0;

                            $suma_puntaje_second_team    = 0;
                            $suma_puntaje_second_team_hc = 0;  
                            $handycap_second             = 0;                                                  

                            foreach ($rol_tournament_sessions_points_first_team as $key => $value) {                                                            
                                $suma_puntaje_first_team    = $suma_puntaje_first_team + $value->points;
                                $suma_puntaje_first_team_hc = $suma_puntaje_first_team_hc + $value->points + $value->handycap_calculated;
                                $handycap_first             = $value->handycap_calculated;
                            }

                            foreach ($rol_tournament_sessions_points_second_team as $key => $value) {
                                $suma_puntaje_second_team    = $suma_puntaje_second_team + $value->points;
                                $suma_puntaje_second_team_hc = $suma_puntaje_second_team_hc + $value->points + $value->handycap_calculated;
                                $handycap_second             = $value->handycap_calculated;
                            }
                            
                            // error_log( $suma_puntaje_first_team_hc.">".$suma_puntaje_second_team_hc);
                            
                            if($rol_tournament->first_team_id == 22 && $rol_tournament->second_team_id == 45){
                                // error_log( $rol_tournament->first_team_id." == ".$rol_tournament->second_team_id );
                                // error_log( $suma_puntaje_second_team );
                            }

                            if( $rol_tournament->first_team_suspended == 1 || $rol_tournament->second_team_suspended == 1 ){
                                
                                
                                if( $rol_tournament->first_team_suspended == 1 ){
                                    // error_log( "Equipo => ".  $rol_tournament->second_team_id);
                                    // error_log($suma_puntaje_second_team_hc);
                                    // error_log("El primero suspendido =>".$suma_puntaje_second_team_hc.">".$handicap_each_categories->base_dummy);
                                    if( $suma_puntaje_second_team_hc > $handicap_each_categories->base_dummy ){                            
                                        // $win_first_team = 1;
                                        $win_second_team = 1;
                                    }else if( $suma_puntaje_second_team_hc < $handicap_each_categories->base_dummy ){                                
                                        // $win_first_team = 0;
                                        $win_second_team = 0;
                                    }else{

                                    }

                                    RolTournamentSession::where("id", $rol_tournament_sessions_second_team->id)->update([
                                        "win" => $win_second_team,
                                        "sum_puntaje" => $suma_puntaje_second_team, 
                                        "sum_handycap" => $handycap_second,
                                        "sum_puntaje_handycap" => $suma_puntaje_second_team_hc  
                                    ]);                                                                        

                                }

                                if( $rol_tournament->second_team_suspended == 1 ){
                                    error_log("El segundo suspendido");
                                    if( $suma_puntaje_first_team_hc > $handicap_each_categories->base_dummy ){                            
                                        $win_first_team = 1;
                                        // $win_second_team = 1;
                                    }else if( $suma_puntaje_first_team_hc < $handicap_each_categories->base_dummy ){                                
                                        $win_first_team = 0;
                                        // $win_second_team = 0;
                                    }else{

                                    }

                                    RolTournamentSession::where("id", $rol_tournament_sessions_first_team->id)->update([
                                        "win" => $win_first_team,
                                        "sum_puntaje" => $suma_puntaje_first_team, 
                                        "sum_handycap" => $handycap_first,
                                        "sum_puntaje_handycap" => $suma_puntaje_first_team_hc  
                                    ]); 
                                }

                            }else{

                                if( $suma_puntaje_first_team_hc > $suma_puntaje_second_team_hc ){                            
                                    $win_first_team = 1;
                                    $win_second_team = 0;
                                }else if( $suma_puntaje_first_team_hc < $suma_puntaje_second_team_hc ){                                
                                    $win_first_team = 0;
                                    $win_second_team = 1;
                                }else{
                                    
                                    RolTournamentSession::where("id", $rol_tournament_sessions_first_team->id)->update([
                                        "win" => 0,
                                        "sum_puntaje" => $suma_puntaje_first_team, 
                                        "sum_handycap" => $handycap_first,
                                        "sum_puntaje_handycap" => $suma_puntaje_first_team_hc  
                                    ]);
    
                                    RolTournamentSession::where("id", $rol_tournament_sessions_second_team->id)->update([
                                        "win" => 0,
                                        "sum_puntaje" => $suma_puntaje_second_team,
                                        "sum_handycap" => $handycap_second,
                                        "sum_puntaje_handycap" => $suma_puntaje_second_team_hc
                                    ]);
    
                                }
    
                                if( $win_first_team == 1 ){
                                    
                                    RolTournamentSession::where("id", $rol_tournament_sessions_first_team->id)->update([
                                        "win" => 1,
                                        "sum_puntaje" => $suma_puntaje_first_team, 
                                        "sum_handycap" => $handycap_first,
                                        "sum_puntaje_handycap" => $suma_puntaje_first_team_hc  
                                    ]);
                                   
                                }else{
                                    RolTournamentSession::where("id", $rol_tournament_sessions_second_team->id)->update([
                                        "win" => 1,
                                        "sum_puntaje" => $suma_puntaje_second_team,
                                        "sum_handycap" => $handycap_second,
                                        "sum_puntaje_handycap" => $suma_puntaje_second_team_hc
                                    ]);
                                    
                                }
                                
                            }


                        }

                    }
    
                }

            }
            
        }

    }

    public static function get_recalcular_ronda_de_posiciones($rol_id, $iterator_week, $next_iterator_week, $category_id, $type)
    {

        //Vamos a recalcular la ronda de posiciones, primero obtenemos la tabla de posiciones de la semana actual
        $posiciones = self::get_tabla_de_posiciones_by_rol_week($rol_id, $iterator_week, $category_id)->toArray();

        $tabla_posiciones = self::get_tabla_de_posiciones_global($rol_id, $iterator_week, $category_id);             

        // error_log( $tabla_posiciones );

        //Obtenemos la ronda de posiciones de la siguiente semana para colocar los equipos conforme vengan en la tabla de posiciones y asignar el primero contra el segundo, tercero contra el cuarto y asi sucesivamente.
        $recalculo_posiciones = RolTournament::where("rol_id", $rol_id)->where("category_id", $category_id)->where("iterator_week", $next_iterator_week)->orderBy("id", "ASC")->get();        

        // SE CALIFICA UNA SEMANA DESPUES
        // $recalculo_posiciones = RolTournament::where("rol_id", $rol_id)->where("category_id", $category_id)->where("iterator_week", $next_iterator_week - 2)->orderBy("id", "ASC")->get();

        $primer_equipo_test  = 0;
        $segundo_equipo_test = 1;
        $order_tabla_posiciones = [];
        $num = 0;

        //REAGRUPAMOS
        foreach( $tabla_posiciones as $posicion ){  
            // error_log("num => ". $num. " ==== ". $posicion['team_id'] );        
            array_push( $order_tabla_posiciones, array( "id" => $num, "team_id" => $posicion['team_id'], "total_win" => $posicion['total_win'], "name" => $posicion['get_team']['name'], "team_total" => $posicion['team_total']  ));
            $num ++;
        }        

        foreach ($recalculo_posiciones as $rol_tournament) {


            $rol_tournament->first_team_id  = $order_tabla_posiciones[$primer_equipo_test]["team_id"];
            $rol_tournament->second_team_id = $order_tabla_posiciones[$segundo_equipo_test]["team_id"];
            
            if ($type == "ronda_posiciones_virtual") {
                $rol_tournament->first_team_total  = $posiciones[$primer_equipo_test]["team_total"];
                $rol_tournament->second_team_total = $posiciones[$segundo_equipo_test]["team_total"];
            }      

            if($rol_tournament->save()){
                error_log("Se actualiza ====>". $next_iterator_week);
            }

            $primer_equipo_test += 2;
            $segundo_equipo_test += 2;
        }
        
        /*
        $primer_equipo = 0;
        $segundo_equipo = 1;        
        
        foreach ($recalculo_posiciones as $rol_tournament) {

            $rol_tournament->first_team_id = $posiciones[$primer_equipo]["team_id"];
            $rol_tournament->second_team_id = $posiciones[$segundo_equipo]["team_id"];

            //Si es virtual, copiamos el total de puntos la semana actual. Cuando es virtual es como si ya jugaron con el nuevo recalculo respecto a la tabla general de la semana actual
            //adicional se toma el puntaje que hicieron en la semana para simular como si recibieron este puntaje.
            if ($type == "ronda_posiciones_virtual") {
                $rol_tournament->first_team_total = $posiciones[$primer_equipo]["team_total"];
                $rol_tournament->second_team_total = $posiciones[$segundo_equipo]["team_total"];
            }

            error_log( $posiciones[$segundo_equipo]["team_total"]);

            $rol_tournament->save();

            $primer_equipo += 2;
            $segundo_equipo += 2;

        }

        */

        // return;
        
        //ESTO CAMBIA: DEBE CALIFICAR CUANDO ESTE EN UNA SEMANA DESPUES DE LA RONDA VIRTUAL
        
        //Tenemos que copiar tambien el puntaje y la sesion de puntaje generado en la iteracion actual para la siguiente que es ronda de posiciones, solamente aplica para rondas virtuales
        /*
        if ($type == "ronda_posiciones_virtual") {
            
            //Obtenemos la siguiente semana que es ronda de posiciones virtual para irla iterando
            $rol_tournaments = RolTournament::where("rol_id", $rol_id)->where("category_id", $category_id)->where("iterator_week", $next_iterator_week)->orderBy("id", "ASC")->get();
            
            //Primero eliminamos toda la información que tenga la ronda de posiciones virtual para no tener datos basura, mas abajo se agregará dicha información
            foreach ($rol_tournaments as $rol_tournament) {
                $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournament->id);
                RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions->pluck("id")->toArray())->delete();
                $rol_tournament_sessions->delete();
            }            
            

            foreach ($rol_tournaments as $rol_tournament) {

                for ($i = 0; $i < 2; $i++) {

                    if ($i == 0) {
                        $team_id = $rol_tournament->first_team_id;
                    } else {
                        $team_id = $rol_tournament->second_team_id;
                    }

                    //Obtenemos la equivalencia de roul_tournament para la semana anterior de ronda de posiciones
                    $rol_tournament_from_copy = RolTournament::where("rol_id", $rol_id)->where("category_id", $category_id)->where("iterator_week", $iterator_week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();

                    //Copiamos rol_tournament_sessions para la semana de posiciones
                    $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournament_from_copy->id)->where("team_id", $team_id)->get();

                    foreach ($rol_tournament_sessions as $rol_tournament_session) {

                        $rol_tournament_sessions_insert = new RolTournamentSession();
                        $rol_tournament_sessions_insert->rol_tournament_id = $rol_tournament->id;
                        $rol_tournament_sessions_insert->team_id = $team_id;
                        $rol_tournament_sessions_insert->session_number = $rol_tournament_session->session_number;
                        $rol_tournament_sessions_insert->in_process_to_validate = 1;
                        $rol_tournament_sessions_insert->validate = 1;
                        $rol_tournament_sessions_insert->save();

                        //Por cada rol_tournament_session vamos a obtener los puntajes ingresados
                        $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->get();

                        foreach ($rol_tournament_session_points as $rol_tournament_session_point) {

                            $rol_tournament_session_points_insert = new RolTournamentSessionPoint();
                            $rol_tournament_session_points_insert->rol_tournament_session_id = $rol_tournament_sessions_insert->id;
                            $rol_tournament_session_points_insert->player_id = $rol_tournament_session_point->player_id;
                            $rol_tournament_session_points_insert->handycap_calculated = $rol_tournament_session_point->handycap_calculated;
                            $rol_tournament_session_points_insert->points = $rol_tournament_session_point->points;
                            $rol_tournament_session_points_insert->save();

                        }

                    }


                    //$rol_tournament_session_points_totals = RolTournamentSessionPointsTotal::where("rol_tournament_id", $rol_tournament_from_copy->id)->get();
                    //
                    ////Eliminamos en caso de que exista para no tener información repetida
                    //RolTournamentSessionPointsTotal::where("rol_tournament_id", $rol_tournament->id)->delete();
                    //
                    //foreach ($rol_tournament_session_points_totals as $rol_tournament_session_points_total) {
                    //
                    //    $rol_tournament_session_points_totals_insert = new RolTournamentSessionPointsTotal();
                    //    $rol_tournament_session_points_totals_insert->rol_tournament_id = $rol_tournament->id;
                    //    $rol_tournament_session_points_totals_insert->player_id = $rol_tournament_session_points_total->player_id;
                    //    $rol_tournament_session_points_totals_insert->serie_total = $rol_tournament_session_points_total->serie_total;
                    //    $rol_tournament_session_points_totals_insert->pt = $rol_tournament_session_points_total->pt;
                    //    $rol_tournament_session_points_totals_insert->handycap_calculated = $rol_tournament_session_points_total->handycap_calculated;
                    //    $rol_tournament_session_points_totals_insert->save();
                    //
                    //}

                }

            }

        }
        
        */
    }


    public static function get_tabla_de_posiciones_by_rol_week($rol_id, $iterator_week, $category_id)
    {

        // error_log( $rol_id."---".$iterator_week."--".$category_id );

        $first_team  = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $iterator_week)->where("category_id", $category_id)->select(["rol_id", "category_id", "iterator_week", "date", "type", "first_table_number as table_number", "first_team_id as team_id", "first_team_total as team_total"]);
        $second_team = RolTournament::where("rol_id", $rol_id)->where("iterator_week", $iterator_week)->where("category_id", $category_id)->select(["rol_id", "category_id", "iterator_week", "date", "type", "second_table_number as table_number", "second_team_id as team_id", "second_team_total as team_total"]);

        $second_team->union($first_team)->orderBy("team_total", "DESC");

        // error_log( print_r( $second_team->get(), true ) );
        $teams = $second_team->get();
        //Por cada equipo, vamos a verificar cuanto puntaje tiene en total y cuando gano o perdio
        foreach ($teams as $key => $team) {

        }

        return $second_team->get();
    }

    //Se calcula la ronda deposiciones sumando el total de todas las semanas sucesoras a la recibida, por ejemplo si se quiere la ronda de posiciones de la semana 5
    //se generará la sumatoria de todos los pinos para la semana 1,2,3,4 y 5
    public static function get_tabla_de_posiciones_global($rol_id, $iterator_week, $category_id)
    {
        
        $rol    = Rol::find($rol_id);
        $league = League::find($rol->league_id);
        $number_game_by_session = $league->get_player_types["number_game_by_session"];        
        
        //Obtenemos un listado de todos los equipos
        $first_team  = RolTournament::where("rol_id", $rol_id)->where("iterator_week", 0)->where("category_id", $category_id)->with("get_team")->select(["id", "rol_id", "category_id", "first_team_id as team_id"]);
        $second_team = RolTournament::where("rol_id", $rol_id)->where("iterator_week", 0)->where("category_id", $category_id)->with("get_team")->select(["id", "rol_id", "category_id", "second_team_id as team_id"]);
        
        $second_team->union($first_team);
        $teams = $second_team->get();
        
        //Por cada equipo, vamos a verificar cuanto puntaje tiene en total y cuando gano o perdio
        foreach ($teams as $key => $team) {                         

            //->where("week",$week_number_pagination)
            $teams_added_in_current_tournament = TeamsAddedInCurrentTournament::where("team_id",$team->team_id)->where("league_id", $league->id)->where("type",1)->first();                    
            
            /********************* SERIE GANADA O PERDIDA **********************/
            //Obtenemos el total serie ganadas
            if( $teams_added_in_current_tournament ){
                $first_place_serie_win_total  = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->where("first_team_win_serie", ">=", 1)->whereNotNull("first_team_win_serie")->sum("first_team_win_serie");
                $second_place_serie_win_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->where("second_team_win_serie", ">=", 1)->whereNotNull("second_team_win_serie")->sum("second_team_win_serie");                
            }else{
                $first_place_serie_win_total  = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->where("first_team_win_serie", ">=", 1)->whereNotNull("first_team_win_serie")->sum("first_team_win_serie");
                $second_place_serie_win_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->where("second_team_win_serie", ">=", 1)->whereNotNull("second_team_win_serie")->sum("second_team_win_serie");
            }

            $total_win = $first_place_serie_win_total + $second_place_serie_win_total;  
            

            //Obtenemos el total serie perdidas
            if( $teams_added_in_current_tournament ){
                $first_place_serie_loss_total  = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->where("first_team_win_serie", 0)->whereNotNull("first_team_win_serie")->count();
                $second_place_serie_loss_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->where("second_team_win_serie", 0)->whereNotNull("second_team_win_serie")->count();
            }else{
                $first_place_serie_loss_total  = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->where("first_team_win_serie", 0)->whereNotNull("first_team_win_serie")->count();
                $second_place_serie_loss_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->where("second_team_win_serie", 0)->whereNotNull("second_team_win_serie")->count();
            }

            $total_loss = $first_place_serie_loss_total + $second_place_serie_loss_total;

            /********************* TOTAL PUNTOS **********************/
            //Obtenemos el total de puntaje por serie
            if( $teams_added_in_current_tournament ){                
                $first_place_serie_total  = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->sum("first_team_total");
                $second_place_serie_total = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->sum("second_team_total");                
            }else{
                $first_place_serie_total  = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->sum("first_team_total");
                $second_place_serie_total = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->sum("second_team_total");

                if( $team->team_id == 6 ){
                    // $first = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->get();
                    // $sec = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->get();
                    // error_log($first);
                    // error_log($sec);
                    // error_log($second_place_serie_total);
                }
            }

            
            $total_serie_pt = $first_place_serie_total + $second_place_serie_total; 

            // if( $team->team_id == 6 ){

            //     $first_place_serie_total_test  = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->pluck("id")->toArray();

            //     // error_log( print_r( $first_place_serie_total_test, true ) );

            //     // error_log( "SUMATORIA => ".$first_place_serie_total." === ".$second_place_serie_total );
            // }

            /********************* TOTAL JUEGOS GANADO O PERDIDO **********************/
            //Obtener el total de juegs ganados
            if( $teams_added_in_current_tournament ){
                $first_place_game_win_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 1);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
                
            }else{
                $first_place_game_win_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 1);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
            }

            if( $teams_added_in_current_tournament ){
                $second_place_game_win_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 1);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
            }else{
                $second_place_game_win_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 1);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();                
            }

            $total_win_games = $first_place_game_win_total + $second_place_game_win_total;


            // if( $team->team_id == 45 ){
            //     error_log(" total_win_games =>".$total_win_games);
            //     error_log(" first_place_game_win_total =>".$first_place_game_win_total);
            //     error_log(" second_place_game_win_total =>".$second_place_game_win_total);
            // }

            
            //Obtener el total de juegs perdidos
            if( $teams_added_in_current_tournament ){
                $first_place_game_loss_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 0);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
            }else{
                $first_place_game_loss_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("first_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 0);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
            }
            
            if( $teams_added_in_current_tournament ){
                $second_place_game_loss_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1) )->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 0);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
            }else{
                $second_place_game_loss_total = RolTournament::where("rol_id", $rol_id)->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->where("second_team_id", $team->team_id)->with(["get_rol_tournament_sessions" => function ($query) use ($team) {
                    $query->where("team_id", $team->team_id)->where("win", 0);
                }])->get()->pluck("get_rol_tournament_sessions")->flatten()->count();
            }


            $total_loss_games = $first_place_game_loss_total + $second_place_game_loss_total;
            
            //Le sumamos a total_win y a total_loss la cantidad de juegos ganados de total_win_games y total_loss_games          
            $total_win  += $total_win_games;
            $total_loss += $total_loss_games;    

            if( $team->team_id == 45 ){
                error_log("total_win => " .$total_win);
            }
            
            //Asignamos los totales a cada equipo
            
            if( $teams_added_in_current_tournament ){

                //consulta el numero de semanas omitiendo ronda de posiciones
                // $semanas_virtuales = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", ">=", ($teams_added_in_current_tournament->week-1))->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->whereRaw("(first_team_id = " . $team->team_id . " OR second_team_id = " . $team->team_id . ")")->count();

                $r_t_avegage = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")                                            
                                            ->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)
                                            ->whereRaw("(first_team_id = " . $team->team_id . " OR second_team_id = " . $team->team_id . ")")                                            
                                            ->pluck("id")->toArray();

                $num_semana = self::numSemanasTorneo( $r_t_avegage, $team->team_id ); 

                // error_log( $semanas_virtuales );
                if( $iterator_week >= ($teams_added_in_current_tournament->week-1)  ){
                    $avegage = $total_serie_pt / ($number_game_by_session * $num_semana);                    
                }else{
                    $avegage = 0;
                }
                
            }else{

               //consulta el numero de semanas omitiendo ronda de posiciones
                $r_t_avegage = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")                                            
                                            ->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)
                                            ->whereRaw("(first_team_id = " . $team->team_id . " OR second_team_id = " . $team->team_id . ")")                                            
                                            ->pluck("id")->toArray();

                $num_semana = self::numSemanasTorneo( $r_t_avegage, $team->team_id );                
                
                // $semanas_virtuales = RolTournament::where("rol_id", $rol_id)->where("type", "!=", "ronda_posiciones_virtual")->where("iterator_week", "<=", $iterator_week)->where("category_id", $category_id)->whereRaw("(first_team_id = " . $team->team_id . " OR second_team_id = " . $team->team_id . ")")->count();
                // $semanas_virtuales

                $number_game_by_session = $number_game_by_session ?  $number_game_by_session : 1;
                $num_semana = $num_semana ? $num_semana : 1;

                $avegage = $total_serie_pt / ($number_game_by_session * $num_semana );
                
            }


            $avegage = round($avegage, 0, PHP_ROUND_HALF_UP);                        
            
            $teams[$key]->total_win  = $total_win;
            $teams[$key]->total_loss = $total_loss;
            $teams[$key]->total_serie_pt_average = $avegage;
            $teams[$key]->total_serie_pt = $total_serie_pt;
            
            if( $team->team_id == 6 ){
                error_log($total_serie_pt);
            }


            $teams[$key]->total_to_order = ($total_win != 0 ? $total_win : 0.5) * $total_serie_pt;  //Este valor lo ocupamos para ordenar, ya que se ordena por ganados y luego por el total de pinos

        }

        // ANTES
        // $teams = $teams->sortBy("total_win", 1, true);

        //REALIZA EL ORDEN EN BASE A LO SIGUIENTE
        $criteria = ["total_win" => "desc", "total_serie_pt" => "desc"];
        $comparer = self::makeComparer($criteria);
        $sorted = $teams->sort($comparer);
        // $actual = $sorted->values()->toArray();

        $teams = $sorted;
        
        $dummy_team = null;
        foreach ($teams as $key => $team) {            

            if ($team->get_team["name"] == "DUMMY") {
                $dummy_team = $teams[$key];
                unset($teams[$key]);
            }

        }
        
        $teams->push($dummy_team);
        
        return $teams;
    }

    public static function numSemanasTorneo( $rol_tournament, $team_id ){
        $num = 0;
        foreach ($rol_tournament as $key => $r_t) {
            $r_t_s = RolTournamentSession::where("rol_tournament_id",$r_t)->where("team_id", $team_id)->count();
            if( $r_t_s > 0 ){                
                $num++;
            }
            // error_log($team_id ."===". $r_t_s);
        }
        // error_log("================");
        return $num;
    }

    public static function makeComparer($criteria) {
        $comparer = function ($first, $second) use ($criteria) {
          foreach ($criteria as $key => $orderType) {
            // normalize sort direction
            $orderType = strtolower($orderType);
            if ($first[$key] < $second[$key]) {
              return $orderType === "asc" ? -1 : 1;
            } else if ($first[$key] > $second[$key]) {
              return $orderType === "asc" ? 1 : -1;
            }
          }
          // all elements were equal
          return 0;
        };
        return $comparer;
      }
        
}

































