<?php

namespace MetodikaTI\Http\Controllers\API;

use Carbon\Carbon;
use Illuminate\Contracts\Logging\Log;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use MetodikaTI\AppUserTokenSession;
use MetodikaTI\CategoryRule;
use MetodikaTI\GameIndividualStatistic;
use MetodikaTI\Http\Controllers\Back\Liga\RolesController;
use MetodikaTI\Http\Controllers\Controller;
use MetodikaTI\Http\Requests\API\DoLoginRequest;
use MetodikaTI\Http\Requests\API\DoRegisterRequest;
use MetodikaTI\Http\Requests\API\RecoverPasswordRequest;
use MetodikaTI\Http\Requests\API\UpdateProfileRequest;
use MetodikaTI\League;
use MetodikaTI\Library\Pastora;
use MetodikaTI\Player;
use MetodikaTI\PlayerInTeamEachCategory;
use MetodikaTI\PlayersInGame;
use MetodikaTI\PlayerType;
use MetodikaTI\Rol;
use MetodikaTI\RolTournament;
use MetodikaTI\RolTournamentSession;
use MetodikaTI\RolTournamentSessionPoint;
use MetodikaTI\Team;
use MetodikaTI\TeamCategory;
use MetodikaTI\TeamEachCategory;
use PhpOffice\PhpSpreadsheet\Shared\Date;
use App\library\conekta\lib\Conekta;
use MetodikaTI\Handicap;
use MetodikaTI\PlayerRules;
use MetodikaTI\HandicapEachCategory;
use MetodikaTI\Practice;
use MetodikaTI\PracticesPlay;
use MetodikaTI\PracticeForGames;
use MetodikaTI\TeamsAddedInCurrentTournament;
use MetodikaTI\AlertsMovil;
use MetodikaTI\DummyPenalty;
use MetodikaTI\Penalties;
use MetodikaTI\LeagueContact;
use MetodikaTI\PremiacionEachCategory;
use MetodikaTI\PremiacionLugares;


use MetodikaTI\TeamAndCategorySpecification;

use Validator;

class APIController extends Controller
{

    public function __construct(){
        \Conekta\Conekta::setApiKey( "key_YinuQH4ZD6iZmqHsyxubag" );
        \Conekta\Conekta::setApiVersion("2.0.0");
    }

    public function do_login(DoLoginRequest $request)
    {

        $response = array();
        $response["message"] = "";

        $is_player_exists = Player::where("email", $request->email)->where("password", md5($request->password))->first();

        if ($is_player_exists != null) {

            //Primero eliminamos los token ascociados a este usuario
            AppUserTokenSession::where("player_id", $is_player_exists->id)->delete();

            //Generamos el token
            $app_user_token_sessions = new AppUserTokenSession();
            $app_user_token_sessions->access_token = md5(date("Y-m-d H:i:s") . "-" . $is_player_exists->id);
            $app_user_token_sessions->player_id = $is_player_exists->id;
            $app_user_token_sessions->expirated_at = Carbon::now()->addYears(2);
            $app_user_token_sessions->save();

            $response["message"] = "Ingreso correcto.";
            $response["token"] = $app_user_token_sessions->access_token;
            return response()->json($response);

        } else {

            //No existe el usuario o credenciales incorrectas
            $response["message"] = "Correo electrónico o contraseña incorrectos, verifica el acceso a tu cuenta.";
            return response()->json($response, 504);

        }

    }


    public function recover_password(RecoverPasswordRequest $request)
    {


        $response = array();
        $response["message"] = "";

        $is_player_exists = Player::where("email", $request->email)->first();

        if ($is_player_exists != null) {

            //Existe el usuario, generamos una nueva contraseña
            $new_password = Pastora::randomPassword();

            //Asignamos la nueva contraseña
            $is_player_exists->password = md5($new_password);
            $is_player_exists->save();

            \Mail::send('mail.recover_password_app', ['user' => $is_player_exists, "password" => $new_password], function ($m) use ($is_player_exists) {
                $m->from('soporte@beonline.com', 'BeOnline');
                $m->to($is_player_exists->email, $is_player_exists->name)->subject('Restablecimiento de contraseña');
            });

            $response["message"] = "Te hemos enviado un correo electrónico con tu nueva contraseña de acceso.";
            return response()->json($response);

        } else {

            //No existe el usuario
            $response["message"] = "El correo electrónico no se encuentra registrado.";
            return response()->json($response, 504);

        }

    }


    public function do_register(DoRegisterRequest $request)
    {

        $response = array();
        $response["message"] = "";

        $user = new Player();
        $user->name = $request->name;
        $user->last_name = $request->last_name;
        $user->second_last_name = $request->second_last_name;
        $user->nickname = $request->nickname;
        $user->birthday = Carbon::createFromFormat("d/m/Y", $request->birthday)->format("Y-m-d");
        $user->gender = $request->gender;
        $user->city = $request->city;
        $user->state = $request->state;
        $user->country = $request->country;
        $user->nationality = $request->nationality;
        $user->nationality_second = $request->nationality_second;
        $user->residence_city = $request->residence_city;
        $user->cellphone = $request->cellphone;
        $user->email = $request->email;
        $user->password = md5($request->password);

        if ($user->save()) {

            //Enviamos por correo electronico el acceso
            \Mail::send('mail.register_app', ['user' => $user, "password" => $request->password], function ($m) use ($user) {
                $m->from('soporte@beonline.com', 'BeOnline');
                $m->to($user->email, $user->name)->subject('Bienvenido a BeOnline');
            });

            //Generamos el token
            $app_user_token_sessions = new AppUserTokenSession();
            $app_user_token_sessions->access_token = md5(date("Y-m-d H:i:s") . "-" . $user->id);
            $app_user_token_sessions->player_id = $user->id;
            $app_user_token_sessions->expirated_at = Carbon::now()->addYears(2);
            $app_user_token_sessions->save();

            $response["message"] = "Te hemos registrado correctamente, bienvenido a BeOnline.";
            $response["token"] = $app_user_token_sessions->access_token;
            return response()->json($response);

        } else {

            //No existe el usuario o credenciales incorrectas
            $response["message"] = "Correo electrónico o contraseña incorrectos, verifica el acceso a tu cuenta.";
            return response()->json($response, 504);

        }

    }


    public function get_profile_data(Request $request)
    {

        $response = array();
        $response["message"] = "";

        // unset($request->player_object["password"]);

        // $response["player"] = $request->player_object;     


        $player = Player::where("email", $request->email)->where("password", md5($request->password))->first();
        $response["player"] = $player;

        return response()->json($response);
    }


    public function update_profile_data(UpdateProfileRequest $request)
    {

        $response = array();
        $response["message"] = "";

        $user = Player::find($request->player_object->id);
        $user->name = $request->name;
        $user->last_name = $request->last_name;
        $user->second_last_name = $request->second_last_name;
        $user->nickname = $request->nickname;
        $user->birthday = Carbon::createFromFormat("d/m/Y", $request->birthday)->format("Y-m-d");
        $user->gender = $request->gender;
        $user->city = $request->city;
        $user->state = $request->state;
        $user->country = $request->country;
        $user->nationality = $request->nationality;
        $user->nationality_second = $request->nationality_second;
        $user->residence_city = $request->residence_city;
        $user->cellphone = $request->cellphone;
        $user->email = $request->email;

        if ($request->password != "") {
            $user->password = md5($request->password);
        }

        if ($user->save()) {

            $response["message"] = "La información de tu cuenta se ha actualizado correctamente.";
            return response()->json($response);

        } else {

            //No existe el usuario o credenciales incorrectas
            $response["message"] = "No hemos podido actualizar la información de tu cuenta, intentalo nuevamente.";
            return response()->json($response, 504);

        }

    }


    public function search_my_leagues(Request $request)
    {

        $response = array();
        $response["data"] = "";
        $response["message"] = "";

        $league_text = $request->league_text;

        $player_in_team_each_categories = PlayerInTeamEachCategory::where("player_id", $request->player_object["id"])->whereHas("team_each_categories.get_team_categories.get_team_and_category_specifications.get_leagues", function ($get_leagues) use ($league_text) {
            $get_leagues->where("league_name", "LIKE", "%" . $league_text . "%");
        });

        $leagues = array();

        if ($player_in_team_each_categories->count() > 0) {

            foreach ($player_in_team_each_categories->get() as $player_in_team_each_categories) {

                $team_each_categories = $player_in_team_each_categories->team_each_categories;

                foreach ($team_each_categories->get_team_categories as $get_team_categories) {
                    foreach ($get_team_categories->get_team_and_category_specifications as $get_team_and_category_specifications) {
                        foreach ($get_team_and_category_specifications->get_leagues as $get_leagues) {
                            $league = $get_leagues;
                            $league->category_id = $team_each_categories->team_category_id;
                            $leagues[] = $league;
                        }
                    }
                }

            }

            $response["data"] = $leagues;
            $response["message"] = "";
        } else {
            $response["data"] = [];
            $response["message"] = "No existen ligas con el criterio de búsqueda ingresado.";
        }

        return response()->json($response);
    }


    public function get_rol_general(Request $request)
    {

        $response = array();
        $response["rol"] = "";
        $response["teams_in_rol"] = "";

        $league = League::find($request->league_id);
        $rol_data = Rol::where("league_id", $league->id)->first();
        $rol = array();

        //Obtenemos la clase de RolesController para usar funciones
        $roles_controller = new RolesController();

        //Generamos el array como cuando genera el rol de juegos
        //Iteramos por cada categoria
        foreach ($league->get_team_and_category_specification->get_categories->where("id", $request->category_id) as $category) {

            $rol_content = array("category" => array(), "fechas_torneo" => array(), "select_equipos" => array(), "mesas" => array(), "equipos" => array());

            //************************************************************
            //************************************ CATEGORY
            //************************************************************
            $rol_content["category"] = $category;


            //************************************************************
            //************************************ SELECT EQUIPOS
            //************************************************************
            $rol_content["select_equipos"] = $category->get_teams_for_select();


            //************************************************************
            //************************************ MESAS
            //************************************************************
            //Generamos el array para las mesas dependiendo de la cantidad de equipos por categoria
            $mesas = array();
            $table_number_start = 1;

            if ($category->is_table_asigned == 1) {
                $table_number_start = $category->table_number_start;
            }

            for ($i = 0; $i < $category->get_team_each_category->count(); $i++) {
                $mesas[] = $table_number_start;
                $table_number_start++;
            }

            $rol_content["mesas"] = $mesas;


            //************************************************************
            //************************************ EQUIPOS
            //************************************************************
            $rol_content["equipos"] = $category->get_team_each_category;


            //************************************************************
            //************************************ FECHAS TORNEO
            //************************************************************
            $fechas_torneo = array();
            $total_groups_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->groupBy("iterator_week")->get()->count();

            $iterador_semana = 1;
            for ($i = 0; $i < $total_groups_iterator_week; $i++) {

                $contenido_fecha_torneo = array("fecha" => "", "type" => "", "partidos" => "");
                $games_in_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("iterator_week", $i)->orderBy("id")->get();
                $first_game_in_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("iterator_week", $i)->orderBy("id")->first(); //Este nos servidora para obtener la fecha y el tipo

                $contenido_fecha_torneo["fecha"] = Carbon::createFromFormat("Y-m-d", $first_game_in_iterator_week->date);
                $contenido_fecha_torneo["type"] = $first_game_in_iterator_week->type;
                $contenido_fecha_torneo["colspan"] = count($category->get_team_each_category) + ((count($category->get_team_each_category) / 2) - 1);

                //Obtenemos la descripcion del tipo de fila
                if ($first_game_in_iterator_week->type == "festivo") {
                    $contenido_fecha_torneo["descripion"] = $roles_controller->get_festivo_description($first_game_in_iterator_week->date);
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "partido") {
                    $contenido_fecha_torneo["descripion"] = "Partido";
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "ronda_final") {
                    $contenido_fecha_torneo["descripion"] = "Ronda final";
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "ronda_inicial") {
                    $contenido_fecha_torneo["descripion"] = "Ronda inicial";
                    $contenido_fecha_torneo["semana"] = $iterador_semana++;
                } elseif ($first_game_in_iterator_week->type == "ronda_posiciones") {
                    $contenido_fecha_torneo["descripion"] = "Ronda de posiciones";

                    if (($i == ($total_groups_iterator_week - 2))) {
                        $contenido_fecha_torneo["semana"] = $iterador_semana -= 1;
                        $iterador_semana++;
                    } else {
                        $contenido_fecha_torneo["semana"] = $iterador_semana++;
                    }

                } elseif ($first_game_in_iterator_week->type == "ronda_posiciones_virtual") {

                    $contenido_fecha_torneo["descripion"] = "Ronda de posiciones virtual";

                    //Para el caso cuando se agrega la ronda de posiciones al final, revisaremos si estamos iterando dicho registro para evaluar el numero de semana
                    if (($i == ($total_groups_iterator_week - 2))) {
                        $contenido_fecha_torneo["semana"] = $iterador_semana -= 1;
                        $iterador_semana++;
                    } else {
                        $contenido_fecha_torneo["semana"] = $iterador_semana;
                    }
                }

                $games_in_week = array();
                foreach ($games_in_iterator_week as $games) {

                    $games_in_week[] =
                        array(
                            "Home" =>
                                array("id" => $games->id, "team_category_id" => $games->category_id, "team_id" => $games->first_team_id, "table_number" => $games->first_table_number),
                            "Away" =>
                                array("id" => $games->id, "team_category_id" => $games->category_id, "team_id" => $games->second_team_id, "table_number" => $games->second_table_number)
                        );

                }

                $contenido_fecha_torneo["partidos"] = $games_in_week;

                $fechas_torneo[] = $contenido_fecha_torneo;

            }

            $rol_content["fechas_torneo"] = $fechas_torneo;

            $rol[] = $rol_content;
        }


        //return view('api.rol.rol', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol));
        //return view('api.rol.equipos-en-rol', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol));

        $response["rol"] = view('api.rol.rol', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol))->render();
        $response["teams_in_rol"] = view('api.rol.equipos-en-rol', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol))->render();

        return response()->json($response);
    }


    public function get_rol_detalle(Request $request)
    {

        $response = array();
        $response["rol_detalle"] = "";

        //Solo obtendremos el rol de la caegoria a la que pertenece el jugador
        //league_id
        //category_id
        //$request->player_object["id"]

        $league = League::find($request->league_id);
        $rol_data = Rol::where("league_id", $league->id)->first();
        $rol = array();


        //Generamos el array como cuando genera el rol de juegos
        //Iteramos por cada categoria
        foreach ($league->get_team_and_category_specification->get_categories->where("id", $request->category_id) as $category) {

            $rol_content = array("category" => array(), "fechas_torneo" => array(), "select_equipos" => array(), "mesas" => array(), "equipos" => array());

            //************************************************************
            //************************************ CATEGORY
            //************************************************************
            $rol_content["category"] = $category;


            //dd($category->get_teams_for_select());

            //************************************************************
            //************************************ SELECT EQUIPOS
            //************************************************************
            $rol_content["select_equipos"] = $category->get_teams_for_select();


            //************************************************************
            //************************************ MESAS
            //************************************************************
            //Generamos el array para las mesas dependiendo de la cantidad de equipos por categoria
            $mesas = array();
            $table_number_start = 1;

            if ($category->is_table_asigned == 1) {
                $table_number_start = $category->table_number_start;
            }

            for ($i = 0; $i < $category->get_team_each_category->count(); $i++) {
                $mesas[] = $table_number_start;
                $table_number_start++;
            }

            $rol_content["mesas"] = $mesas;


            //************************************************************
            //************************************ EQUIPOS
            //************************************************************
            $rol_content["equipos"] = $category->get_team_each_category;


            //************************************************************
            //************************************ FECHAS TORNEO
            //************************************************************
            $fechas_torneo = array();
            $total_groups_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->groupBy("iterator_week")->get()->count();

            for ($i = 0; $i < $total_groups_iterator_week; $i++) {

                $contenido_fecha_torneo = array("fecha" => "", "type" => "", "partidos" => "");
                $games_in_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("iterator_week", $i)->orderBy("id")->get();
                $first_game_in_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("iterator_week", $i)->orderBy("id")->first(); //Este nos servidora para obtener la fecha y el tipo

                $contenido_fecha_torneo["fecha"] = Carbon::createFromFormat("Y-m-d", $first_game_in_iterator_week->date);
                $contenido_fecha_torneo["type"] = $first_game_in_iterator_week->type;

                $games_in_week = array();
                foreach ($games_in_iterator_week as $games) {

                    $games_in_week[] =
                        array(
                            "Home" =>
                                array("id" => $games->id, "team_category_id" => $games->category_id, "team_id" => $games->first_team_id, "team_name" => $games->get_first_team["name"], "table_number" => $games->first_table_number),
                            "Away" =>
                                array("id" => $games->id, "team_category_id" => $games->category_id, "team_id" => $games->second_team_id, "team_name" => $games->get_second_team["name"], "table_number" => $games->second_table_number)
                        );

                }

                $contenido_fecha_torneo["partidos"] = $games_in_week;

                $fechas_torneo[] = $contenido_fecha_torneo;
            }

            $rol_content["fechas_torneo"] = $fechas_torneo;


            $rol[] = $rol_content;
        }


        //return view('api.rol.rol-detalle', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol));

        $response["rol_detalle"] = view('api.rol.rol-detalle', array("league" => $league, "rol_data" => $rol_data, "rol" => $rol))->render();

        return response()->json($response);
    }


    public function get_list_week_games(Request $request)
    {

        $response = array();
        $response["rol_detalle"] = "";

        error_log( json_encode( $request->all() ) );

        $league = League::find($request->league_id);
        $rol_data = Rol::where("league_id", $league->id)->first();


        //Obtenemos la categoria
        $category = ($league->get_team_and_category_specification->get_categories)->where("id", $request->category_id)->first();

        //************************************************************
        //************************************ FECHAS TORNEO
        //************************************************************
        $fechas_torneo = array();
        $total_groups_iterator_week = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->groupBy("iterator_week")->orderBy("iterator_week")->get();        

        $mes = ['ENE', 'FEB', 'MAR', 'ABR', 'MAY', 'JUN', 'JUL', 'AGO', 'SEP', 'OCT', 'NOV', 'DIC'];

        foreach ($total_groups_iterator_week as $week_game) {

            $date_game = Carbon::createFromFormat("Y-m-d", $week_game->date)->endOfDay();

            $date = $mes[$date_game->format("n") - 1] . " " . $date_game->format("d") . " " . $date_game->format("y");

            $fechas_torneo[] = array("week" => $week_game->iterator_week + 1, "date_formatted" => $date, "date_object" => $date_game->format("Y-m-d"), "type" => $week_game->type, "active" => false, "carbon_date" => $date_game);
        }


        //Colocamos la fecha activa, colocaremos como activa la primera mayor a la fecha actual
        //$fechas_torneo[2]["active"] = true;
        $now = Carbon::now()->startOfDay();
        $date_already_selected = false;
        foreach ($fechas_torneo as $key => $value) {
            if ($value["carbon_date"]->greaterThanOrEqualTo($now) && !$date_already_selected) {
                $fechas_torneo[$key]["active"] = true;
                $date_already_selected = true;
            }
        }


        return response()->json($fechas_torneo);
    }


    public function get_games_by_date(Request $request)
    {

        //league_id
        //category_id
        //date_object

        if( $request->type == "festivo" ){
            $response["status"] = false;
            $response["mensaje"] = "Este día esta registrado como festivo.";
            return response()->json($response);
        }

        $response = array("game_available" => true, "teams_rivality" => "", "message" => "");        

        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        $rol_data = Rol::where("league_id", $league->id)->first();

        /******* Obtener los equipos a los que pertenece el usuario *******/
        //Obtenemos el listado de los equipos en los que se enuentra el usuario quien realiza la petición a este webserice, esto para colocar las opciones de agregar jugadores o colocar la puntuación del juego
        $team_id_own_user = array();
        foreach ($league->get_team_and_category_specification->get_categories as $category) {
            foreach ($category->get_team_each_category as $team) {
                foreach ($team->get_player_in_team_each_category as $players) {
                    if ($players->player_id == $request->player_object->id) {
                        $team_id_own_user[] = $team->team_id;
                    }
                }
            }
        }
        

        //Revisamos si es fecha y hora del partido para poder habilitar la pantalla
        $now = Carbon::now();
        $date_object = Carbon::createFromFormat("Y-m-d H:i:s", $request->date_object . " " . $league->hour_start);
        $mes = ['ENE', 'FEB', 'MAR', 'ABR', 'MAY', 'JUN', 'JUL', 'AGO', 'SEP', 'OCT', 'NOV', 'DIC'];

        //Obtenemos la categoria
        $category = $league->get_team_and_category_specification->get_categories->where("id", $request->category_id)->first();    

        //************************************************************
        //************************************ FECHAS TORNEO
        //************************************************************
        $games_by_date = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)->where("date", $request->date_object)->where("type", $request->type)->get();              

        $games = array();
        foreach ($games_by_date as $rivality) {

            $row = array("team_1" => null, "team_2" => null);   

            $r_t_session_first = RolTournamentSession::where( "rol_tournament_id", $rivality->id )->where("team_id", $rivality->first_team_id)->sum("win");
            
            //Equipo 1
            $team = Team::find($rivality->first_team_id);

            $all_weeks_first = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)
                                    // ->where("iterator_week", "<=",$request->week)
                                    // ->where("type", "!=", "ronda_posiciones_virtual")->where("suspended", 0)
                                    ->where("first_team_id", $team->id)
                                    // ->where("first_team_suspended", 0)                                                                        
                                    ->pluck("id")->toArray();

            $all_weeks_second = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)
                                    // ->where("iterator_week", "<=",$request->week)
                                    // ->where("type", "!=", "ronda_posiciones_virtual")->where("suspended", 0)
                                    ->where("second_team_id", $team->id)
                                    // ->where("second_team_suspended", 0)                                                                        
                                    ->pluck("id")->toArray();                                    

            
            $all_weeks = array_merge($all_weeks_first, $all_weeks_second);
                                      
            $rol_sesion_win  = RolTournamentSession::whereIn("rol_tournament_id", $all_weeks)->where("team_id",$team->id)->where("win",1)->count(); 
            $rol_sesion_lost = RolTournamentSession::whereIn("rol_tournament_id", $all_weeks)->where("team_id",$team->id)->where("win",0)->count();                                                              

            $serie_win_first = RolTournament::whereIn("id", $all_weeks)->where("first_team_id", $team->id)
                                    ->where("first_team_win_serie",">=",1)->whereNotNull("first_team_win_serie")->sum("first_team_win_serie");

            $serie_win_second = RolTournament::whereIn("id", $all_weeks)->where("second_team_id", $team->id)                                    
                                    ->where("second_team_win_serie",">=",1)->whereNotNull("second_team_win_serie")->sum("second_team_win_serie");


            $serie_win_first_lost = RolTournament::whereIn("id", $all_weeks)->where("first_team_id", $team->id)
                                    ->where("first_team_win_serie",0)->whereNotNull("first_team_win_serie")->count();

            $serie_win_second_lost = RolTournament::whereIn("id", $all_weeks)->where("second_team_id", $team->id)                                    
                                    ->where("second_team_win_serie",0)->whereNotNull("second_team_win_serie")->count();                                    


            // error_log( "rol_tournamentSession_menor => ". $rol_sesion_win ."-". $rol_sesion_lost );                                                             
            $conteo_ganados  = 0;
            $conteo_perdidos = 0;
            

            $row_team = array();
            $row_team["table_number"] = $rivality->first_table_number;
            $row_team["logo_team"] = ( $team->image_path != null ) ? url($team->image_path) : url("assets/images/team.svg");
            $row_team["team_name"] = $team->name;
            $row_team["puntaje"] = 0;
            $row_team["team_id"] = $team->id;
            $row_team["rol_tournament_id"] = $rivality->id;
            $row_team["active_click"] = false; // Activo para asignar jugadores o puntaje de partidos
            $row_team["date_object"] = $request->date_object;
            $row_team["status"] = "";      
            $row_team["r_t_session_first"] = $r_t_session_first + $rivality->first_team_win_serie;            
            $row_team["no_asisitio"] = $rivality->first_team_suspended;                   
            $row_team["ganados"]  = $rol_sesion_win + $serie_win_first + $serie_win_second;
            $row_team["perdidos"] = $rol_sesion_lost + $serie_win_first_lost + $serie_win_second_lost;
            $row["team_1"] = $row_team; 


            $r_t_session_second = RolTournamentSession::where( "rol_tournament_id", $rivality->id )->where("team_id", $rivality->second_team_id)->sum("win");
            //Equipo 2
            $team = Team::find($rivality->second_team_id);            

            $all_weeks_first = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)
                                ->where("iterator_week", "<=",$request->week)
                                // ->where("type", "!=", "ronda_posiciones_virtual")
                                // ->where("suspended", 0)
                                ->where("first_team_id", $team->id)
                                ->where("first_team_suspended", 0)                                                                        
                                ->pluck("id")->toArray();

            $all_weeks_second = RolTournament::where("rol_id", $rol_data->id)->where("category_id", $category->id)
                        ->where("iterator_week", "<=",$request->week)
                        // ->where("type", "!=", "ronda_posiciones_virtual")
                        // ->where("suspended", 0)
                        ->where("second_team_id", $team->id)
                        ->where("second_team_suspended", 0)                                                                        
                        ->pluck("id")->toArray();                                    


            $all_weeks = array_merge($all_weeks_first, $all_weeks_second);
                        
            $rol_sesion_win  = RolTournamentSession::whereIn("rol_tournament_id", $all_weeks)->where("team_id",$team->id)->where("win",1)->count(); 
            $rol_sesion_lost = RolTournamentSession::whereIn("rol_tournament_id", $all_weeks)->where("team_id",$team->id)->where("win",0)->count();                                                              

            $serie_win_first = RolTournament::whereIn("id", $all_weeks)->where("first_team_id", $team->id)
                        ->where("first_team_win_serie",">=",1)->whereNotNull("first_team_win_serie")->sum("first_team_win_serie");

            $serie_win_second = RolTournament::whereIn("id", $all_weeks)->where("second_team_id", $team->id)                                    
                        ->where("second_team_win_serie",">=",1)->whereNotNull("second_team_win_serie")->sum("second_team_win_serie");


            $serie_win_first_lost = RolTournament::whereIn("id", $all_weeks)->where("first_team_id", $team->id)
                        ->where("first_team_win_serie",0)->whereNotNull("first_team_win_serie")->count();

            $serie_win_second_lost = RolTournament::whereIn("id", $all_weeks)->where("second_team_id", $team->id)                                    
                        ->where("second_team_win_serie",0)->whereNotNull("second_team_win_serie")->count();                                    

                  
            $conteo_ganados = 0;
            $conteo_perdidos = 0;

            // if( $team->id == 34 ){
            //     error_log(json_encode( $juegos_gnados_perdidos ));
            // }
         
            $row_team = array();
            $row_team["table_number"] = $rivality->second_table_number;
            $row_team["logo_team"] = ($team->image_path != null) ? url($team->image_path) : url("assets/images/team.svg");
            $row_team["team_name"] = $team->name;
            $row_team["puntaje"] = 0;
            $row_team["team_id"] = $team->id;
            $row_team["rol_tournament_id"] = $rivality->id;
            $row_team["active_click"] = false; // Activo para asignar jugadores o puntaje de partidos
            $row_team["date_object"] = $request->date_object;
            $row_team["status"] = "";
            $row_team["r_t_session_second"] = $r_t_session_second + $rivality->second_team_win_serie;;            
            $row_team["no_asisitio"] = $rivality->second_team_suspended;         
            $row_team["ganados"]  =  $rol_sesion_win + $serie_win_first + $serie_win_second;
            $row_team["perdidos"] = $rol_sesion_lost + $serie_win_first_lost + $serie_win_second_lost;;  
            $row["team_2"] = $row_team;       
            //Verificamos a que equipo pertenece el usuario, si al 1 o 2 para definir status
            //Los estatus que manejaremos son:
            //"" => Ningun status, ya que el usuario no pertenece al equipo o no se permite ninguna otra funcionalidad
            //asignar_jugadores => Estatus que permitirá abrir la pantalla para seleccionar los jugadores.
            //puntaje_juego => Estatus que permitirá abrir la pantalla para ingresar el el puntaje por cada jugador seleccionado en asignar_jugadores Todo: * Falta definir esta accion

            //Revisamos que estatus colocaremos si "asignar_jugadores" o "puntaje_juego"
            //* asignar_jugadores: se detonará cuando la fecha y hora del juego no esta despues de la fecha y hora actual.
            // En caso de que la fecha y hora del juego es mayor a la actual pero no han seleccionado ningun jugador tambén estará activo.
            //puntaje_juego: se detonará cuando la fecha y hora del partido es mayor o igual a la actual y ya seleccionador los jugadores que jugarán. En esta opción permitirá al equipo ingresar el puntaje del juego.
            //valida_puntaje: será el equipo que validará el puntaje del juego
            //ver_puntaje: Significa que el juego ya finalizó y podemos ver el puntaje de ambos equipos
            $response["editar_alineacion"] = false;

            if (in_array($row["team_1"]["team_id"], $team_id_own_user)) {

                //Verificamos si el partido ya finalizo
                $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $row["team_1"]["rol_tournament_id"])
                                                                ->where("validate", 1)->groupBy("session_number")->get()->toArray();
                

                if (count($rol_tournament_sessions) >= $league["get_player_types"]->number_game_by_session) {

                    $row["team_1"]["status"]       = "ver_puntaje";
                    $row["team_1"]["active_click"] = true;
                    $row["team_1"]["termiado"]     = true;

                } 
                //$now->lessThan($date_object)
                else if ( false ) { //Si la fecha actual es menor a la fecha y hora del partido, entonces pueden seleccionar los jugadores                    

                    $row["team_1"]["status"] = "asignar_jugadores";
                    // $row["team_1"]["status"] = "puntaje_juego";
                    $row["team_1"]["status_fuera_horario"] = true;
                    $row["team_1"]["active_click"] = true;                    

                }else {                    

                    //Si la fecha actual es mayor o igual a la fehca y hora del partido, significa que ahora mostraremos la opción de agregar el puntaje, en caso que el equipo no tenga jugadores se habilitará
                    //la opción de volver a seleccionar los jugadores.
                    //Verificamos que jugadores se encuentran seleccionados
                    
                    // ->where("week", $request->week)
                    // error_log("weeek ==========>". $rivality->iterator_week );
                    $players_in_game = PlayersInGame::where("league_id", $request->league_id)->where("week", $rivality->iterator_week)
                                       ->where("category_id", $request->category_id)->where("team_id", $row["team_1"]["team_id"])->count();
                    
                    // error_log("players_in_game ==> ".$players_in_game);

                    if ($players_in_game > 0) {

                        //Si el equipo contrario es equipo DUMMY, entonces siempre será puntaje de juego ya que no habra quien valide el puntaje                        

                        if ($row["team_2"]["team_name"] == "DUMMY") {
                            $response["game_available"] = true;
                            $row["team_1"]["status"] = "puntaje_juego";
                            $row["team_1"]["active_click"] = true;
                        } else {


                            error_log("primeri => ". $rivality->first_team_suspended);

                            // error_log("league_id =>".  $request->league_id);
                            // error_log("category_id =>".  $request->category_id);
                            // error_log("team =>".  $row["team_1"]["team_id"]);

                            // error_log( json_encode($team_id_own_user) );                            
                                //Si ya hay jugadores registrados, colocamos la opcion de agregar puntaje
                            if( $rivality->second_team_suspended == 0 ){

                                $response["game_available"] = true;                                
                                $row["team_1"]["status"] = ($row["team_1"]["table_number"] % 2 != 0) ? "puntaje_juego" : "valida_puntaje";                                
                                $row["team_1"]["active_click"] = true;

                            }else{
                                $response["game_available"] = true;                                
                                $row["team_1"]["status"] = "puntaje_juego";//($row["team_1"]["table_number"] % 2 != 0) ? "puntaje_juego" : "valida_puntaje";                                
                                $row["team_1"]["active_click"] = true;
                                $row["team_1"]["termiado"] = false;
                            }
                            
                            
                        }
                        

                    } else {

                        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $row["team_1"]["rol_tournament_id"])
                                                                ->where("validate", 1)->groupBy("session_number")->get()->toArray();
                
                        error_log( json_encode($rol_tournament_sessions) );                                                                

                        //No hay jugadores registrados, colocamos la opción de seleccionar jugadores
                        $response["game_available"] = true;
                        $row["team_1"]["status"] = "asignar_jugadores";
                        $row["team_1"]["active_click"] = true;

                        // error_log("ERROR LOGGGGG");

                    }

                }

            } else if (in_array($row["team_2"]["team_id"], $team_id_own_user)) {

                //Verificamos si el partido ya finalizo
                $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $row["team_1"]["rol_tournament_id"])->where("validate", 1)->groupBy("session_number")->get()->toArray();

                

                if (count($rol_tournament_sessions) >= $league["get_player_types"]->number_game_by_session) {

                    $row["team_2"]["status"] = "ver_puntaje";
                    $row["team_2"]["active_click"] = true;
                    $row["team_2"]["termiado"] = true;
                
                //$now->lessThan($date_object)
                }else if (false) { //Si la fecha actual es menor a la fecha y hora del partido, entonces pueden seleccionar los jugadores

                    $row["team_2"]["status"] = "asignar_jugadores";
                    // $row["team_2"]["status"] = "puntaje_juego";
                    $row["team_2"]["status_fuera_horario"] = true;
                    $row["team_2"]["active_click"] = true;
                    

                }else {
                    

                    // error_log( json_encode($team_id_own_user) );
                    //Si la fecha actual es mayor o igual a la fehca y hora del partido, significa que ahora mostraremos la opción de agregar el puntaje, en caso que el equipo no tenga jugadores se habilitará
                    //la opción de volver a seleccionar los jugadores.
                    //Verificamos que jugadores se encuentran seleccionados
                    $players_in_game = PlayersInGame::where("league_id", $request->league_id)->where("week", $rivality->iterator_week)
                                       ->where("category_id", $request->category_id)
                                       ->where("team_id", $row["team_2"]["team_id"])->count();
                    
                    // error_log("Jugadores en el juego => ". $players_in_game);
                    // error_log("Players in game =>". $players_in_game);

                    if ($players_in_game > 0) {
                        
                        //Si el equipo contrario es equipo DUMMY, entonces siempre será puntaje de juego ya que no habra quien valide el puntaje
                        if ($row["team_1"]["team_name"] == "DUMMY") {
                            $response["game_available"] = true;
                            $row["team_2"]["status"] = "puntaje_juego";
                            $row["team_2"]["active_click"] = true;
                        } else {                            
                            //Si ya hay jugadores registrados, colocamos la opcion de agregar puntaje                            
                            //despues de que se añaden jugadores revisar si es un equipo suspendido para poder cambiar a puntaje de juego

                            if( $rivality->first_team_suspended == 0 ){
                                
                                $response["game_available"] = true;
                                $row["team_2"]["status"] = ($row["team_2"]["table_number"] % 2 != 0) ? "puntaje_juego" : 'ver_puntaje' ;//"valida_puntaje";
                                // $row["team_2"]["status"] = ($row["team_2"]["table_number"] % 2 != 0) ? "puntaje_juego" : "puntaje_juego";
                                $row["team_2"]["active_click"] = true;
                                $response["editar_alineacion"] = ($row["team_2"]["table_number"] % 2 != 0) ? null : true ;

                            }else{
                                

                                $response["game_available"] = true;
                                $row["team_2"]["status"] ="puntaje_juego"; //;($row["team_2"]["table_number"] % 2 != 0) ? "puntaje_juego" : 'ver_puntaje' ;//"valida_puntaje";                                
                                $row["team_2"]["active_click"] = true;
                                $row["team_2"]["termiado"] = false;                                

                            }
                            
                        }

                    } else {

                        //No hay jugadores registrados, colocamos la opción de seleccionar jugadores
                        $response["game_available"] = true;
                        $row["team_2"]["status"] = "asignar_jugadores";
                        $row["team_2"]["active_click"] = true;

                    }

                }

            }

            //si el primer equipo esta suspendido en el juego
            if($rivality->first_team_suspended == 1){
                $response["game_available"] = true;
                $row["team_1"]["status"] = "";
                $row["team_1"]["active_click"] = true;
            }            

            if($rivality->second_team_suspended == 1){
                $response["game_available"] = true;
                $row["team_2"]["status"] = "";
                $row["team_2"]["active_click"] = true;
            }

            if($rivality->suspended == 1){                
                $response["game_available"] = true;
                $row["team_2"]["status"] = "";
                $row["team_2"]["active_click"] = true;
                
                $row["team_1"]["status"] = "";
                $row["team_1"]["active_click"] = true;
            }

            $row["hour_start"] = Carbon::createFromFormat("H:i:s", $league->hour_start)->format("H:i");
            // $row["team_2"]["status"] = "asignar_jugadores";

            $games[] = $row;

        }

        // error_log( json_encode($games) );

        $response["teams_rivality"] = $games;        
        $response["games_by_date"] = $games_by_date;

        return response()->json($response);
    }


    private function type_player( $genero ){

        if( $genero == "male" ){
            return 1;
        }

        if( $genero == "female" ){
            return 2;
        }

        if( $genero == "female" ){
            return 2;
        }

        if( $genero == "special" ){
            return 3;
        }        
        

    }   

    public function select_players_in_game(Request $request)
    {

        //league_id
        //category_id
        //date_object
        //team_id
        //rol_tournament_id

        $response = array("message" => "");
        error_log("category_id ID ===>". json_encode( $request->category_id ));
        //Revisamos si el usuario pertenece al equipo seleccionado * Actualización, se comentaron estas lineas ya no son necesarias.
        //$category = TeamCategory::where("id", $request->category_id)->whereHas("get_team_each_category", function ($team_each_category) use ($request) {
        //    return $team_each_category->where("team_id", $request->team_id)->whereHas("get_player_in_team_each_category", function ($player_in_team_each_categories) use ($request) {
        //        return $player_in_team_each_categories->where("player_id", $request->player_object->id);
        //    });
        //})->count();

        //if ($category == 1) {

        //Si pertenece al equipo seleccionado, mandamos los parametros para seleccionar los jugadores

        //Verificamos el maximo jugadores permitidos en la liga
        $total_jugadores = 0;
        $league = League::find($request->league_id);
        $category_rules = CategoryRule::where("league_id", $request->league_id)->first();
        $total_jugadores = $category_rules->maximum_players_by_team;

        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                                ->where("validate", 1)->groupBy("session_number")->get()->toArray();

        $rol_torunament = RolTournament::find($request->rol_tournament_id);                                                                

        $response["num_juego"] = count( $rol_tournament_sessions ) + 1; //esta basado en 0
        error_log("numero de semana ". count( $rol_tournament_sessions ));                                                                        

        // error_log( "CategoryRule => ". $category_rules );

        $response["maximo_jugadores_permitidos"] = $total_jugadores;

        $total_dummy_players_available = Handicap::where("league_id", $request->league_id)->first();
        $total_dummy_players_available = $total_dummy_players_available->dummy_by_team;        

        //Obtenemos los jugadores del equipo
        $players = array();
        
        $team_each_categories = TeamEachCategory::where("team_category_id", $request->category_id)->where("team_id", $request->team_id)->first();
        $players_in_team_category = $team_each_categories->get_player_in_team_each_category;
        
        

        foreach ($players_in_team_category as $player_in_team_category) {            
            // $player = Player::find( $player_in_team_category->id );
            $player_in_team_category = $player_in_team_category->get_player;            
            error_log( json_encode($player_in_team_category->gender));
            $players[] = array("id" => $player_in_team_category->id, "type_player" => $this->type_player( $player_in_team_category->gender ), "player_number" => "", "selected" => false, "nickname" => $player_in_team_category->nickname, "name" => $player_in_team_category->name . " " . $player_in_team_category->last_name, "gender" => $player_in_team_category->gender, "special" => $player_in_team_category->special, "birthday" => $player_in_team_category->birthday );                        
        }

        if( $total_dummy_players_available > 0 ){

            for ($i = 1; $i <= $total_dummy_players_available; $i++) {
                        
                $player = new Player();
                $player->id = ($i * -1);
                $player->name = "DUMMY ".$i;

                $rol_torunament = RolTournament::find( $request->rol_tournament_id );
                $player_in_game = PlayersInGame::where("league_id", $request->league_id)->where("category_id", $request->category_id)->where("team_id", $request->team_id)->where("player_id", $player->id)->where("week", $rol_torunament->iterator_week)->first();

                $type_player = 0;
                if( $player_in_game ){
                    $type_player = $player_in_game->type_player;
                }

                $dummy = new PlayerInTeamEachCategory();
                //"id" => ,
                //"team_each_categories_id" => 0,
                $dummy->player_id = ($i * -1);
                $dummy->player_handy_cap_in_tournament = 0;
                $dummy->get_player = $player;
                // $dummy->get_player = $player;

                $players[] = array("id" => $player->id, "player_number" => "", "selected" => false, "name" => $player->name, "nickname" => $player->nickname, "type_player" => $type_player );                

            }

        }


        //Verificamos que jugadores ya se encuentran registrados para el partido
        foreach ($players as $key => $value) {
            $player_in_game = PlayersInGame::where("league_id", $request->league_id)->where("week",$rol_torunament->iterator_week)->where("category_id", $request->category_id)->where("team_id", $request->team_id)->where("player_id", $value["id"])->first();
            // error_log( $player_in_game );

            if ($player_in_game != null) {
                $players[$key]["selected"] = true;
                $players[$key]["player_number"] = $player_in_game->player_number;
                
            }

        }


        $response["players"] = $players;


        return response()->json($response);

    }

    // , $num, $team_id 
    private function validaDummyReglaEspecial( $equipos_dummys, $reglas){

        $female_temp = 0;
        $resp["status"] = false;

        // error_log($reglas->number_of_female .">". $equipos_dummys["female"]);
        error_log("equipos_dummys => ". count($equipos_dummys["dummys"]));
        if( count($equipos_dummys["dummys"]) === 0 ){
            // error_log("NO HAY DUMMYS");          
            return $resp["status"] = false;
        }        
        
        error_log( "====>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>><". $equipos_dummys["male"] .">". $reglas->number_of_male);
        if( $equipos_dummys["male"] > $reglas->number_of_male  ){
            error_log( "equipos_dummys ==>>". print_r($equipos_dummys, true) );
            return $resp["status"] = false;
        }


        if( $reglas->number_of_female > $equipos_dummys["female"] ){

            $female_temp = $female_temp + $equipos_dummys["female"];
            // $equipos_dummys["dummy"]
            // error_log( print_r( $equipos_dummys["dummys"] ,true ) );
            foreach ($equipos_dummys["dummys"] as $key => $value) {
                if( $value["gender"] === 2 ){
                    $female_temp = $female_temp + 1;
                }
            }                        
            
            // error_log($female_temp. "!==".  $reglas->number_of_female);
            if( $female_temp !==  $reglas->number_of_female ){
                // $team = Team::find($team_id);
                // error_log("numero de jugadores ". $team_id);
                $resp["status"] = true;
                $resp["gender"] = "Alineación incorrecta, favor de revisar el juego ";
                // . $num. " del equipo ". $team->name;
                
            }            

        }
        
        return $resp; 

    }

    private function jugador_jugadores( $num ){
        if( $num > 1 ){
            return $num." jugadores";
        }

        return $num." jugador";
    }

    public function save_players_in_game(Request $request)
    {

        $response = array();
        $response["message"] = "";

        //league_id
        //category_id
        //date_object
        //team_id
        //rol_tournament_id
        //players

        DB::beginTransaction();

        //dd($request->all());
        $alineacion_indevida = false;
        
        //Primero limpiamos el request de los jugadores recibidos ya que devuelve un array con el total de jugadores pero no todos tienen valor
        $total_players_received = 0;
        foreach ($request->players as $key => $player) {
            if (is_array($player)) {
                $total_players_received++;
            }
        }

        // error_log( json_encode( $request->all() ) );

        $player_types = PlayerType::where("league_id", $request->league_id)->first();

        //SE REVISA EL EQUIPO CONTRINCANTE PARA ENVIAR UNA NOTIFICACION
        $rol_torunament = RolTournament::find( $request->rol_tournament_id );

        if ($total_players_received > $player_types->type_game) {
            $response["message"] = "El limite de jugadores es " . $player_types->type_game;
            return response()->json($response, 504);
        }

        if ($total_players_received < $player_types->type_game) {

            $response["message"] = "Selecciona al menos " . $player_types->type_game . " jugadores.";
            return response()->json($response, 504);

        } else {

            //ELIMINAMOS LOS JUGADORS CADA QUE SE INGRESAN NUEVOS PARA QUE NO PERMANESCA BASURA

            PlayersInGame::where("league_id", $request->league_id)->where("category_id", $request->category_id)
                        ->where("team_id", $request->team_id)
                        ->delete();

            //Almacenamos los jugadores nuevos
            $order = 1;
            $players_added_from_app = array();

            $female = 0;
            $male   = 0;   
            $dummy  = array();
            $penalties_join = array();
            
            // error_log( "check_pen ========>" . json_encode( $check_dummy_regla ) );
            
            $check_pen = $this->checkPenalizacion($request->players, $player_types);

            $check_dummy_regla = $this->validaDummyReglaEspecial( $check_pen, $player_types); //$num, $team_id

            error_log("Chacando penalizacion => ". json_encode($check_dummy_regla) );

            if( $player_types->there_are_special_rule_with_the_teams == 1 ){ 
                
                //esta regla es en caso de seleccionar un dummys y no cumplan 
                //ejemplo hay 2 hombres 1 mujer y el dummy debe ser mujer, lo que ara es indicar que añada un dummy demujer
                if( $check_dummy_regla["status"] == true ){
    
                    $response["status"]  = false;
                    $response["message"] = $check_dummy_regla["gender"];                    
                    return response()->json($response);
    
                }else{
                    
                    DummyPenalty::where("team_id", $request->team_id)->where("week", $rol_torunament->iterator_week)
                                ->where("league_id", $request->league_id)->where("category_id", $request->category_id)
                                ->where("sesion", $request->num_juego )
                                ->delete();
    
                }
                
                
                array_push($penalties_join, $check_pen);

            }

            // error_log("Error =====>". print_r($penalties_join, true) );

            if( $player_types->there_are_special_rule_with_the_teams == 1 ){ 

                Penalties::where("team_id", $request->team_id)->where("week",$rol_torunament->iterator_week)                
                        ->where("league_id", $request->league_id )->where("sesion", $request->num_juego )->delete(); 
                                

                foreach ($penalties_join as $key => $penal) {

                    if( $player_types->number_of_female > $penal["female"] || $player_types->number_of_male < $penal["male"] && 
                        $penal["female"] < $player_types->number_of_female ){

                        $penalties = new Penalties();
                
                        $penalties->player_id = 0;
                        // $penalties->player_id = $player->id;
                        $penalties->league_id = $request->league_id;
                        $penalties->week      = $rol_torunament->iterator_week;
                        $penalties->team_id   = $request->team_id;//$penal["team_id"];
                        $penalties->sesion    = $request->num_juego; // 
                        $penalties->rol_tournament_id = $request->rol_tournament_id;
                        $penalties->category_id = $request->category_id;
                        $penalties->save(); 
                        
                        $alineacion_indevida = true;
                            // error_log( $rol_torunament->iterator_week ."--". $request->team_id);

                        foreach ($request->players as $player) {
                            
                            // $penalties = new Penalties();
                            
                            // $penalties->player_id = 0;
                            // // $penalties->player_id = $player->id;
                            // $penalties->league_id = $request->league_id;
                            // $penalties->week      = $rol_torunament->iterator_week;
                            // $penalties->team_id   = $request->team_id;//$penal["team_id"];
                            // $penalties->sesion    = $request->num_juego; // 
                            // $penalties->rol_tournament_id = $request->rol_tournament_id;
                            // $penalties->category_id = $request->category_id;
                            // $penalties->save();  
                            $alineacion_indevida = true;

                        }

                        foreach ($check_pen["dummys"] as $key => $value) {
                            $dummy_penalty = new DummyPenalty();
                            $dummy_penalty->league_id = $request->league_id;
                            $dummy_penalty->team_id   = $request->team_id;
                            $dummy_penalty->player_id = $value["key"];
                            $dummy_penalty->week      = $rol_torunament->iterator_week;
                            $dummy_penalty->sesion    = $request->num_juego;
                            $dummy_penalty->gender    = $value["gender"];  
                            $dummy_penalty->category_id = $request->category_id;                                 
                            $dummy_penalty->save();
                        }      

                    }

                }   

            }


            foreach ($request->players as $player) {
                
                //Verificamos si tiene información del jugador a asignar
                error_log(json_encode($player));
                if (is_array($player)) {
                    // error_log(print_r( $player, true ));
                    //Revisamos si ya existe, si ya existe solo lo reemplazamos si no, lo agregamos como un registro nuevo
                    $already_exist_player = PlayersInGame::where("league_id", $request->league_id)->where("category_id", $request->category_id)->where("team_id", $request->team_id)->where("player_id", $player["id"]);
                    if ($already_exist_player->count() == 1) {

                        //Ya existe
                        $players_in_games = $already_exist_player->first();
                        $players_in_games->player_number = $order;
                        $players_in_games->player_id = $player["id"];
                        $players_in_games->week = $rol_torunament->iterator_week;
                        $players_in_games->type_player = $player["type_player"];
                        $players_in_games->save();

                    } else {                                            

                        //Es nuevo
                        $players_in_games = new PlayersInGame();
                        $players_in_games->league_id = $request->league_id;
                        $players_in_games->category_id = $request->category_id;
                        $players_in_games->team_id = $request->team_id;
                        $players_in_games->player_number = $order;
                        $players_in_games->player_id = $player["id"];
                        $players_in_games->week = $rol_torunament->iterator_week;
                        $players_in_games->type_player = $player["type_player"];
                        $players_in_games->save();

                    }

                    $players_added_from_app[] = $player["id"];

                    $order++;
                }

            }


            //CUANDO SE MENEJAN EJEMPLO 2 HOMBRES Y 2 MUJERES Y SE TIENE QUE PENELIZAR
            //gender 1 = hombre | 2 = mujer | 3 = juvenil | 4 = especial
            if( $player_types->there_are_special_rule_with_the_teams == 1 ){      
                // Penalties::where("team_id", $request->team_id)
                //          ->where("week",$rol_torunament->iterator_week)                
                //          ->where("league_id", $request->league_id)->delete(); 
                // error_log("SE ELIMINA =======>");
            }
        
            
        $team_notify = null;
        $team_name   = null;

        $team_name = Team::find( $request->team_id );

        if( $rol_torunament->first_team_id == $request->team_id ){
            $team_notify = $rol_torunament->second_team_id;
        }

        if( $rol_torunament->second_team_id == $request->team_id ){
            $team_notify = $rol_torunament->first_team_id;
            
        }

        $team_each_categories = TeamEachCategory::where("team_category_id", $request->category_id)->where("team_id", $request->team_id)->first();
        $players_in_team_category = $team_each_categories->get_player_in_team_each_category;
        

        foreach ($players_in_team_category as $player_in_team_category) {
            
            $player_in_team_category = $player_in_team_category->get_player;

            if( $player_in_team_category->token_notification ){
                
                $response["RESPONSE"]["UUID"] = $player_in_team_category->token_notification;
                $response["RESPONSE"]["TEXTO_NOTIFICACION"] = "El equipo ".$team_name->name." asigno a sus jugadores.";
                $response["RESPONSE"]["TITLE"] = "Jugadores asignados";

                Pastora::sendPushNotification($response);

            }
            // error_log("player_in_team_category ==> ". $player_in_team_category->nickname);            
        }
            


        //Eliminamos los jugadores que ya teniamos registrados pero que tal vez se eliminaron en la app
        //    $players_already_registered = PlayersInGame::where("league_id", $request->league_id)->where("category_id", $request->category_id)->where("team_id", $request->team_id)->get();
        //    foreach ($players_already_registered as $player_registered){
        //        if(!in_array($player_registered->player_id, $players_added_from_app)){
        //            $player_registered->delete();
        //        }
        //    }

            $player_types = PlayerType::where("league_id", $request->league_id)->first();                        

            DB::commit();
            $response["status"] = true;
            $response["message"] = $alineacion_indevida ? "Los jugadores se han almacenado correctamente pero existe una alineación indebida por lo cual será penalizado con ".$player_types->penalty_pins. " pinos." : "Los jugadores se han almacenado correctamente.";
            return response()->json($response);

        }

    }


    public function delete_player_in_game(Request $request)
    {

        $response = array();
        $response["message"] = "";

        // $request->rol_tournament_id
        $rol_torunament = RolTournament::find( $request->rol_tournament_id );

        $delete_player_in_juego = PlayersInGame::where("league_id", $request->league_id)->where("category_id", $request->category_id)->where("team_id", $request->team_id)->where("player_id", $request->player_id)->where("week", $rol_torunament->iterator_week);

        $get_del_player_in_game = $delete_player_in_juego->count();

        if( $get_del_player_in_game === 0 ){
            $response["status"] = false;
            return response()->json($response, 200);
        }

        if ($delete_player_in_juego->delete()) {

            //Despues de eliminar el jugador, vamor a reordenar los jugadores
            $order = 1;
            $players_in_game = PlayersInGame::where("league_id", $request->league_id)->where("category_id", $request->category_id)->where("team_id", $request->team_id)->orderBy("player_number", "ASC")->get();
            foreach ($players_in_game as $player_in_game) {
                $player_in_game->player_number = $order;
                $player_in_game->save();
                $order++;
            }

            $response["message"] = "El jugador se ha eliminado de la lista activa de jugadores.";
            return response()->json($response);

        } else {
            $response["message"] = "No hemos podido eliminar el jugador de la lista, intentalo nuevamente.";
            return response()->json($response, 504);
        }

    }

    private function tipoDummy( $handicap, $type_player ){
        // error_log("type_player ====> ".$type_player);
        // error_log("handycap===> ".$handicap);
        //hombre
        if( $type_player == 1 or $type_player == 0 ){
            return $handicap->male_limit_dummy;
        }

        if( $type_player == 2 ){
            return $handicap->female_limit_dummy;
        }

        if( $type_player == 3 ){
            return $handicap->juvenil_limit_dummy;
        }

        if( $type_player == 4 ){
            return $handicap->especial_limit_dummy;
        }

    }

    public function get_game_in_progress(Request $request)
    {

        $response = array();
        $response["players"] = array();
        $response["total_sesiones"] = "";
        $response["sesion_actual"] = "";
        $response["como_se_juega"] = "";
        $response["week_number"] = "";
        $response["date_formatted"] = "";
        $response["game_finished"] = false;

        $rol_tournament = RolTournament::find($request->rol_tournament_id);
        $rol = Rol::find($rol_tournament->rol_id);
        $league = League::with("get_player_types")->where("id", $rol->league_id)->first();
        
        // $handicaps = Handicap::where("league_id", $rol->league_id)->first();
        $handicap_each_categories = HandicapEachCategory::where("team_category_id", $rol_tournament->category_id)->first();        

        $response["total_sesiones"] = $league->get_player_types["number_game_by_session"];
        $response["como_se_juega"] = $league->get_player_types["type_game"];

        $response["week_number"] = $rol_tournament->iterator_week;
        $response["date_formatted"] = $rol_tournament->date;
        $response["league_id"] = $league->id;

        /*********** Verificamos si ya se termino el juego ***********/
        $total_rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("validate", 1)->groupBy("session_number")->get()->toArray();
        // error_log( json_encode( $total_rol_tournament_sessions ) );
        if (count($total_rol_tournament_sessions) >= $league->get_player_types["number_game_by_session"]) {
            $response["message"] = "La serie de juegos ha finalizado.";
            $response["game_finished"] = true;
            return response()->json($response);
        }


        /*********** Insertamos las sesiones por cada equipo (equipo 1 y equipo 2) de acuerdo a la configuración de la liga, si la liga se compone de 3 sesiones, insertamos 3 registros ***********/
        for ($i = 1; $i <= $league->get_player_types["number_game_by_session"]; $i++) {

            //Verificamos que no exista la sesion para el equipo 1
            $rol_tournament_sessions_exists = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $rol_tournament->first_team_id)->where("session_number", $i);

            if ($rol_tournament_sessions_exists->count() <= 0) {
                $rol_tournament_sessions = new RolTournamentSession();
                $rol_tournament_sessions->rol_tournament_id = $request->rol_tournament_id;
                $rol_tournament_sessions->team_id = $rol_tournament->first_team_id;
                $rol_tournament_sessions->session_number = $i;
                $rol_tournament_sessions->validate = 0;
                $rol_tournament_sessions->save();
            }

            //Verificamos que no exista la sesion para el equipo 2
            $rol_tournament_sessions_exists = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $rol_tournament->second_team_id)->where("session_number", $i);

            if ($rol_tournament_sessions_exists->count() <= 0) {
                $rol_tournament_sessions = new RolTournamentSession();
                $rol_tournament_sessions->rol_tournament_id = $request->rol_tournament_id;
                $rol_tournament_sessions->team_id = $rol_tournament->second_team_id;
                $rol_tournament_sessions->session_number = $i;
                $rol_tournament_sessions->validate = 0;
                $rol_tournament_sessions->save();
            }

        }


        /*********** Verificamos en que sesion nos encontramos J1, J2, J3,... ***********/
        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("validate", 0)->orderBy("session_number", "ASC")->first();
        $response["sesion_actual"] = $rol_tournament_sessions->session_number;
        $response["in_validation"] = $rol_tournament_sessions->in_process_to_validate;


        /*********** Obtenemos lo jugadores de cada equipo ***********/
        //Equipo 1
        $players_in_game = PlayersInGame::where("league_id", $league->id)->where("category_id", $rol_tournament->category_id)
                            ->where("team_id", $rol_tournament->first_team_id)->orderBy("player_number", "ASC")
                            ->with("get_player")->get();

        error_log( "players_in_game". json_encode($players_in_game));

        $team = Team::find($rol_tournament->first_team_id);
        error_log( $request->rol_tournament_id );
        // error_log( $team->id );
        // error_log($rol_tournament_sessions->session_number);
        
        $rol_tournament_session_first_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                            ->where("team_id", $team->id)->where("validate", 0)->where("session_number", $rol_tournament_sessions->session_number)
                                            ->orderBy("session_number", "ASC")->first();
                                            
        error_log( "team =>". $rol_tournament_session_first_team );

        $num_dummy = 0;
        

        foreach ($players_in_game as $key => $player_in_game) {
            
            $rol_tou_sess = array();
            if( !empty($rol_tournament_session_first_team) ){
                $rol_tou_sess = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session_first_team->id)->where("player_id", $player_in_game->player_id)->first();          
            }
            
            if( $players_in_game[$key]["get_player"] !== null ){        
                $players_in_game[$key]["dummy"] = false;    
                
                $players_in_game[$key]["rol_tournament_session_points"] = $rol_tou_sess;
            }else{
                $num_dummy++;
                $players_in_game[$key]["dummy"] = true;
                $players_in_game[$key]["get_player_dummy"] = array( "name" => "DUMMY ".$num_dummy );                
                $players_in_game[$key]["puntaje"] = $this->tipoDummy( $handicap_each_categories, $player_in_game->type_player );
                                
            }
        }

        // error_log( json_encode($players_in_game) );
        // error_log( "get_player => ". json_encode( $players_in_game ) );

        $response["first_team"]["rol_tournament_sessions"] = $rol_tournament_session_first_team;
        $response["first_team"]["players"] = $players_in_game;
        $response["first_team"]["team"] = $team;
        $response["first_team"]["team"]["image_path"] = ($team->image_path != null) ? url($team->image_path) : url("assets/images/team.svg");
        $response["first_team"]["table_number"] = $rol_tournament->first_table_number;


        //Equipo 2
        $players_in_game = PlayersInGame::where("league_id", $league->id)->where("category_id", $rol_tournament->category_id)->where("team_id", $rol_tournament->second_team_id)->orderBy("player_number", "ASC")->with("get_player")->get();

        // error_log( "players_in_game". json_encode($players_in_game));

        $team = Team::find($rol_tournament->second_team_id);
        $rol_tournament_session_second_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $team->id)->where("validate", 0)->where("session_number", $rol_tournament_sessions->session_number)->orderBy("session_number", "ASC")->first();

        $num_dummy = 0;

        foreach ($players_in_game as $key => $player_in_game) {

            $rol_tou_sess = array();
            if( !empty($rol_tournament_session_second_team) ){
                $rol_tou_sess = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session_second_team->id)->where("player_id", $player_in_game->player_id)->first();
            }

            if( $players_in_game[$key]["get_player"] !== null ){
                $players_in_game[$key]["dummy"] = false;
                $players_in_game[$key]["rol_tournament_session_points"] = $rol_tou_sess;
            }else{
                $num_dummy++;
                $players_in_game[$key]["dummy"] = true;
                $players_in_game[$key]["get_player_dummy"] = array( "name" => "DUMMY ".$num_dummy );                                
                $players_in_game[$key]["puntaje"] = $this->tipoDummy( $handicap_each_categories, $player_in_game->type_player );
            }
        }
                

        $response["second_team"]["rol_tournament_sessions"] = $rol_tournament_session_second_team;
        $response["second_team"]["players"] = $players_in_game;
        $response["second_team"]["team"] = $team;
        $response["second_team"]["team"]["image_path"] = ($team->image_path != null) ? url($team->image_path) : url("assets/images/team.svg");
        $response["second_team"]["table_number"] = $rol_tournament->second_table_number;

        $response["rol_tournament"] = $rol_tournament;

        //se añade para validar si existen el numero de jugadores correctos.
        // $player_types = PlayerType::where("league_id", $rol->league_id)->first();

        // $response["type_game"] = $player_types->type_game;
        // error_log( "players_dos ===>". json_encode($players_in_game) );

        return response()->json($response);
    }


    public function validate_players_selected_by_rol_tournamet(Request $request)
    {

        $response["message"] = "";
        $rol_tournament = RolTournament::find($request->rol_tournament_id);
        error_log( $rol_tournament );
        $rol = Rol::find($rol_tournament->rol_id);
        $league = League::with("get_player_types")->where("id", $rol->league_id)->first();

        $players_in_game_first_team = PlayersInGame::where("league_id", $league->id)->where("category_id", $rol_tournament->category_id)
                                                    ->where("team_id", $rol_tournament->first_team_id)->orderBy("player_number", "ASC")
                                                    ->where("week", $rol_tournament->iterator_week)->with("get_player")->count();

        $players_in_game_second_team = PlayersInGame::where("league_id", $league->id)->where("category_id", $rol_tournament->category_id)
                                                    ->where("team_id", $rol_tournament->second_team_id)->orderBy("player_number", "ASC")
                                                    ->where("week", $rol_tournament->iterator_week)->with("get_player")->count();

        if( $rol_tournament->first_team_suspended == 0 && $rol_tournament->second_team_suspended == 0 ){

            //Validamos que ambos equipos hallan seleccionado sus jugadores
            if ($players_in_game_first_team <= 0 ) {
    
                $team = Team::find($rol_tournament->first_team_id);
    
                if ($team->name != "DUMMY") {
                    $response["message"] = "El equipo '" . $team->name . "' no ha seleccionado los jugadores que participaran.";
                    return response()->json($response, 504);
                }
    
            } else if ($players_in_game_second_team <= 0) {
    
                $team = Team::find($rol_tournament->second_team_id);
    
                if ($team->name != "DUMMY") {
                    $response["message"] = "El equipo '" . $team->name . "' no ha seleccionado los jugadores que participaran.";
                    return response()->json($response, 504);
                }
    
            }

        }


        return response()->json($response);
    }

    public function contactos( Request $request ){

        $response["status"] = false;

        $contact_league = LeagueContact::where("league_id", $request->league_id)->get();   

        if( count($contact_league) > 0 ){
            $response["status"] = true;
            $response["data"] = $contact_league;
        }else{
            $response["message"] = "No hay contactos asignados";
        }
        
        return response()->json($response);
    }

    public function get_contacto_nip( Request $request ){

        $response["status"] = false;

        $contacto = LeagueContact::where("league_id", $request->league_id)->where("nip", $request->nip)
                                ->where("email", $request->email)->first(); 
                                
        if( !empty( $contacto ) ){
            $response["status"] = true;         
        }else{
            $response["mensaje"] = "El nip es incorrecto, favor de verificar.";
        }   

        return response()->json($response);
    }

    private function partido_suspendido( $request, $rol_tournament, $no_asisitio ){

        return;

    }


    public function penalties( Request $request ){

        $response = array();

        $rol_t = RolTournament::find(  $request->rol_tournament_id );

        $penalties = Penalties::where("league_id", $request->id_liga)->where("category_id", $request->category_id)
                   ->where("week", $rol_t->iterator_week)->get();

        // error_log("penlties => ". print_r( $penalties, true ) );
        $dummy_penalty = DummyPenalty::where("week", $rol_t->iterator_week)->where("category_id", $request->category_id)
                                     ->where("league_id", $request->id_liga)->get();
        
        $penalties_push = array(); 

        
        foreach ($penalties as $key => $value) {
                
            $rol_t_s = RolTournamentSession::where("id",$value->rol_tournament_sesion)->first();//->pluck("id")->toArray();//->get();//
            $suma    = RolTournamentSession::where("rol_tournament_id", $value->rol_tournament_id)
                                            ->where("team_id", $value->team_id)->sum("sum_puntaje_handycap");               

            if( $rol_t_s ){
                array_push($penalties_push, array( "sesion" => $value->sesion, "sum_puntaje_handycap" => $rol_t_s->sum_puntaje_handycap,  "team_id" => $value->team_id, "rol_tournament_session" => $value->rol_tournament_id, "total" => $suma ));
            }                                                

        }   
        
        $response["penalties"]     = $penalties_push;
        $response["dummy_penalty"] = $dummy_penalty;
        
        return response()->json( $response );
    }


    public function save_puntaje_juego(Request $request)
    {

        //first_team
        //second_team
        //current_session
        //rol_tournament_id
        //points_by_player_first_team
        //points_by_player_second_team

        $response = array();
        $response["message"] = "";

        DB::beginTransaction();

        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        $first_team  = Team::find($request->first_team["team"]["id"]);
        $second_team = Team::find($request->second_team["team"]["id"]);

        $player_type = PlayerType::find($request->league_id); 


        /********** Validamos que hallan ingresado todos los puntajes **********/
        //Limpiamos los array que no contengan ningun valor vacio
        $request->points_by_player_first_team = array_filter($request->points_by_player_first_team, function ($value) {
            return !is_null($value) && $value !== '';
        });
        $request->points_by_player_second_team = array_filter($request->points_by_player_second_team, function ($value) {
            return !is_null($value) && $value !== '';
        });

        $rol_t = RolTournament::find($request->rol_tournament_id);       
        
        // error_log("First session =>". $rol_t);   

        //Revisamos que se halla ingresado el total de puntaje
        if ($first_team->name != "DUMMY" && $rol_t->first_team_suspended == 0 ) {
            if (count($request->points_by_player_first_team) < $league["get_player_types"]->type_game) {
                $response["message"] = "Ingresa el puntaje de los " . $league["get_player_types"]->type_game . " juegos para el equipo de '" . $first_team->name . "''.";
                return response()->json($response, 504);
            }
        }

        if ($second_team->name != "DUMMY" && $rol_t->second_team_suspended == 0 ) {
            if (count($request->points_by_player_second_team) < $league["get_player_types"]->type_game) {
                $response["message"] = "Ingresa el puntaje de los " . $league["get_player_types"]->type_game . " juegos para el equipo de '" . $second_team->name . "''.";
                return response()->json($response, 504);
            }
        }


        /********** Revisamos si alguno de los dos equipos es DUMMY **********/
        //Si alguno de los equipos es DUMMY entonces el proceso de validación se omite por lo que el partido va a estar ya validado
        $is_DUMMY_in_game = false;
        if ($first_team->name == "DUMMY" || $second_team->name == "DUMMY") {
            $is_DUMMY_in_game = true;
        }

        /********** Insertamos los puntajes de cada jugador **********/
        //Equipo 1
        $rol_tournament_sessions_first_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                            ->where("team_id", $first_team->id)->where("session_number", $request->current_session)->first();

                                                 

        //Eliminamos el puntaje, cada vez que almacenemos el puntaje tenemos que elminar lo ya almacenado antes de volverlo a insertar
        RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_first_team->id)->delete();

        //tipo de jugadores
        
        // $request->first_team->team->id  //team id
        // $request->first_team->players 
        // $request->first_team->rol_tournament_sessions->id
        // $request->league_id
        // $request->rol_tournament_id
        

        //Si hay suspendidos
        // if( $rol_t->first_team_suspended == 1 ){
        //     $this->partido_suspendido( json_encode( $request->all() ), $rol_t, $rol_t->first_team_id );
        // }

        // if( $rol_t->second_team_suspended == 1 ){
        //     $this->partido_suspendido( json_encode( $request->all() ), $rol_t, $rol_t->second_team_id );
        // }

        
        
        if(count($request->first_team["players"]) < $player_type->type_game ):

        endif;

        if(count($request->second_team["players"]) < $player_type->type_game ):

        endif;
        
        $sum_puntaje  = 0;
        $sum_handycap = 0;


        foreach ($request->points_by_player_first_team as $player_id => $points) {
        
            $rol_tournament_session_points = new RolTournamentSessionPoint();
            $rol_tournament_session_points->rol_tournament_session_id = $rol_tournament_sessions_first_team->id;
            $rol_tournament_session_points->player_id = $player_id;
            $rol_tournament_session_points->points = $points;
            $rol_tournament_session_points->save();

            $sum_puntaje  += $points;

            // error_log($points);

            // $hc_calculado = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_first_team->id)
            //                                         ->where("player_id", $player_id)
            //                                         ->first();

            // $sum_handycap += $hc_calculado->handycap_calculated;
            
            
        }

        // error_log( json_encode( $request->first_team["players"], true ) );

        // $first_team_players =  json_decode( $request->first_team["players"] );
        // $first_team_players =  json_decode( $first_team_players );

        $tram_c_s = TeamAndCategorySpecification::where("league_id", $request->league_id)->first();
        $tram_c_s = TeamCategory::where("team_and_category_specifications_id", $tram_c_s->id)->first();
        
        $category_id = 0;
     
        
        foreach ($request->first_team["players"] as $key => $player ) {
            
            
            //los jugadores dummy tienen numeros negativos como  -1 -2 ellos no aumentan su handicap
            if( $player["player_id"] > 0 ){
                

                $team_e_c     = TeamEachCategory::where("team_category_id", $tram_c_s->id)->where("team_id", $player["team_id"])->first();
                $player_t_e_c = PlayerInTeamEachCategory::where("team_each_categories_id", $team_e_c->id)->where("player_id", $player["player_id"])->first();            

                $update_hc = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_first_team->id)->where("player_id", $player["player_id"])->first();
                                                
                $hc = $this->get_handycap_by_player( $player_t_e_c, 
                                                    //  $player["week"], 
                                                     $rol_t->iterator_week,
                                                     $request->league_id, 
                                                     $player["category_id"], 
                                                     $player["team_id"], 
                                                     $player["type_player"], 
                                                     $rol_t);                                

                
                $update_hc->handycap_calculated = $hc;
                $update_hc->save();
                
                $category_id =  $player["category_id"];  
                $sum_handycap += $hc;   
                
                // $player["player_id"]["handicap"] = $hc;

                error_log( "Handicap =>". $hc);

            }           
            
        }
        

        $penalizacion = Penalties::where( "league_id", $request->league_id )->where("team_id", $first_team->id)->where("week", $rol_t->iterator_week)
                        ->where("category_id", $category_id)->where("rol_tournament_id", $request->rol_tournament_id)->first();
                        // ->where("sesion", $rol_tournament_sessions_first_team->session_number)->first();

       
        $suma_puntaje_hc_total_first = $sum_puntaje + $sum_handycap;

        if( $penalizacion ){
            $suma_puntaje_hc_total_first = $suma_puntaje_hc_total_first - $player_type->penalty_pins;            
            $penalizacion->rol_tournament_sesion = $rol_tournament_sessions_first_team->id;
            $penalizacion->save();           
        }

        $rol_tournament_sessions_first_team->sum_puntaje  = $sum_puntaje;        
        $rol_tournament_sessions_first_team->sum_handycap = $sum_handycap;
        $rol_tournament_sessions_first_team->sum_puntaje_handycap = $suma_puntaje_hc_total_first;//$sum_puntaje + $sum_handycap;        
        $rol_tournament_sessions_first_team->in_process_to_validate = 1;
        $rol_tournament_sessions_first_team->validate = ($is_DUMMY_in_game) ? 1 : 1; //Si hay un equipo DUMMY jugando el juego ya estará validado ya que el DUMMY es ficticio.
        $rol_tournament_sessions_first_team->save();
        

        
        $sum_puntaje_second  = 0;
        $sum_handycap_second = 0; 
        
        
    // if( $rol_t->second_team_suspended == 1 ){        
        
        //Equipo 2
        $rol_tournament_sessions_second_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $second_team->id)->where("session_number", $request->current_session)->first();

        //Eliminamos el puntaje, cada vez que almacenemos el puntaje tenemos que elminar lo ya almacenado antes de volverlo a insertar
        RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_second_team->id)->delete();                                


        foreach ($request->points_by_player_second_team as $player_id => $points) {
            $rol_tournament_session_points = new RolTournamentSessionPoint();
            $rol_tournament_session_points->rol_tournament_session_id = $rol_tournament_sessions_second_team->id;
            $rol_tournament_session_points->player_id = $player_id;
            $rol_tournament_session_points->points = $points;
            $rol_tournament_session_points->save();            
            $sum_puntaje_second  += $points;
            // $hc_calculado = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_second_team->id)
            //                                         ->where("player_id", $player_id)
            //                                         ->first();
            // $sum_handycap_second += $hc_calculado->handycap_calculated;                                  

        }              


        foreach ($request->second_team["players"] as $key => $player ) {
            
            // error_log( "get_pleyr".$player["player_id"] );
    
            if( $player["player_id"] > 0 ){

                $team_e_c     = TeamEachCategory::where("team_category_id", $tram_c_s->id)->where("team_id", $player["team_id"])->first();
                $player_t_e_c = PlayerInTeamEachCategory::where("team_each_categories_id", $team_e_c->id)->where("player_id", $player["player_id"])->first();            
                $update_hc = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_second_team->id)->where("player_id", $player["player_id"])->first();


                $hc = $this->get_handycap_by_player( $player_t_e_c, 
                                                    //$player["week"], 
                                                    $rol_t->iterator_week,
                                                    $request->league_id, 
                                                    $player["category_id"], 
                                                    $player["team_id"], 
                                                    $player["type_player"], 
                                                    $rol_t);

                $update_hc->handycap_calculated = $hc;
                $update_hc->save();
                         
                $category_id =  $player["category_id"];                                              
                
                $sum_handycap_second += $hc;    
            
            }
            
        }

    // }

        $penalizacion_dos = Penalties::where( "league_id", $request->league_id )->where("team_id", $second_team->id)->where("week", $rol_t->iterator_week)
                        ->where("category_id", $category_id)->where("rol_tournament_id", $request->rol_tournament_id)->first();
                        // ->where("sesion", $rol_tournament_sessions_first_team->session_number)->first();

        $suma_puntaje_hc_total_second = $sum_puntaje_second + $sum_handycap_second;

        if( $penalizacion_dos ){
            $suma_puntaje_hc_total_second = $suma_puntaje_hc_total_second - $player_type->penalty_pins;            
            $penalizacion_dos->rol_tournament_sesion = $rol_tournament_sessions_second_team->id;
            $penalizacion_dos->save();           
        }
        
        //se añadio
        $rol_tournament_sessions_second_team->sum_puntaje  = $sum_puntaje_second;
        $rol_tournament_sessions_second_team->sum_handycap = $sum_handycap_second;
        $rol_tournament_sessions_second_team->sum_puntaje_handycap = $suma_puntaje_hc_total_second;//$sum_puntaje_second + $sum_handycap_second;
        $rol_tournament_sessions_second_team->in_process_to_validate = 1;
        $rol_tournament_sessions_second_team->validate = ($is_DUMMY_in_game) ? 1 : 1; //Si hay un equipo DUMMY jugando el juego ya estará validado ya que el DUMMY es ficticio.
        $rol_tournament_sessions_second_team->save();

        
        // $suma_puntaje_hc_total_second = $sum_puntaje_second + $sum_handycap_second;


        $rol_t_s_first = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                ->where("team_id", $first_team->id)
                                ->where("session_number", $request->current_session)
                                ->first();
    
                                
        $rol_t_s_second = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                ->where("team_id", $second_team->id)
                                ->where("session_number", $request->current_session)
                                ->first(); 


        /** REVISA LA CONFIGURACIÓN DE LA LIGA EN HANDICAP **/
        $handicap_each_categories =  HandicapEachCategory::where("team_category_id", $request->league_id)->first();

        $puntos_por_seria_ganada = $league->get_player_types["point_by_win_serie"];        
        $total_confi_liga        =  $league->get_player_types["number_game_by_session"] * $handicap_each_categories->base_dummy;

        if( $rol_t->first_team_suspended == 1 ){       
            //SI EL PRIMERO ES SUSPENDIDO EL SEGUNDO SERA CALIFICADO VS LA CONFIGURACION DE LA LIGA
            
            if( $suma_puntaje_hc_total_second > $handicap_each_categories->base_dummy ){
    
                $rol_t_s_first->win  = 0;
                $rol_t_s_second->win = 1;                                      
    
            }else if( $suma_puntaje_hc_total_second < $handicap_each_categories->base_dummy ){
    
                $rol_t_s_first->win  = 1;
                $rol_t_s_second->win = 0;
    
            }else{
    
                $rol_t_s_first->win  = 0;
                $rol_t_s_second->win = 0;
            
            }

        }else if( $rol_t->second_team_suspended == 1 ){
        
            
            if( $suma_puntaje_hc_total_first > $handicap_each_categories->base_dummy ){
    
                $rol_t_s_first->win  = 1;
                $rol_t_s_second->win = 0;                                      
    
            }else if( $suma_puntaje_hc_total_first < $handicap_each_categories->base_dummy ){
    
                $rol_t_s_first->win  = 0;
                $rol_t_s_second->win = 1;
    
            }else{
    
                $rol_t_s_first->win  = 0;
                $rol_t_s_second->win = 0;
            
            }

        }else{

            if( $suma_puntaje_hc_total_first > $suma_puntaje_hc_total_second ){
    
                $rol_t_s_first->win  = 1;
                $rol_t_s_second->win = 0;                                      
    
            }else if( $suma_puntaje_hc_total_first < $suma_puntaje_hc_total_second ){
    
                $rol_t_s_first->win  = 0;
                $rol_t_s_second->win = 1;
    
            }else{
    
                $rol_t_s_first->win  = 0;
                $rol_t_s_second->win = 0;
            
            }
        }

        //GUARDA SESION GANADA NORMA O VS LIGA
        $rol_t_s_second->save();
        $rol_t_s_first->save();
        

        $total_first = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                            ->where("team_id", $first_team->id)
                                            ->sum("sum_puntaje");

        $total_first_hc = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                ->where("team_id", $first_team->id)
                                                ->sum("sum_handycap");  


        $total_second = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                            ->where("team_id", $second_team->id)
                                            ->sum("sum_puntaje");

        $total_second_hc = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                ->where("team_id", $second_team->id)
                                                ->sum("sum_handycap");     
                            
                            
        $total_first_hc_new  = $total_first  + $total_first_hc;
        $total_second_hc_new = $total_second + $total_second_hc;            

        //QUIEN GANA LA SERIE                                    

        $rol_t->first_team_total  = $total_first_hc_new;
        $rol_t->second_team_total = $total_second_hc_new;

        if( $rol_t->first_team_suspended == 1 ){
            // $total_confi_liga
            //si el total de laliga es mayor el equipo pierde la serie
            if( $total_confi_liga > $total_second_hc_new ){
                
                $rol_t->first_team_win_serie  = 0;
                $rol_t->second_team_win_serie = 0; 

            }else if( $total_confi_liga < $total_second_hc_new ){
                //GANA EL SEGUNDO EQUIPO POR QUE EL PUNTAJE DE LA LIGA ES MENOR
                $rol_t->first_team_win_serie  = 0;
                $rol_t->second_team_win_serie = $player_type->point_by_win_serie;

            }else{
                $rol_t->first_team_win_serie  = 0;
                $rol_t->second_team_win_serie = 0;
            }

        }else if( $rol_t->second_team_suspended == 1 ){

            if( $total_confi_liga > $total_first_hc_new ){
                
                $rol_t->first_team_win_serie  = 0;
                $rol_t->second_team_win_serie = 0; 

            }else if( $total_confi_liga < $total_first_hc_new ){
                //GANA EL SEGUNDO EQUIPO POR QUE EL PUNTAJE DE LA LIGA ES MENOR
                $rol_t->first_team_win_serie  = $player_type->point_by_win_serie;
                $rol_t->second_team_win_serie = 0;

            }else{
                $rol_t->first_team_win_serie  = 0;
                $rol_t->second_team_win_serie = 0;
            }

        }else{

            if( $total_first_hc_new > $total_second_hc_new ){
                $rol_t->first_team_win_serie  = $player_type->point_by_win_serie;
                $rol_t->second_team_win_serie = 0;
            }else if( $total_first_hc_new < $total_second_hc_new ){
                $rol_t->first_team_win_serie  = 0;
                $rol_t->second_team_win_serie = $player_type->point_by_win_serie;
            }else{
                // si el total es empatado se revisa cuantos juegos gano cada 1 para dar el punto por serie

                $total_first_wins = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                            ->where("team_id", $first_team->id)
                                            ->sum("win");

                $total_second_wins = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                            ->where("team_id", $second_team->id)
                                            ->sum("win");   
                                            
                if( $total_first_wins > $total_second_wins ){

                    $rol_t->first_team_win_serie  = 1;
                    $rol_t->second_team_win_serie = 0;
                    
                }else if( $total_first_wins < $total_second_wins ){

                    $rol_t->first_team_win_serie  = 0;
                    $rol_t->second_team_win_serie = 1;

                }else{
                    //otro desempate 
                    $rol_t->first_team_win_serie  = 0;
                    $rol_t->second_team_win_serie = 0;
                }                                            
              
            }

        }

        //GUARDA QUIEN GANA LA SERIE NORMAL Y VS LA LIGA        
        

        //revisar si es el utimo juego 
        $rol_tournament_sessions_session = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                                ->where("validate", 1)
                                                                ->groupBy("session_number")
                                                                ->get()->toArray();

        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        
        $finaliza_juego = false;

        if( count( $rol_tournament_sessions_session ) >= $league["get_player_types"]->number_game_by_session ){

            $rol_t->save(); //se guarda quien gana y pierde cuando es el ultimo juego

            $finaliza_juego = true;
            // error_log("ES TU ÚLTIMO JUEGO");            

            //Valida el empate al finalizar el juego
            $rol_t_s_first_final = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                ->where("team_id", $first_team->id)
                                // ->where("session_number", $request->current_session)
                                ->get();
    
                                
            // $rol_t_s_second_final = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
            //                         ->where("team_id", $second_team->id)
            //                         // ->where("session_number", $request->current_session)
            //                         ->get();

            $numero_empates = 0;

            foreach ($rol_t_s_first_final as $key => $value) {

                $rol_t_s_second_final = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                    ->where("team_id", $second_team->id)->where("session_number", $value->session_number)->first();
                                    
                //Si hay un score con hc igual se revisa quien gana la serie para dar el gane
                if( $value->sum_puntaje_handycap == $rol_t_s_second_final->sum_puntaje_handycap ){
                    
                    error_log( $rol_t->first_team_total .">". $rol_t->second_team_total );

                    if( $rol_t->first_team_total > $rol_t->second_team_total){
                     
                        $rol_t_s_final = RolTournamentSession::find( $value->id );
                        $rol_t_s_final->win = 1;
                        $rol_t_s_final->save();

                    }else if( $rol_t->first_team_total < $rol_t->second_team_total ){
                    
                        $rol_t_s_final = RolTournamentSession::find( $rol_t_s_second_final->id);
                        $rol_t_s_final->win = 1;
                        $rol_t_s_final->save();                      

                    }else{
                        $numero_empates++;
                        //si es igual en total de juegos y serie se genera un empate salomonico 2-2 en caso de ser 3 juegos y las serie
                        if( $league["get_player_types"]->number_game_by_session == $numero_empates ){
                            $rol_t_s_first = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                    ->where("team_id", $first_team->id)                                                    
                                                    ->get();

                            foreach ($rol_t_s_first as $key => $rol_s) {

                                if (($rol_s->session_number % 2) == 0) {
                                    
                                    RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                    ->where("team_id", $first_team->id) 
                                                    ->where("session_number", $rol_s->session_number)                                                   
                                                    ->update(["win" => 0]);
                                    
                                }else{

                                    RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                    ->where("team_id", $first_team->id) 
                                                    ->where("session_number", $rol_s->session_number)                                                   
                                                    ->update(["win" => 1]);                                    
                                }

                            }

                            RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                    ->where("team_id", $second_team->id) 
                                                    ->where("session_number", 2)                                                   
                                                    ->update(["win" => 1]);

                            RolTournament::where("id",$request->rol_tournament_id)->update([ "second_team_win_serie" => 1 ]);                                                         
                            
                                                                                                        
                        }
                        //EN CASO DE EMPATE 

                    }                                      
                }
            }                                    

        }

    
        // REVISA LA SEMANA ANTERIOR PARA SABER SI ESTA SUSPENDIDO        
        if( $rol_t->iterator_week > 0 ){
            error_log("NO ES SEMANA 0");
            //Se revisa los 2 id first_team_id y second_team_id en dos consultas
            $rol_tournaments_previous     = RolTournament::where('iterator_week', $rol_t->iterator_week - 1 )->where("suspended",true)->where('category_id',$rol_t->category_id )->whereIn('first_team_id', array($rol_t->first_team_id, $rol_t->second_team_id) )->get();
            $rol_tournaments_previous_dos = RolTournament::where('iterator_week', $rol_t->iterator_week - 1 )->where("suspended",true)->where('category_id',$rol_t->category_id )->whereIn('second_team_id', array($rol_t->first_team_id, $rol_t->second_team_id) )->get();
        
            foreach ($rol_tournaments_previous as $key => $value) {
                if( $value ){
                    // $request->points_by_player_first_team, $request->points_by_player_second_team, 
                    $this->insertar_puntaje_suspendido_new( $request->points_by_player_first_team,
                                                            $value, $rol_tournament_sessions_first_team, $rol_tournament_sessions_second_team,
                                                            $rol_t );
                }
            }

            foreach ($rol_tournaments_previous_dos as $key => $value) {
                if( $value ){
                    // $request->points_by_player_first_team, $request->points_by_player_second_team, 
                    $this->insertar_puntaje_suspendido_new( $request->points_by_player_first_team,
                                                            $value, $rol_tournament_sessions_first_team, $rol_tournament_sessions_second_team,
                                                            $rol_t );
                }
            }

            // foreach ($rol_tournaments_previous as $key => $value) {
            //     $this->insertar_puntaje_suspendido_new( $request->puntaje[$rol_tournaments_id], $value, $request, $rol_tournaments_id, $league, $rol_tournaments_previous );
            // }

            // foreach ($rol_tournaments_previous_dos as $key => $value) {
            //     $this->insertar_puntaje_suspendido_new( $request->puntaje[$rol_tournaments_id], $value, $request, $rol_tournaments_id, $league, $rol_tournaments_previous_dos );
            // }

        }         


        // $response = array();
        // $response["message"] = "";

        // $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        // $first_team = Team::find($request->first_team["team"]["id"]);
        // $second_team = Team::find($request->second_team["team"]["id"]);

        // $rol_tournament_sessions_first_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $first_team->id)->where("session_number", $request->current_session)->first();
        // $rol_tournament_sessions_first_team->validate = 1;
        // $rol_tournament_sessions_first_team->save();

        // $rol_tournament_sessions_second_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $second_team->id)->where("session_number", $request->current_session)->first();
        // $rol_tournament_sessions_second_team->validate = 1;
        // $rol_tournament_sessions_second_team->save();

        // $response["message"] = "El puntaje del juego se ha validado correctamente.";

        // return response()->json($response);

        // $first_team  = Team::find($request->first_team["team"]["id"]);
        // $second_team = Team::find($request->second_team["team"]["id"]);

        PlayersInGame::whereRaw("(team_id = " . $request->first_team["team"]["id"] . " OR team_id = " . $request->second_team["team"]["id"] . ")")                    
                     ->where("league_id", $league->id)
                     ->delete();

        DB::commit();
                

        if ($is_DUMMY_in_game) {
            $response["message"] = "El puntaje del juego se ha almacenado correctamente.";
        } else {
            $response["message"] = "El puntaje del juego se ha almacenado correctamente.";
            //"El puntaje del juego se ha enviado al equipo contrario para su validación.";
        }

        return response()->json($response);
    }

    private function insertar_puntaje_suspendido_new( $points_by_player_team, $rol_tournament_suspendido, 
                    $rol_tournament_sessions_first_team, $rol_tournament_sessions_second_team, $rol_tournament_s_actual ){

        // error_log( "juego suspendido". $rol_tournament_suspendido );                        
        // error_log( "juego nuevo". $rol_tournament_s_actual );                        
            //si recibe puntaje 
                

            //copia al primer equipo de juego suspendido
            if( $rol_tournament_s_actual->first_team_id == $rol_tournament_suspendido->first_team_id or 
                $rol_tournament_s_actual->first_team_id == $rol_tournament_suspendido->second_team_id){
                                        
                    // $rol_tournament_sessions_first_team_new = $rol_tournament_sessions_first_team->replicate();
                    // $rol_tournament_sessions_first_team_new->rol_tournament_id = $rol_tournament_suspendido->id;
                    // $rol_tournament_sessions_first_team_new->save();  
                    error_log( $rol_tournament_s_actual->first_team_id ."==". $rol_tournament_suspendido->first_team_id ."or". $rol_tournament_s_actual->first_team_id ."==". $rol_tournament_suspendido->second_team_id );

                    error_log("rol_tournament_suspendido => ". $rol_tournament_suspendido->id);

            }   
            

            if( $rol_tournament_s_actual->second_team_id == $rol_tournament_suspendido->first_team_id or 
                $rol_tournament_s_actual->second_team_id == $rol_tournament_suspendido->second_team_id){
                    error_log("COINCIDE EL segundo ORDEN");

                    // error_log( $rol_tournament_s_actual->second_team_id ."==". $rol_tournament_suspendido->first_team_id ."or". $rol_tournament_s_actual->second_team_id ."==". $rol_tournament_suspendido->second_team_id );
                    
                    error_log("rol_tournament_suspendido => ". $rol_tournament_suspendido->id);

                    RolTournamentSession::where("rol_tournament_id", $rol_tournament_suspendido->id)->where("team_id", $rol_tournament_s_actual->second_team_id)->where("session_number", $rol_tournament_s_actual->second_team_idcurrent_session)->first();
                    // RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_first_team->id)->delete();

                    // RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_first_team->id)->delete();
                    // if(  ){

                    // }
                    
                    $rol_tournament_sessions_second_team_new = $rol_tournament_sessions_second_team->replicate();
                    $rol_tournament_sessions_second_team_new->rol_tournament_id = $rol_tournament_suspendido->id;
                    $rol_tournament_sessions_second_team_new->save();  


                    foreach ($points_by_player_team as $player_id => $points) {

                        $rol_tournament_session_points = new RolTournamentSessionPoint();
                        $rol_tournament_session_points->rol_tournament_session_id = $rol_tournament_sessions_second_team_new->id;
                        $rol_tournament_session_points->player_id = $player_id;
                        $rol_tournament_session_points->points = $points;
                        $rol_tournament_session_points->save();
            
                        $sum_puntaje  += $points;
            
                        $hc_calculado = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions_second_team_new->id)
                                                                ->where("player_id", $player_id)
                                                                ->first();
                        $sum_handycap += $hc_calculado->handycap_calculated;
                        
                    }


            }

            // //se crean las sesiones ya que no tienen
            // $rol_tournament_sessions_first_team_new = $rol_tournament_sessions_first_team->replicate();
            // $rol_tournament_sessions_first_team_new->rol_tournament_id = $rol_tournament_suspendido->id;
            // $rol_tournament_sessions_first_team_new->save();      

        
       

    }

    private function checkPenalizacion( $players, $player_type ){
        $female = 0;
        $male   = 0;   
        $dummy  = array();

        foreach ($players as $key => $player) {

            if( $player ){
                // error_log( "birthday". json_encode( $player ) );
                if( $player["id"] > 0 ){
                    
                    if( $player["gender"] === "female" && $player["special"] === 0 ){                            
                        $female++;
                    }
    
                    if( $player["gender"] === "male" && $player["special"] === 0 ){
                        $male++;            
                    }                                       
                    
                    $date = Carbon::parse($player["birthday"])->age;
                    //Juvenil           
                    if( $date < $player_type->young_less_than_year ){
                        // error_log("se cumple juvenil ===========> ". $player);
                        $female++;
                    }  
                    
                    if( $player["special"] === 1 ){
                        error_log("se cumple ESPECIAL ===========>");
                        $female++;
                    }  
    
                }else{
                    //hombre
                    if( $player["type_player"] == 1 ){                          
                        $male++;       
                    }

                    //MUJER
                    if( $player["type_player"] == 2 ){                       
                        $female++;                        
                    }

                    //JUVENIL
                    if( $player["type_player"] == 3 ){                       
                        $female++;                        
                    }

                    //ESPECIAL
                    if( $player["type_player"] == 4 ){                       
                        $female++;                        
                    }

                    array_push($dummy, array(  "gender" => $player["type_player"], "key" => $key  ));

                }

            }

            // error_log("male". $male);
            // error_log("female". $female);
            
        }

        return array( "male" => $male, "female" => $female, "dummys" => $dummy);
    }

    private function checkPenalizacion_( $puntajes, $team_id, $player_type, $num, $add_dummy = null, $week_number_pagination, $league_id, $category_id ){
        $female = 0;
        $male   = 0;   
        $dummy  = array();  
        
        // DummyPenalty::where("team_id", $team_id)->where("week",$week_number_pagination)
        //                         ->where("league_id",$league_id)->where("league_id", $league_id)
        //                         // ->where("player_id",$key)->where("sesion",$num)
        //                         // ->where("gender", $add_dummy[$team_id][$num][$key])
        //                         ->where("category_id", $category_id)->delete(); 

        // error_log( "checkPenalizacion => ". print_r($add_dummy[$team_id], true) );    
        
        // $equipos_dummys["dummys"]

        foreach ($puntajes as $key => $puntaje) {

            error_log( $key );

            if( $key > 0 && $puntaje ){

                $player = Player::find( $key );

                if( $player->gender === "female" && $player->special === 0 ){                            
                    $female++;
                }

                if( $player->gender === "male" && $player->special === 0 ){
                    $male++;            
                }                                       
                
                $date = Carbon::parse($player->birthday)->age;
                //Juvenil           
                if( $date < $player_type->young_less_than_year ){
                    error_log("se cumple juvenil ===========> ". $player);
                    $female++;
                }  
                
                if( $player->special === 1 ){
                    error_log("se cumple ESPECIAL ===========>");
                    $female++;
                }  
                
            }else{
                

                if( isset( $add_dummy[$team_id][$num][$key] ) ){
                    
                    error_log( "checkPenalizacion => ". print_r($add_dummy[$team_id][$num], true) );
                                                    
                    //HOMBRE
                    if( $add_dummy[$team_id][$num][$key] == 1 ){                          
                        $male++;       
                    }

                    //MUJER
                    if( $add_dummy[$team_id][$num][$key] == 2 ){                       
                        $female++;                        
                    }

                    //JUVENIL
                    if( $add_dummy[$team_id][$num][$key] == 3 ){                       
                        $female++;                        
                    }

                    //ESPECIAL
                    if( $add_dummy[$team_id][$num][$key] == 4 ){                       
                        $female++;                        
                    }

                    array_push($dummy, array( "league_id" => $league_id, "team_id" => $team_id, "key" => $key, "week" => $week_number_pagination, "sesion" => $num, "gender" => $add_dummy[$team_id][$num][$key] ));                                                     
                    
                    
                }
                                
            }
        }

        return array( "female" => $female, "male" => $male, "team_id" => $team_id, "num" => $num, "dummys" => $dummy );

    }


    public function rechazar_puntaje_juego(Request $request)
    {

        //league_id
        //first_team
        //second_team
        //current_session
        //rol_tournament_id
        //points_by_player_first_team
        //points_by_player_second_team

        $response = array();
        $response["message"] = "";

        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        $first_team = Team::find($request->first_team["team"]["id"]);
        $second_team = Team::find($request->second_team["team"]["id"]);

        $rol_tournament_sessions_first_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $first_team->id)->where("session_number", $request->current_session)->first();
        $rol_tournament_sessions_first_team->in_process_to_validate = 0;
        $rol_tournament_sessions_first_team->save();

        $rol_tournament_sessions_second_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $second_team->id)->where("session_number", $request->current_session)->first();
        $rol_tournament_sessions_second_team->in_process_to_validate = 0;
        $rol_tournament_sessions_second_team->save();

        $response["message"] = "El puntaje del juego se ha rechazado correctamente. Revisa con el equipo contrario el mótivo del rechazo del puntaje.";

        return response()->json($response);


    }


    public function validar_puntaje_juego(Request $request)
    {

        //league_id
        //first_team
        //second_team
        //current_session
        //rol_tournament_id
        //points_by_player_first_team
        //points_by_player_second_team

        $response = array();
        $response["message"] = "";

        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        $first_team = Team::find($request->first_team["team"]["id"]);
        $second_team = Team::find($request->second_team["team"]["id"]);

        $rol_tournament_sessions_first_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $first_team->id)->where("session_number", $request->current_session)->first();
        $rol_tournament_sessions_first_team->validate = 1;
        $rol_tournament_sessions_first_team->save();

        $rol_tournament_sessions_second_team = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $second_team->id)->where("session_number", $request->current_session)->first();
        $rol_tournament_sessions_second_team->validate = 1;
        $rol_tournament_sessions_second_team->save();

        $response["message"] = "El puntaje del juego se ha validado correctamente.";

        return response()->json($response);

    }

    
    public function get_ligas_registradas( Request $request ){

    }

    public function get_tabla_de_posiciones( Request $request ){

        // $rol = Rol::find($rol_id);
        // $league = League::find($rol->league_id);
        // $rol_torunament = RolTournament::where("rol_id", $rol->id)->where("iterator_week", $iterator_week)->where("category_id", $category_id)->first();

        // //Obtenemos la tabla de posiciones por categoria
        // $posiciones = array();
        // $posiciones["tabla_posiciones"] = Pastora::get_tabla_de_posiciones_global($rol->id, $iterator_week, $category_id);
        // $posiciones["category"] = TeamCategory::find($category_id);
        // $posiciones["rol_tournament"] = $rol_torunament;

        // $view = view("back.league.asignar_puntaje.tabla_de_posiciones", compact("posiciones"))->render();

        // return response()->json($view);
        $response = array();
        $response["message"] = "";
        $response["data"] = array();;

        $rol = Rol::where('league_id', $request->league_id)->first();
        $league = League::find($rol->league_id);
        $rol_torunament = RolTournament::where("rol_id", $rol->id)->where("iterator_week", $request->iterator_week)->where("category_id", $request->category_id)->first();

        $response["tabla_posiciones"] = Pastora::get_tabla_de_posiciones_global($rol->id, $request->iterator_week, $request->category_id);
        $response["category"] = TeamCategory::find($request->category_id);
        $response["rol_tournament"] = $rol_torunament;
        
        
        return response()->json($response);
    }

    public function get_player( Request $request ){
        $response = array();
        $response["message"] = "";

        $q = Player::where( 'id', $request->id )->first();

        $response["data"] = $q;

        return response()->json($response, 200);

    }

    public function update_nip( Request $request ){

        $response = array();
        $response["message"] = "";

        $is_player_exists = Player::where("email", $request->email)->where("password", md5($request->pass))->first();
        if ($is_player_exists != null) {
            
            $update = Player::where("email", $request->email)->update(['nip'=> $request->nip]);

            if( $update != null ){
                $response["message"] = "El nip se actualizó correctamente.";
                $response["status"] = true;
                return response()->json($response);
            }else{
                $response["message"] = "Lo sentimos algo salio mal, contacta a soporte para más informacón.";
                return response()->json($response, 504);
            }

        }else{
            $response["message"] = "Tu contraseña es incorrecta.";
            return response()->json($response, 504);
        }
    }

    public function valida_nip(Request $request)
    {

        $response = array();
        $response["message"] = "";

        $is_player_exists = Player::where("email", $request->email)->where("nip", $request->nip)->first();

        if( $is_player_exists !== null ){
            $response["nip"] = true;
            $response["message"] = "Nip correcto";
            return response()->json($response, 200);
        }else{
            $response["message"] = "Tu nip es incorrecto.";
            return response()->json($response, 504);
        }

    }
    
    public function estadisticas( Request $request ){

        $response = array();
        $response["status"] = $request;
        $response["message"] = "";

        $rol = Rol::where('league_id', $request->league_id)->first();
        
        $rol_tournaments_by_week = RolTournament::where("rol_id", $rol->id)->where("category_id", $request->category_id)->where("iterator_week", $request->iterator_week)->where("type", $request->type)->groupBy("iterator_week")->orderBy("iterator_week", "ASC")->get();

        
        $response["data"] = $rol_tournaments_by_week;
        
        return response()->json($response, 200);
    }

    public function player_in_games( Request $request ){

        $players = PlayersInGame::where('player_id', $request->player_id)->get();

        $response = array();
        $response["message"] = "";
        $response["data"] = $players;

        return response()->json($response, 200);
    }

    //cuando se crea un player se crea un customer 
    public function create_customer( Request $request ){

        // \Conekta\Conekta::setApiKey( "key_YinuQH4ZD6iZmqHsyxubag" );
        // \Conekta\Conekta::setApiVersion( 2.0 );

        $response = array();
        $response["message"] = "";

        try{

            $this->create = \Conekta\Customer::create(
                array(
                    "name"  => $request->name,
                    "email" => $request->email,
                    "phone" => "+52181818181",
                    // "payment_sources" => array(
                    //   array(
                    //       "type" => "card",
                    //       "token_id" => $request->token
                    //   )
                    // )
                )
            );

            $response["create"] = $this->create;

            return response()->json($response, 200);
    

        } catch (\Conekta\ProccessingError $error){
            $response["message"] = "Error al registrar Customer: ProccessingError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }catch (\Conekta\ParameterValidationError $error){
            $response["message"] = "Error al registrar Customer: ParameterValidationError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        } catch (\Conekta\Handler $error){
            $response["message"] = "Error al registrar Customer: Handler";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }

    }

    public function get_cards( Request $request ){

        $response = array();
        $response["message"] = "";

        // \Conekta\Conekta::setApiKey( "key_Y  inuQH4ZD6iZmqHsyxubag" );

        try{
            $this->find = \Conekta\Customer::find($request->id);
            
            $response["customer"] = $this->find;
            
            return response()->json($response, 200);


        } catch (\Conekta\ProccessingError $error){
            $response["message"] = "Error al registrar Customer: ProccessingError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }catch (\Conekta\ParameterValidationError $error){
            $response["message"] = "Error al registrar Customer: ParameterValidationError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        } catch (\Conekta\Handler $error){
            $response["message"] = "Error al registrar Customer: Handler";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }

    }

    public function add_card( Request $request ){

        $response = array();
        $response["message"] = "";

        // \Conekta\Conekta::setApiKey( "key_YinuQH4ZD6iZmqHsyxubag" );

        try{
            $this->find = \Conekta\Customer::find($request->client_id_conekta);
            $this->data = $this->find->createPaymentSource( array( "type"=>"card", "token_id" => $request->token_id ) );
            $response["data"] = $this->data;

            return response()->json($response, 200);


        } catch (\Conekta\ProccessingError $error){
            $response["message"] = "Error al registrar Customer: ProccessingError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }catch (\Conekta\ParameterValidationError $error){
            $response["message"] = "Error al registrar Customer: ParameterValidationError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        } catch (\Conekta\Handler $error){
            $response["message"] = "Error al registrar Customer: Handler";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }

    }

    public function update_card( Request $request ){

        $response = array();
        $response["message"] = "";

        try{

            $this->find = \Conekta\Customer::find($request->client_id_conekta);
         
            foreach ($this->find->payment_sources as $card => $valor) {
                
                if( $request->card_id == $this->find->payment_sources[$card]->id ){
                    $response["status"] = true;
                    $response["card"] = $this->find->payment_sources[$card]->update( array( "exp_month" => $request->exp_month, "exp_year" => $request->exp_year, "name" => $request->name ) );
                }
                
            }

            return response()->json($response, 200);


        } catch (\Conekta\ProccessingError $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: ProccessingError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }catch (\Conekta\ParameterValidationError $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: ParameterValidationError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        } catch (\Conekta\Handler $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: Handler";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }
        // return response()->json($response, 200);
    }

    public function delete_card( Request $request ){
        $response = array();
        $response["message"] = "";

        try{

            $this->find = \Conekta\Customer::find($request->client_id_conekta);
         
            foreach ($this->find->payment_sources as $card => $valor) {
                
                if( $request->card_id == $this->find->payment_sources[$card]->id ){
                    $response["status"] = true;
                    $response["card"] = $this->find->payment_sources[$card]->delete();
                }
                
            }

            return response()->json($response, 200);


        } catch (\Conekta\ProccessingError $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: ProccessingError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }catch (\Conekta\ParameterValidationError $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: ParameterValidationError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        } catch (\Conekta\Handler $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: Handler";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }
    }

    public function to_pay( Request $request ){

        $response = array();
        $response["message"] = "";

        try{

            $this->order = \Conekta\Order::create( 
                [
                    "currency" => "MXN",
                    "customer_info" => [
                        "customer_id" => $request->client_id_conekta//"cus_2mcUE2NZYVndZJVSV"
                    ], 
                    "line_items" => [
                        [
                            "name" => $request->name,
                            "unit_price" => $request->unit_price * 100,//$request->unit_price,
                            "quantity" => 1
                        ]
                    ],
                    "metadata" => [
                        "description" => "Membresía",
                        "reference"   => $request->name
                    ],
                    // "shipping_lines" => [
                    //     [
                    //       "amount" => 50,//$request->unit_price,
                    //       "carrier" => "OXXO"
                    //     ]
                    // ], 
                    // "pre_authorize" => true,
                    "charges" => [
                        [
                          "payment_method" => [
                            "type" => "card",
                            "payment_source_id" => $request->payment_source_id//"src_2md7bpnEzeakB4jNm"
                          ]
                        ]
                    ]
                ]
            );

            $response["status"] = true;
            $response["data"]   = $this->order;

            $user = Player::find($request->id);

            $user->membership = $request->membership;


            if ($user->save()) {
                
            }

            return response()->json($response, 200);


        } catch (\Conekta\ProccessingError $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: ProccessingError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }catch (\Conekta\ParameterValidationError $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: ParameterValidationError";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        } catch (\Conekta\Handler $error){
            $response["message"] = $error->getMessage();//"Error al registrar Customer: Handler";
            $response["error"] = $error->getMessage(); 
            return response()->json($response, 504);
        }

    }

    public function puntuaciones( Request $request ){

        $response = array();
        $response["message"] = "";

        $data = League::find($request->league_id);
        $rol = Rol::where("league_id", $data->id)->first();

        //Obtenemos la cantidad de equipos dummy que podemos insertar en cada partido
        $total_dummy_players_available = Handicap::where("league_id", $data->id)->first();
        $total_dummy_players_available = $total_dummy_players_available->dummy_by_team;

        /************* Obtenemos el número de semana *************/
        //Obtenemos el número de la semana segun el iterator_week, ya que tenemos semanas de ronda de posiciones virtuales
        //->groupBy("iterator_week")->orderBy("id", "ASC")
        $rol_tournaments_by_week = RolTournament::where("rol_id", $rol->id)->where("category_id", $request->category_id)->where("iterator_week", $request->week)->where("type", $request->type)->groupBy("iterator_week")->orderBy("iterator_week", "ASC")->get();

        $players = TeamEachCategory::where("team_category_id", $request->category_id)->where("team_id", 20)->with("get_player_in_team_each_category.get_player")->first();
        
        // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $request->category_id)->where("iterator_week", $request->week)->with(["get_first_team", "get_second_team"])->orderBy("id", "ASC")->get();
        // --- $players = TeamEachCategory::where("team_category_id", $request->category_id)->where("team_id", 20)->with("get_player_in_team_each_category.get_player")->first();

        // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $request->category_id)->where("iterator_week", $request->week)->orderBy("id", "ASC")->get();

        //Obtenemos el puntaje
        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", 12449)->where("team_id", 12)->orderBy("session_number")->get();

        //**$rol_tournaments_by_week["first_team_players"] = $players["get_player_in_team_each_category"];

        
        // foreach ($rol_tournaments_by_week["first_team_players"] as $key_player => $player) {
        //      //Iteramos cada sesion y despues cada jugador para obtener su puntaje
        //      $puntaje_por_session = array();
        //      foreach ($rol_tournament_sessions as $rol_tournament_session) {
        //         $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", $player->player_id)->first();

        //         if ($rol_tournament_session_points != null) {
        //             $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
        //         }
        //      }

        //      $rol_tournaments_by_week["first_team_players"][$key_player]->rol_tournament_session_points = $puntaje_por_session;
        // }

        $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", 262)->where("player_id", 261)->first();

        // $rol_tournaments_by_week["first_team_rol_tournament_sessions"] = $rol_tournament_sessions;

        // $response["players"] = $players;
        // $response["rol_tournament_session_points"] = $rol_tournament_session_points;
        $response["rol_tournaments_by_week"] = $rol_tournaments_by_week;

        return response()->json($response, 200);
    }

    public function puntaje( Request $request ){

        $response = array();
        $response["message"] = "";

        $id = $request->league_id;
        $iterator_week = $request->week;
        $category_id = $request->category_id;
        $type = $request->type;

        // $id = base64_decode($id);

        $data = League::find($id);
        $rol = Rol::where("league_id", $data->id)->first();

        //Obtenemos la cantidad de equipos dummy que podemos insertar en cada partido
        $total_dummy_players_available = Handicap::where("league_id", $data->id)->first();
        $total_dummy_players_available = $total_dummy_players_available->dummy_by_team;


        //Vamos a obtener los juegos de cada semana, primero la semana 1 y asi sucesivamente.
        //Dejaremos de mostrar los juegos cuando en una semana este pendiente de llenar el puntaje, esto por las rondas de posiciones virtuales.
        //Por ejemplo si estamos en la semana 4 y no han llenado de información, no mostraremos la semana 5 en adelante hasta que termine de llenar la información de la semana 4.

        $categories = $data->get_team_and_category_specification->get_categories;
        $juegos_por_semana = array();


        /************* Obtenemos el número de semana *************/
        //Obtenemos el número de la semana segun el iterator_week, ya que tenemos semanas de ronda de posiciones virtuales
        $rol_tournaments_weeks = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("type", "!=", "ronda_posiciones_virtual")->groupBy("iterator_week")->orderBy("id", "ASC")->get();
        $week_label_number = 1;
        foreach ($rol_tournaments_weeks as $rol_tournaments_week) {
            if ($rol_tournaments_week->iterator_week < $iterator_week) {
                $week_label_number++;
            }
        }


        /************* Obtenemos el paginador para la semana anterior y siguiente *************/
        $semana_anterior = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ($iterator_week - 1))->first();
        $semana_siguiente = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ($iterator_week + 1))->first();


        //Cuando es día festivo no tendremos datos, solo mostramos que es festivo
        if ($type == "festivo") {

            $response["festivo"] = true;
            $response["mensaje"] = "Este día esta registrado como festivo.";

            return response()->json($response, 200);

            // $semana_actual = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", $iterator_week)->first();

            // return view('back.league.asignar_puntaje.create', array("rol_id" => $rol->id, 'juegos_por_semana' => null, "league" => $data, "enable_go_to_next_week" => true, "week_number" => $iterator_week, "total_dummy_players_available" => null, "categories" => $categories, "category_id" => $category_id, "semana_anterior" => $semana_anterior, "semana_siguiente" => $semana_siguiente, "type" => $type, "semana_actual" => $semana_actual, "week_label_number" => $week_label_number));
        }

        $enable_go_to_next_week = true;


        //Obtenemos la semana del torneo
        $rol_tournaments_by_week = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", $iterator_week)->where("type", $type)->groupBy("iterator_week")->orderBy("iterator_week", "ASC")->get();

        foreach ($rol_tournaments_by_week as $rol_tournament_by_week) {

            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", $iterator_week)->with(["get_first_team", "get_second_team"])->orderBy("id", "ASC")->get();

            /******** Para cada rol_tournament vamos a obtener los jugadores del primer y segundo equipo ********/
            foreach ($rol_tournaments as $key => $value) {

                /******** Primer equipo ********/
                $rol_tournaments[$key]["first_team_players"] = null;
                $players = TeamEachCategory::where("team_category_id", $category_id)->where("team_id", $rol_tournaments[$key]["first_team_id"])->with("get_player_in_team_each_category.get_player")->first();
                $rol_tournaments[$key]["first_team_players"] = $players["get_player_in_team_each_category"];

                //Agregamos como jugadores adicionales los dummy
                for ($i = 1; $i <= $total_dummy_players_available; $i++) {

                    $player = new Player();
                    $player->id = ($i * -1);
                    $player->name = "DUMMY";

                    $dummy = new PlayerInTeamEachCategory();
                    //"id" => ,
                    //"team_each_categories_id" => 0,
                    $dummy->player_id = ($i * -1);
                    $dummy->player_handy_cap_in_tournament = 0;
                    $dummy->get_player = $player;
                    $rol_tournaments[$key]["first_team_players"]->push($dummy);

                }

                //Obtenemos el puntaje
                $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $value->id)->where("team_id", $rol_tournaments[$key]["first_team_id"])->orderBy("session_number")->get();
                foreach ($rol_tournaments[$key]["first_team_players"] as $key_player => $player) {

                    //Iteramos cada sesion y despues cada jugador para obtener su puntaje
                    $puntaje_por_session = array();
                    foreach ($rol_tournament_sessions as $rol_tournament_session) {

                        $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", $player->player_id)->first();

                        if ($rol_tournament_session_points != null) {
                            $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
                        }

                    }

                    $rol_tournaments[$key]["first_team_players"][$key_player]->rol_tournament_session_points = $puntaje_por_session;

                }

                $rol_tournaments[$key]["first_team_rol_tournament_sessions"] = $rol_tournament_sessions;


                /******** Segundo equipo ********/
                $rol_tournaments[$key]["second_team_players"] = null;
                $players = TeamEachCategory::where("team_category_id", $category_id)->where("team_id", $rol_tournaments[$key]["second_team_id"])->with("get_player_in_team_each_category.get_player")->first();
                $rol_tournaments[$key]["second_team_players"] = $players["get_player_in_team_each_category"];

                //Agregamos como jugadores adicionales los dummy
                for ($i = 1; $i <= $total_dummy_players_available; $i++) {

                    $player = new Player();
                    $player->id = ($i * -1);
                    $player->name = "DUMMY";

                    $dummy = new PlayerInTeamEachCategory();
                    //"id" => ,
                    //"team_each_categories_id" => 0,
                    $dummy->player_id = ($i * -1);
                    $dummy->player_handy_cap_in_tournament = 0;
                    $dummy->get_player = $player;
                    $rol_tournaments[$key]["second_team_players"]->push($dummy);

                }

                //Obtenemos el puntaje
                $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $value->id)->where("team_id", $rol_tournaments[$key]["second_team_id"])->orderBy("session_number")->get();
                foreach ($rol_tournaments[$key]["second_team_players"] as $key_player => $player) {

                    //Iteramos cada sesion y despues cada jugador para obtener su puntaje
                    $puntaje_por_session = array();
                    foreach ($rol_tournament_sessions as $rol_tournament_session) {

                        $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", $player->player_id)->first();

                        if ($rol_tournament_session_points != null) {
                            $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
                        }

                    }

                    $rol_tournaments[$key]["second_team_players"][$key_player]->rol_tournament_session_points = $puntaje_por_session;

                }

                $rol_tournaments[$key]["second_team_rol_tournament_sessions"] = $rol_tournament_sessions;

            }

            $category_info = TeamCategory::find($category_id);

            $juegos_por_semana[$category_id][$rol_tournament_by_week->iterator_week]["category"] = $category_info;
            $juegos_por_semana[$category_id][$rol_tournament_by_week->iterator_week]["rol_tournament_information"] = $rol_tournament_by_week;
            $juegos_por_semana[$category_id][$rol_tournament_by_week->iterator_week]["rol_tournaments"] = $rol_tournaments;

        }


        //Revisaremos si podremos podremos continuar a la siguiente semana
        $rol_tournaments_by_week = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", $iterator_week)->whereRaw("(first_team_total is null or second_team_total is null)")->count();
        if ($rol_tournaments_by_week > 0) {
            $enable_go_to_next_week = false;
        }


        /************* Revisaremos el handycap de cada jugador *************/
        foreach ($juegos_por_semana as $category_id => $juego_por_semana) {

            foreach ($juego_por_semana[$iterator_week]["rol_tournaments"] as $key_rol_tournament => $rol_tournament) {

                foreach ($rol_tournament["first_team_players"] as $key_first_team_players => $first_team_players) {
                    //Calculamos el handycap ya sea el inicial o dependiendo de las reglas que tenemos
                    $team_id = $rol_tournament->first_team_id;                    
                    $juegos_por_semana[$category_id][$iterator_week]["rol_tournaments"][$key_rol_tournament]["first_team_players"][$key_first_team_players]["handycap_calculated"] = $this->get_handycap_by_player($first_team_players, $iterator_week, $data->id, $category_id, $team_id, $type, $rol_tournament);
                }

                foreach ($rol_tournament["second_team_players"] as $key_second_team_players => $second_team_players) {
                    //Calculamos el handycap ya sea el inicial o dependiendo de las reglas que tenemos
                    $team_id = $rol_tournament->second_team_id;
                    $juegos_por_semana[$category_id][$iterator_week]["rol_tournaments"][$key_rol_tournament]["second_team_players"][$key_second_team_players]["handycap_calculated"] = $this->get_handycap_by_player($second_team_players, $iterator_week, $data->id, $category_id, $team_id, $type, $rol_tournament);
                }

            }

        }   


        $response["juegos_por_semana"] = $juegos_por_semana;
        // $response["league"] = $data;
        // $response["total_dummy_players_available"] = $total_dummy_players_available;
        

        return response()->json($response, 200);

        // return view('back.league.asignar_puntaje.create', array("rol_id" => $rol->id, 'juegos_por_semana' => $juegos_por_semana, "league" => $data, "enable_go_to_next_week" => $enable_go_to_next_week, "week_number" => $iterator_week, "total_dummy_players_available" => $total_dummy_players_available, "categories" => $categories, "category_id" => $category_id, "semana_anterior" => $semana_anterior, "semana_siguiente" => $semana_siguiente, "type" => $type, "week_label_number" => $week_label_number, 'permitions' => URI::checkPermitions()));

    }

    public function player_in_game_id( Request $request ){
        $response = array();
        $response["message"] = "";


        $team_specifications = TeamAndCategorySpecification::where( "league_id", $request->league_id )->first();
        // error_log($team_specifications);
        $team_categories = TeamCategory::where( "team_and_category_specifications_id", $team_specifications->id )->first();

        $player_in_te = PlayerInTeamEachCategory::where("player_id", $request->player_id)->pluck("team_each_categories_id")->toArray();        
        $team_each    = TeamEachCategory::whereIn("id",$player_in_te)->where("team_category_id", $team_categories->id)->first();  
        // $player_in_te_new = PlayerInTeamEachCategory::where()

        // $player_in_te = PlayerInTeamEachCategory::where("player_id", $request->player_id)->first();
        // $team_each    = TeamEachCategory::where("id",$player_in_te->team_each_categories_id)->first();        
        // error_log( json_encode( $request->all() ) );
        
        // $playerInGame = PlayersInGame::where('player_id',$request->player_id)->where('league_id',$request->league_id)->where('category_id',$request->category_id)->get();
        $response["data"]   = $team_each;//$playerInGame;
        // $response["player"] = $player_in_te;

        error_log( $team_each );

        return response()->json($response, 200);

    }

    public function get_handycap_by_player(PlayerInTeamEachCategory $player_in_team_each_category, $week, $league_id, $category_id, $team_id, $type, $rol_tournament, $tabla_posiciones = false, $semana_query = "<")
    {        

        // error_log("TEAM ID => ". $team_id);
        /**** JUGADOR DUMMY ****/
        //Si el jugador a procesar es un jugador DUMMY, retornaremos el HandyCap en 0
        if ($player_in_team_each_category->player_id <= 0) {
            // if( $team_id == 45 ){
            //     error_log("Se regresa un 0". $player_in_team_each_category);
            // }
            return 0;
        }    

        /**** CALCULO DE HANDYCAP ****/
        $player_rules = PlayerRules::where("league_id", $league_id)->first();
        $handicaps = Handicap::where("league_id", $league_id)->first();
        $rol = Rol::where("league_id", $league_id)->first();
        $handicap_each_categories = HandicapEachCategory::where("team_category_id", $category_id)->first();
        // error_log($league_id);
        //Un jugador nuevo lo determinaremos si el handycap inicial es null.
        $is_player_new = (!is_null($player_in_team_each_category->player_handy_cap_in_tournament)) ? false : true;        
                
        $cumple_primera_condicion = false;
        $cumple_segunda_condicion = false;

        //La diferencia entre la condición 1 y 2 es que la condición 1 le da ventaja a los juegadores nuevos de que se genere su handycap con menos jugadas
        //Comunmente se configura que los jugadores nuevos tienen que jugar al menos 2 partidos en primer semana mientras que los no nuevos tienen que acumular 3.
            
        $semana_uno_equipo_nuevo = false;

        $teams_added_team_new = TeamsAddedInCurrentTournament::where("league_id", $league_id)->where("type",1)->where("team_id",$team_id)->first();                        
        
        // error_log("teams_added_team_new ==>" .$teams_added_team_new );

        if( $teams_added_team_new ){
        
            if( ($teams_added_team_new->week-1) == $week ){   
                $semana_uno_equipo_nuevo = true;               
            }
            
            //SE SE CONSULTA UNA SEMANAS ATRAS DE LA SEMANA QUE INGRESO SE REGRESA 0                    
            if( ($teams_added_team_new->week - 1) > $week ){
                return 0;
            }

            //VERIFICAMOS QUE EQUIPO ES EL NUEVO EN LE TORNEO PARA CAMBIAR SU SEMANA INICIAL
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ($teams_added_team_new->week-1) )->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();                              
            

        }else{
            // error_log("SE cumple el segundo => ". $rol_tournament);
            //revisa la primer semana de los equipos normales
            $rol_tournaments = RolTournament::where("rol_id", $rol_tournament->rol_id)->where("category_id", $category_id)->where("iterator_week", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();                  
        }
        
        //Para jugadores nuevos, vamos a verificar que en la primer semana, el jugador haya jugado al menos la cantidad configurada en "Handicap", campo "Cuantos juegos debe de realizar en la primer semana"
        //Sacamos cuantos juegos realizo el jugador en la primer semana
        

        // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();               
        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournaments->id)->where("team_id", $team_id)->pluck("id")->toArray();
        $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->count();
        
        // error_log( print_r( $rol_tournament_sessions, true ) );

        // Si es 0 no se toma en cuenta la condicion
        if( $handicaps->news_players_how_many_games_play_on_first_week != 0 ){

            if ($rol_tournament_session_points >= $handicaps->news_players_how_many_games_play_on_first_week) {
                $cumple_primera_condicion = true;
            }

        }


        //SE REVISA SI LA SEMANA ANTERIOR ES UNA RONDA DE POSICIONES
        $rol_tournaments_prev = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", $week-1)->first();                 
                
        /**** SEGUNDA CONDICION ****/
        //Sean jugadores nuevos o no nuevos, se hace la condición 2
        //La segunda condición consiste en revisar cuantos partidos ha tenido el jugador desde el inicio. el jugador haya jugado al menos la cantidad configurada en "Handicap", campo "Cuantos juegos debe de jugar un jugador para obtener su Handicap nuevo"        
        
        // antes -  teams_added_team_new
        if( $semana_uno_equipo_nuevo ){            
            //Sacamos cuantos juegos realizo el jugador desde la primer semana el equipo nuevo
            $semanas_torneo_equipo_nuevo = $this->semanas_torneo_equipo_nuevo( $handicaps->weeks_to_complete_games, $week, $teams_added_team_new->week );                    
            // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ">", ($teams_added_team_new->week-1))->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();            
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereIn("iterator_week", $semanas_torneo_equipo_nuevo)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                    
        }else{
            
            $semanas_torneo = [];
            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("type", "!=", "ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<",$week)->where("suspended", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->whereRaw("(first_team_suspended = " . 0 . " OR second_team_suspended = " . 0 . ")")->pluck("id")->toArray();            
        
        }        
        
        //CONSULTA ANTERIOR       
        
        $player = Player::find($player_in_team_each_category->player_id);          
    
        // if( $player->id == 75 ){
            //EJEMPLO
            // error_log(print_r($rol_tournaments, true));
            
        foreach ($rol_tournaments as $key => $value) {
            
            $arr = array_slice($rol_tournaments, $key, $handicaps->weeks_to_complete_games);
            $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $arr)->where("team_id", $team_id)->pluck("id")->toArray();
            $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->count();                     
        
            if ($rol_tournament_session_points >= $handicaps->how_many_games_to_get_new_handicap) {            
                $cumple_segunda_condicion = true;    
                break;        
            }
        }
        // }

        
        //Primero revisamos si es la primera semana
        if ($week == 0 || $semana_uno_equipo_nuevo) {
           
            //Cuando es la semana 1, solamente verificaremos si colocamos el handycap inicial o en 0 dependiendo de lo que se configure en "Configuración de jugadores", campo " ¿Jugadores inician con handicap en "0"?"
            if ($player_rules->players_start_with_zero_handicap == 1) {
                return 0; //Jugadores inician con handycap en 0
            } else {                

                //Si es un jugador nuevo, verificamos si genero un handycap, si no genero le regresamos 0 ya que ira generando su handycap conforme se le vaya asignando puntaje
                if ($is_player_new) {

                    if ($cumple_primera_condicion) {

                        //Para la semana 1 (0) al menos para jugadores nuevos si generaremos su handycap de forma dinamica. Para jugadores viejos no generaremos su handycap.
                        //Para generar el nuevo handycap tenemos una formula que desglosaremos
                        //Suma de todos los Pinos tirados anteriores a la semana actual/cantidad de juegos jugados
                        
                        if( $semana_uno_equipo_nuevo ){                            
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", ($teams_added_team_new->week-1))->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        }else{                    
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        }                                            

                        $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();

                        $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->avg("points");
                        $base = $handicap_each_categories->base_normal;
                        
                        return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories);

                    } else {
                        return 0;
                    }

                } else {                    

                    // si estrue validara y contara la semana para la tabla de posiciones individual
                    if( $tabla_posiciones ){

                        if( $semana_uno_equipo_nuevo ){                            
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", ($teams_added_team_new->week-1))->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        }else{                    
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        }  

                        $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();

                        $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->avg("points");
                        $base = $handicap_each_categories->base_normal;
                        
                        return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories);
                    }
                    //Jugadores inician con handycap inicial indicado en "Jugadores", campo "HAndyCap inicial"
                    return $player_in_team_each_category->player_handy_cap_in_tournament;
                }

            }

        } else {            
                        
            
            /**** GENERAR NUEVO HANDYCAP ****/
            //Revisamos si cumple al menos una condición, si la cumple generamos su nuevo handycap.
            //Si no la cumple pierde su handycap y le colocamos el handycap en 0
            if ($cumple_primera_condicion || $cumple_segunda_condicion) {               
                
                
                //SE AÑADE ES TABLA PARA EL CONTROL DE EQUIPOS AÑADIDOS EN UN JUGO EN CURSOR
                //EN ESTE CASO ES LA SUSTITUCION DEL DUMMY
                //CUENTA A PARTIR DE LA SEMANA EN QUE SE AÑADIO EL EQUIPO Y SUS JUGADORES                           
                       
                
                //Para generar el nuevo handycap tenemos una formula que desglosaremos
                //Suma de todos los Pinos tirados anteriores a la semana actual/cantidad de juegos jugados   
                
                //JUEGOS SUSPENDIDOS INDIVIDUALES                
                $teams_added_partido = TeamsAddedInCurrentTournament::where("team_id", $team_id)->where("type",2)->where("league_id", $league_id)->where("week", "<" ,$week)->get();

                //REVISA SI HAY NUEVO EQUIPO SOLO DEBE SER UNO POR AHORA
                $teams_added_team_new = TeamsAddedInCurrentTournament::where("league_id", $league_id)->where("type",1)->where("team_id",$team_id)->first();                
                
                //SE VALIDA SI HAY JUEGOS SUSPENDIDOS
                //SI HAY JUEGO SUSPENDIDOS SE REALIZAN OTRAS CONSULTAS YA QUE HAY QUE OMITIR ESA SEMANA EN EL HANDICAP
                
                // //SE REVISA SI LA SEMANA ANTERIOR ES UNA RONDA DE POSICIONES
                // $rol_tournaments_prev = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", $week-1)->first();
                                
                
                if( count( $teams_added_partido ) > 0 ){                                                              
        
                    //REVISA SI HAY UN NUEVO EQUIPO
                    if( $teams_added_team_new ){  
                                                                        
                        //SE CONTARA A PARTIR DE LA SEMANA EN QUE INGRESO Y SE REVISA SI TIENE JUEGOS SUEPENDIDOS
                        //acumula las semanas a excluir                       
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        
                    }else{
                        
                        //NO HAY UN EQUIPO NUEVO SOLO JUEGOS SUSPENDIDOS                        
                        //acumula las semanas a excluir                                            
                        // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->whereIn("iterator_week", $semanas)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("first_team_suspended","=",0)->where("second_team_suspended","=",0)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        
                    }
                   
                }else{    
                                                          
                    if( $teams_added_team_new ){
                       
                        //SI LA SEMANA REGISTRADA ES MENOR A LA ACTUAL COMIENZA A CONTAR A PARTIR DE AHI                                                
                        if( ($teams_added_team_new->week-1) < $week ){                                                        
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->whereBetween("iterator_week",[ ($teams_added_team_new->week-1), ($week-1)])->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                                                                      

                        }else{                            
                            // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                       
                            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->whereBetween("iterator_week",[ ($teams_added_team_new->week - 1), ($week-1)])->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                                                                
                        }                                                

                    }else{                                                    
                                                
                        /*
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        */
                        
                        $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                        
                        // if( $player->id == 75 ){
                        //     error_log("ROL TORNAMET OK => teams_added_team_new". print_r($rol_tournaments , true));
                        // }
                        
                        // if( $rol_tournaments_prev->type == 'ronda_posiciones_virtual'){   

                            
                        // }else{
                            
                        //     if( $type == "ronda_posiciones_virtual" ){
                        //         // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                            
                        //     }else{                               

                        //         // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("suspended","=",0)->where("type","!=","ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();                                                                                                                                               
                                
                        //     }                                    

                        // }                        
                        
                    }                                         
                    
                } 
                                           
                
                $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();
                $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->avg("points");

                $base = $handicap_each_categories->base_normal;                               
                
                
                // VALIDA EN EQUIPOS AGREGADOS EN TORNEO EN CURSO QUE SI EXISTE UN JUGADOR Y NO HA TENIDO NUNGUN SCORE
                // TENDRA UN HANDUCAP DE 0
                if( $teams_added_team_new ){                    
                    
                    $r_t_s_p = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->count();                    

                    if( $r_t_s_p == 0 ){                        
                        return 0;
                    }

                }          
                
                return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories, $team_id);
                

            } else {
                return 0;
            }

        }

    }

    // public function get_handycap_by_player(PlayerInTeamEachCategory $player_in_team_each_category, $week, $league_id, $category_id, $team_id, $type, $rol_tournament)
    // {


    //     /**** JUGADOR DUMMY ****/
    //     //Si el jugador a procesar es un jugador DUMMY, retornaremos el HandyCap en 0
    //     if ($player_in_team_each_category->player_id <= 0) {
    //         return 0;
    //     }

    //     ///**** CALCULO DE HANDYCAP PARA RONDA DE POSICIONES VIRTUAL ****/
    //     ////Cuando sea una ronda de posiciones virtual, vamos a retornar el valor de handycap que tenga almacenado ya que corresponde al de la semana anterior del cual se hizo la copia
    //     //if ($type == "ronda_posiciones_virtual") {
    //     //
    //     //    $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournament->id)->where("team_id", $team_id)->first();
    //     //    $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_sessions->id)->where("player_id", $player_in_team_each_category->player_id)->first();
    //     //
    //     //    if ($rol_tournament_session_points != null) {
    //     //        return $rol_tournament_session_points->handycap_calculated;
    //     //    } else {
    //     //        return 0;
    //     //    }
    //     //
    //     //}

    //     /**** CALCULO DE HANDYCAP ****/
    //     $player_rules = PlayerRules::where("league_id", $league_id)->first();
    //     $handicaps = Handicap::where("league_id", $league_id)->first();
    //     $rol = Rol::where("league_id", $league_id)->first();
    //     $handicap_each_categories = HandicapEachCategory::where("team_category_id", $category_id)->first();

    //     //Un jugador nuevo lo determinaremos si el handycap inicial es null.
    //     $is_player_new = (!is_null($player_in_team_each_category->player_handy_cap_in_tournament)) ? false : true;
        

    //     $cumple_primera_condicion = false;
    //     $cumple_segunda_condicion = false;

    //     //La diferencia entre la condición 1 y 2 es que la condición 1 le da ventaja a los juegadores nuevos de que se genere su handycap con menos jugadas
    //     //Comunmente se configura que los jugadores nuevos tienen que jugar al menos 2 partidos en primer semana mientras que los no nuevos tienen que acumular 3.
            
    //     $semana_uno_equipo_nuevo = false;

    //     $teams_added_team_new = TeamsAddedInCurrentTournament::where("league_id", $league_id)->where("type",1)->where("team_id",$team_id)->first();                        

    //     error_log( "teams_added_team_new => ". $rol. " => category_id". $category_id. "team_id => ". $team_id );
        
    //     if( $teams_added_team_new ){

    //         if( ($teams_added_team_new->week-1) == $week ){                
    //             $semana_uno_equipo_nuevo = true;
    //         }
            
    //         //SE SE CONSULTA UNA SEMANAS ATRAS DE LA SEMANA QUE INGRESO SE REGRESA 0                    
    //         if( $teams_added_team_new->week > $week ){
    //             return 0;
    //         }

    //         //VERIFICAMOS QUE EQUIPO ES EL NUEVO EN LE TORNEO PARA CAMBIAR SU SEMANA INICIAL
    //         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ($teams_added_team_new->week-1) )->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();                  
    //         // error_log( "rol_tournaments => ".$rol_tournaments );

    //     }else{            
    //         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();       
    //     }
        
    //     //Para jugadores nuevos, vamos a verificar que en la primer semana, el jugador haya jugado al menos la cantidad configurada en "Handicap", campo "Cuantos juegos debe de realizar en la primer semana"
    //     //Sacamos cuantos juegos realizo el jugador en la primer semana
        

    //     // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->first();       
    //     error_log($rol_tournaments);
    //     $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $rol_tournaments->id)->where("team_id", $team_id)->pluck("id")->toArray();
    //     $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->count();

    //     if ($rol_tournament_session_points >= $handicaps->news_players_how_many_games_play_on_first_week) {
    //         $cumple_primera_condicion = true;
    //     }
                
    //     /**** SEGUNDA CONDICION ****/
    //     //Sean jugadores nuevos o no nuevos, se hace la condición 2
    //     //La segunda condición consiste en revisar cuantos partidos ha tenido el jugador desde el inicio. el jugador haya jugado al menos la cantidad configurada en "Handicap", campo "Cuantos juegos debe de jugar un jugador para obtener su Handicap nuevo"
        
    //     if( $teams_added_team_new ){
    //         //Sacamos cuantos juegos realizo el jugador desde la primer semana el equipo nuevo
    //         $semanas_torneo_equipo_nuevo = $this->semanas_torneo_equipo_nuevo( $handicaps->weeks_to_complete_games, $week, $teams_added_team_new->week );
    //         // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", ">", ($teams_added_team_new->week-1))->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereIn("iterator_week", $semanas_torneo_equipo_nuevo)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();        
    //     }else{
    //         $semanas_torneo = $this->semanas_torneo($handicaps->weeks_to_complete_games, $week);         
    //         //Sacamos cuantos juegos realizo el jugador desde la primer semana
    //         //actualizacion segun esto debe ser en base a configuracion de la liga
    //         // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereIn("iterator_week", $semanas_torneo)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();

    //     }
        
    //     //CONSULTA ANTERIOR
    //     // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();        

    //     //ACTUALIZACION SE REVISA SEMANA POR SEMANA PARA 
    //     $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();
    //     $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->count();
        
    //     if($teams_added_team_new){
    //         // error_log( "rol_tournament_sessions ==>--NUM=> ".$rol_tournament_session_points );
    //     }else{
    //         // error_log( "NORMAL ==>--NUM=> ".$rol_tournament_session_points );
    //     }

        
    //     if ($rol_tournament_session_points >= $handicaps->how_many_games_to_get_new_handicap) {
    //         $cumple_segunda_condicion = true;            
    //     }        
        
        
    //     $player = Player::find($player_in_team_each_category->player_id);
        
    //     // error_log("=============================================================");
    //     // error_log( $rol_tournament_session_points .">=". $handicaps->how_many_games_to_get_new_handicap );
    //     // error_log( "cumple_segunda_condicion ===> ".$cumple_segunda_condicion);
    //     // error_log( "cumple_primera_condicion ===> ".$cumple_primera_condicion);
    //     // error_log("player_id => ".$player->nickname);

    //     //Primero revisamos si es la primera semana
    //     if ($week == 0 || $semana_uno_equipo_nuevo) {

    //         //Cuando es la semana 1, solamente verificaremos si colocamos el handycap inicial o en 0 dependiendo de lo que se configure en "Configuración de jugadores", campo " ¿Jugadores inician con handicap en "0"?"
    //         if ($player_rules->players_start_with_zero_handicap == 1) {
    //             return 0; //Jugadores inician con handycap en 0
    //         } else {
    //             //Si es un jugador nuevo, verificamos si genero un handycap, si no genero le regresamos 0 ya que ira generando su handycap conforme se le vaya asignando puntaje
    //             if ($is_player_new) {

    //                 if ($cumple_primera_condicion) {

    //                     //Para la semana 1 (0) al menos para jugadores nuevos si generaremos su handycap de forma dinamica. Para jugadores viejos no generaremos su handycap.
    //                     //Para generar el nuevo handycap tenemos una formula que desglosaremos
    //                     //Suma de todos los Pinos tirados anteriores a la semana actual/cantidad de juegos jugados

    //                     if( $semana_uno_equipo_nuevo ){                            
    //                         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", ($teams_added_team_new->week-1))->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //                     }else{                    
    //                         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //                     }
                        
    //                     $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();

    //                     $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->avg("points");
    //                     $base = $handicap_each_categories->base_normal;
                        
    //                     return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories);

    //                 } else {
    //                     return 0;
    //                 }

    //             } else {

    //                 //Jugadores inician con handycap inicial indicado en "Jugadores", campo "HAndyCap inicial"
    //                 return $player_in_team_each_category->player_handy_cap_in_tournament;
    //             }

    //         }

    //     } else {

    //         /**** GENERAR NUEVO HANDYCAP ****/
    //         //Revisamos si cumple al menos una condición, si la cumple generamos su nuevo handycap.
    //         //Si no la cumple pierde su handycap y le colocamos el handycap en 0
    //         if ($cumple_primera_condicion || $cumple_segunda_condicion) {
                
                
    //             //SE AÑADE ES TABLA PARA EL CONTROL DE EQUIPOS AÑADIDOS EN UN JUGO EN CURSOR
    //             //EN ESTE CASO ES LA SUSTITUCION DEL DUMMY
    //             //CUENTA A PARTIR DE LA SEMANA EN QUE SE AÑADIO EL EQUIPO Y SUS JUGADORES
    //             $teams_added_in_current_tournament =  TeamsAddedInCurrentTournament::where("team_id", $team_id)->where("league_id", $league_id)->where("week", $week)->first();
                
    //             //Para generar el nuevo handycap tenemos una formula que desglosaremos
    //             //Suma de todos los Pinos tirados anteriores a la semana actual/cantidad de juegos jugados   
                
    //             //JUEGOS SUSPENDIDOS INDIVIDUALES                
    //             $teams_added_partido = TeamsAddedInCurrentTournament::where("team_id", $team_id)->where("league_id", $league_id)->where("week", "<" ,$week)->get();
                
    //             //REVISA SI HAY NUEVO EQUIPO SOLO DEBE SER UNO POR AHORA
    //             //$teams_added_team_new = TeamsAddedInCurrentTournament::where("league_id", $league_id)->where("type",1)->where("team_id",$team_id)->first();                
                
    //             //SE VALIDA SI HAY JUEGOS SUSPENDIDOS
    //             //SI HAY JUEGO SUSPENDIDOS SE REALIZAN OTRAS CONSULTAS YA QUE HAY QUE OMITIR ESA SEMANA EN EL HANDICAP
                
    //             //SE REVISA SI LA SEMANA ANTERIOR ES UNA RONDA DE POSICIONES
    //             $rol_tournaments_prev = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "=", $week-1)->first();
                                
    //             if( count( $teams_added_partido ) > 0 ){
                    
    //                 //REVISA SI HAY UN NUEVO EQUIPO
    //                 if( $teams_added_team_new ){
    //                     //SE CONTARA A PARTIR DE LA SEMANA EN QUE INGRESO Y SE REVISA SI TIENE JUEGOS SUEPENDIDOS
    //                     //acumula las semanas a excluir
    //                     $acumulado = $this->acumulaIdsAExcluir( $teams_added_partido );
    //                     error_log("SI hay equipo =>" );
    //                     //SEMANAS QUE CONSULTARA
    //                     $semanas = $this->excluir_semanas_new($week, $acumulado, ($teams_added_team_new->week - 1));

    //                     $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereIn("iterator_week", $semanas)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();

    //                 }else{

    //                     //NO HAY UN QUIPO NUEVO SOLO JUEGOS SUSPENDIDOS
    //                     error_log("no hay equipo");
    //                     //acumula las semanas a excluir
    //                     $acumulado = $this->acumulaIdsAExcluir( $teams_added_partido );
    //                     //SEMANAS QUE CONSULTARA
    //                     $semanas = $this->excluir_semanas_new($week, $acumulado);

    //                     $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereIn("iterator_week", $semanas)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();

    //                 }
                   
    //             }else{
                    
                    
    //                 if( $teams_added_team_new ){
    //                     // error_log("normal =====================". $teams_added_team_new);
    //                     //SI LA SEMANA REGISTRADA ES MENOR A LA ACTUAL COMIENZA A CONTAR A PARTIR DE AHI                        
    //                     // error_log( "team added team new ======>>>>" );
    //                     if( $teams_added_team_new->week < $week ){
                            
    //                         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereBetween("iterator_week",[ $teams_added_team_new->week, $week])->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                            
    //                     }else{
    //                         // $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
                            
    //                         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->whereBetween("iterator_week",[ ($teams_added_team_new->week - 1), $week])->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //                     }
                        
    //                 }else{
                                                
    //                     /*
    //                     $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //                     */
                        
    //                     if( $rol_tournaments_prev->type == 'ronda_posiciones_virtual'){                            
    //                         //SI ES RONDA DE POSICIONES SE ANULA EL ID PARA QUE NO REALICE EL CONTEO DE HANDICAP
    //                         //PARA QUE ES SU SIGUIENTE SEMANA COMIENCEN LOS JUGADORES CON EL MISMO HANDYCAP                            
    //                         error_log("WEEK ==> ".$week);
    //                         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week-3)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //                     }else{
    //                         $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("iterator_week", "<", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
    //                         // error_log("semana week => ". print_r($rol_tournaments, true));
    //                     }
                        
                        
    //                 }            
                    
    //             }            

    //             $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();
                
    //             $promedio = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $player_in_team_each_category->player_id)->avg("points");
    //             $base = $handicap_each_categories->base_normal;                                

    //             return $this->calcular_handycap($player, $promedio, $base, $handicap_each_categories);
                

    //         } else {
    //             return 0;
    //         }

    //     }

    // }

    private function acumulaIdsAExcluir( $suspendidos ){

        $result = array();

        foreach ($suspendidos as $key => $value) {
            array_push( $result, $value->week );
        }

        return $result;

    }

    private function excluir_semanas_new( $week, $acumulado, $inicio = 0 ){
        $valor = array();
        for ($i = $inicio; $i <= $week; $i++) {
            if( $i !== $this->excluir_semanas( $acumulado, $i ) ){
                array_push($valor, $i);
            }                        
        }
        return $valor;   
    }

    private function excluir_semanas( $semanas, $num ){
        foreach ($semanas as $key => $value) {
            if( $value == $num ){
                return $num;
            }
        }        
    }

    private function semanas_torneo_equipo_nuevo( $weeks_to_complete_games, $week, $week_equipo_new ){ 
        $ids = [];
        // 1 4 4
        if( $week >= $week_equipo_new ){
            for ($i = $week-$weeks_to_complete_games; $i <= $week; $i++) {            
                if( !$this->is_negative_number($i) ){
                    if( $i > $week_equipo_new ){
                        array_push( $ids, $i );
                    }
                }
            }
            return $ids;
        }else{
            return [];
        }

    }

    private function semanas_torneo( $weeks_to_complete_games, $week ){      
        $ids = [];
        for ($i = $week-$weeks_to_complete_games; $i <= $week; $i++) {
            // error_log( "weeks_to_complete_games". $i );
            if( !$this->is_negative_number($i) ){
                array_push( $ids, $i );
            }
        }

        return $ids;
    }

    public function calcular_handycap($player, $promedio, $base, $handicap_each_categories)
    {

        $porcentaje = 0;
        $tope = 0;

        //2.- Obtenemos el jugador para saber a que categoria pertenece, si es especial, joven, masculino o femenino (se toma ese orden de importancia)
        //Obtenemos el valor del porcentaje a multiplicar el promedio y el tope
        if ($player->player_type == "special") {
            $porcentaje = ($handicap_each_categories->especial_normal / 100);
            $tope = $handicap_each_categories->especial_normal_tope;
        } elseif ($player->player_type == "young") {
            $porcentaje = ($handicap_each_categories->juvenil_normal / 100);
            $tope = $handicap_each_categories->juvenil_normal_tope;
        } elseif ($player->player_type == "male") {
            $porcentaje = ($handicap_each_categories->male_normal / 100);
            $tope = $handicap_each_categories->male_normal_tope;
        } elseif ($player->player_type == "female") {
            $porcentaje = ($handicap_each_categories->female_normal / 100);
            $tope = $handicap_each_categories->female_normal_tope;
        }


        //3.- Generamos la formula
        if ($promedio >= $base) {
            return 0;
        } else {

            $handycap = (($base - $promedio) * $porcentaje);
            $handycap_decimal = $handycap - floor($handycap);

            //Redondeamos el handycap
            if ($handycap_decimal <= 0.5) {
                $handycap = floor($handycap); //Redondeo hacia abajo
            } else {
                $handycap = floor($handycap) + 1; //Redondeo hacia arriba
            }

            if ($handycap > $tope) {
                return $tope;
            } else {
                return $handycap;
            }

        }

    }

    public function practicas( Request $request ){

        $response = array();
        $response["message"] = "";
        $practicas = Practice::where("id_player", $request->id_player )->get();

        $response["practicas"] = $practicas;

        return response()->json($response);
    }

    public function add_practica( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = new Practice();
        $practica->id_player     = $request->id_player;
        $practica->name         = $request->name;
        $practica->name_bowling = $request->name_boliche;
        $practica->ball         = $request->ball;
        $practica->marker_type  = $request->tipo_marcador;
        $practica->score        = $request->score;

        if( $practica->save() ){
            //si es 2 es por pinos se añade un juego a la tabla practice_for_games
            //para llevar el control de juegos por ronda
            if(  $request->tipo_marcador == 2 ){   
                $practica_por_juego =  new PracticeForGames();
                $practica_por_juego->id_practice = $practica->id;//$practica->id_player;
                
                $practica_por_juego->save();

            }

            $response["message"] = "Se guardo correctamente";
            return response()->json($response);
        }else{
            $response["message"] = "Error al guardar practica";
            return response()->json($response);
        }

    }

    public function practicas_play_id( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = PracticesPlay::where("id_practice",$request->id_practice)->get();

        $response["data"] = $practica;
        return response()->json($response);

    }

    public function add_score( Request $request ){
        $response = array();
        $response["message"] = "";

        $practice_play = new PracticesPlay();

        $practice_play->id_practice = $request->id_practice;
        $practice_play->score = $request->score;

        if( $practice_play->save() ){
            
            $practica = Practice::find( $request->id_practice );
            $practica->finalize = 3;
            $practica->save();

            $response["message"] = "Se guardo correctamente";
            return response()->json($response);
        }else{
            $response["message"] = "Error al guardar practica";
            return response()->json($response);
        }

    }

    public function get_practica_id( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = Practice::find( $request->id );

        $response["data"] = $practica;
        return response()->json($response);        

    }

    public function status_practica( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = Practice::find( $request->id_practice );
        $practica->finalize = $request->finalize;

        if( $practica->save() ){
            $response["message"] = "Se guardo correctamente";
            return response()->json($response);
        }else{
            $response["message"] = "Error al guardar status";
            return response()->json($response);
        }   

    }

    public function status_practica_por_pinos( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = PracticeForGames::find( $request->id );
        $practica->finalize    = $request->finalize;
        $practica->score_final = $request->score_final;

        if( $practica->save() ){

            $practica_for_games = Practice::find( $request->id_practice );
            $practica_for_games->finalize = 1;
            $practica_for_games->save();

            $response["message"] = "Se guardo correctamente";
            return response()->json($response);
        }else{
            $response["message"] = "Error al guardar status";
            return response()->json($response);
        }   

    }

    // por score individual
    public function practica_anterior( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = Practice::where('id', '<', $request->id)->where("finalize",1)->where("marker_type",$request->tipo)->orderBy('id', 'desc')->first();
        if($practica){

            if( $request->tipo == 1 ){

                $califi = PracticesPlay::where("id_practice",$practica->id)->get();
                $response["status"] = true;
                $response["data"] = $practica;
                $response["data_califi"] = $califi;

            }

            if( $request->tipo == 2 ){
                $prac_games = PracticeForGames::where("id_practice", $practica->id)->where("finalize",1)->get();
                $response["status"] = true;
                $response["data"] = $prac_games;
            }

            
        }else{
            $response["status"] = false;
        }
        
        return response()->json($response);
    }

    public function delete_practica( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = Practice::find($request->id);

        if( $practica->delete() ){

            $califi = PracticesPlay::where("id_practice",$practica->id)->delete();
            //En caso de que sea practica por pinos se eliminan los juegos tambien
            if( $practica->marker_type == 2 ){
                $practica_por_juego = PracticeForGames::where( "id_practice", $practica->id)->delete();
                $califiDos = PracticesPlay::where("id_practice_for_games",$practica->id)->delete();
            }

            $response["status"] = true;
            $response["mensaje"] = "La practica se elimino correctamente";

        }else{
            $response["status"] = true;
            $response["mensaje"] = "Error al eliminar practica.";
        }
        return response()->json($response);

    }

    public function juegos_por_pinos( Request $request ){

        $response = array();
        $response["message"] = "";

        $practica = PracticeForGames::where('id_practice', $request->id)->orderBy('id', 'asc')->get();

        if( $practica ){
            $response["status"] = true;
            $response["data"] = $practica;
        }else{
            $response["status"] = true;
            $response["mensaje"] = "Error al consultar practicas.";
        }

        return response()->json($response);

    }

    public function score_game( Request $request ){
        $response = array();
        $response["message"] = "";
        
        // $califi = PracticesPlay::where("id_practice",$request->id_practice)->get();
        $califi = PracticesPlay::where("id_practice_for_games",$request->id_practice_for_games)->where("id_practice",$request->id_practice)->get();

        if( $califi ){

            // $calificaciones = array();

            // foreach ($califi as $key => $value) {
            //     # code...
            //     $calificaciones["thrown_pines"] = unserialize( $value->thrown_pines );
            // }            

            $response["status"] = true;
            $response["data"] = $califi;
            // $response["data_cali"] = $calificaciones;
        }else{
            $response["status"] = true;
            $response["mensaje"] = "Error al consultar practicas.";
        }

        return response()->json($response);

    }

    public function add_score_for_pines( Request $request ){
        
        $response = array();
        $response["message"] = "";

        $practice_play = new PracticesPlay();

        $practice_play->id_practice           = $request->id_practice;
        $practice_play->score                 = $request->score;
        $practice_play->id_practice_for_games = $request->id_practice_for_games;
        $practice_play->thrown_pines          = $request->thrown_pines;
        $practice_play->score_order           = $request->id_score_order;
        // $practice_play->id_practice_for_games = $request->id_practice_for_games;

        if( $practice_play->save() ){
            
            
            if( $request->score == 10 or $request->score == "10" ){
                
                if( $request->primer_tiro_cero == false ){

                    $practice_play_dos = new PracticesPlay();
                    
                    $practice_play_dos->id_practice           = $request->id_practice;
                    $practice_play_dos->score                 = -1;
                    $practice_play_dos->id_practice_for_games = $request->id_practice_for_games;
                    $practice_play_dos->thrown_pines          = $request->thrown_pines;
                    $practice_play_dos->score_order           = $request->id_score_order;
                    
                    $practice_play_dos->save();
                }

                
                // $practica = PracticeForGames::where('id_practice', $request->id_practice)->update(['finalize' => 3]);
            }
            
            // $practica = Practice::find( $request->id_practice );
            // $practica->finalize = 3;
            // $practica->save();

            $response["message"] = "Se guardo correctamente";
            return response()->json($response);
        }else{
            $response["message"] = "Error al guardar practica";
            return response()->json($response);
        }

    }

    public function update_conekta_id( Request $request ){

        $response = array();
        $response["message"] = "";

       
        $player = Player::where("id", $request->id)->update(['client_id_conekta'=> $request->client_id_conekta]);
        
        $response["player"] = $player;
        
        return response()->json($response, 200);

    }

    public function update_url_image_profile( Request $request, $id ){

        $response = array();
        $response["message"] = "";

        $player = Player::findOnFail($id);
        $player->fill( $request->update );

        $player->save();

    }

    public function upload_imagen( Request $request){

        $response = array();
        $response["message"] = "";
 
        $target_path = public_path("uploads/");//"public/uploads/";
        $target_path = $target_path . basename( $_FILES['file']['name']);
 
        if(move_uploaded_file($_FILES['file']['tmp_name'], $target_path)) {
            header('Content-type: application/json');
            $data = ['success' => true, 'message' => 'Upload and move success'];
            echo json_encode( $data );
        } else{
            header('Content-type: application/json');
            $data = ['success' => false, 'message' => 'There was an error uploading the file, please try again!'];
            echo json_encode( $data );
        }      

     
    }

    public function update_url_img_player( Request $request ){
        $response = array();
        $response["message"] = "";

        // $player = Player::where("id", $request->id)->update(['img_profile'=> $request->url_img]);
        $player = Player::find($request->id);
        
        $response["player"] = $player;

        return response()->json($response, 200);
        
    }

    public function add_game_por_pinos( Request $request ){
        
        $response = array();
        $response["message"] = "";

        $practica_por_juego =  new PracticeForGames();
        $practica_por_juego->id_practice = $request->id_practice;

        if( $practica_por_juego->save() ){
            return response()->json($response, 200);
        }else{
            $response["message"] = "Error al añadir juego";
            return response()->json($response);
        }

    }

    public function login_facebook( Request $request ){
        
        $response["data"] = "";
        $response["mensaje"] = "";

        $validators = $this->validate($request,[            
            'email'  =>'required|email',
            'id'     =>'required'                                    
        ]);
        
        //->where("user_id_facebook", $request->id)
        $is_player_exists = Player::where("email", $request->email)->first();
        
        if( $is_player_exists ){

            $id = Player::where("user_id_facebook", $request->id)->first();

            //Primero eliminamos los token ascociados a este usuario
            AppUserTokenSession::where("player_id", $is_player_exists->id)->delete();

            //Generamos el token
            $app_user_token_sessions = new AppUserTokenSession();
            $app_user_token_sessions->access_token = md5(date("Y-m-d H:i:s") . "-" . $is_player_exists->id);
            $app_user_token_sessions->player_id = $is_player_exists->id;
            $app_user_token_sessions->expirated_at = Carbon::now()->addYears(2);
            $app_user_token_sessions->save();

            $response["message"] = "Ingreso correcto.";
            $response["token"] = $app_user_token_sessions->access_token;
            return response()->json($response);

        }else{
            //Registra el usuario en base de datos

            $user = new Player();
            
            $user->name      = $request->first_name;
            $user->last_name = $request->last_name;
            $user->nickname  = $request->first_name;
            $user->email     = $request->email;
            $user->user_id_facebook = $request->id;
            $user->img_profile      = $request->picture;
        
            if ($user->save()) {

                //Enviamos por correo electronico el acceso
                // \Mail::send('mail.register_app_facebook', ['user' => $user], function ($m) use ($user) {
                //     $m->from('soporte@beonline.com', 'BeOnline');
                //     $m->to($user->email, $user->name)->subject('Bienvenido a BeOnline');
                // });
    
                //Generamos el token
                $app_user_token_sessions = new AppUserTokenSession();
                $app_user_token_sessions->access_token = md5(date("Y-m-d H:i:s") . "-" . $user->id);
                $app_user_token_sessions->player_id = $user->id;
                $app_user_token_sessions->expirated_at = Carbon::now()->addYears(2);
                $app_user_token_sessions->save();
    
                $response["message"] = "Te hemos registrado correctamente, bienvenido a BeOnline.";
                $response["token"] = $app_user_token_sessions->access_token;
                return response()->json($response);
    
            } else {
    
                //No existe el usuario o credenciales incorrectas
                $response["message"] = "Correo electrónico o contraseña incorrectos, verifica el acceso a tu cuenta.";
                return response()->json($response, 504);
    
            }

        }

        // $response["player"] = $is_player_exists;

        // return response()->json($response);
    }

    /*
        practices_plays.score_order status donde se maneja calificacion por pinos
        1: primer calificacion
        2: segunda calificacion
        3: chuza
    */

    public function finalize_score( Request $request ){
        /*
            0.- default
            1.- score finalizado
            2.- add un juego mas
            3.- pendiente de confirmar si continua calificando o finalizara esto en caso 
                de que salga de la app
        */

        $response = array();
        $response["message"] = "";
    }  
    
    /*  
        Se añade score_order para saber en caso de jugar por bolos 
        saber el tipo de tipo que tiene, si tiro 4 despues tendra otro tiro

    */

    public function tabla_posiciones_individual( Request $request ){
        
        $response["status"]  = false;  
            
        $team_each = TeamEachCategory::where("team_category_id", $request->category_id)->get();  
        $rol = Rol::where("league_id", $request->id_liga)->first();        

        $data_result = [];

        foreach($team_each as $key => $team_id){        
            //$request->week_number
            $result = $this->player_tabla_posiciones(null, $request->id_liga, $request->category_id, $request->week_number, $team_id->team_id, $rol, "<=");
            array_push($data_result, $result);
        }
                
        $response["data"] = $data_result;
        $response["num"]  = count( $team_each ); 

        return response()->json( $response );
    }

    private function player_tabla_posiciones( $puntaje = null, $league_id = null, $category_id, $week, $team_id, $rol, $semana_query = "<=" ){
        
        // $rol = Rol::where("league_id", $league_id)->first();
      
        // foreach ($puntaje as $team_id => $series) {                          
            $rol_tournament = RolTournament::where("iterator_week", $week)->where("category_id", $category_id)->first();

            $rol_tournaments = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("type", "!=", "ronda_posiciones_virtual")->where("suspended", 0)->where("iterator_week", "<=", $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->pluck("id")->toArray();
            $rol_tournament_sessions = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->pluck("id")->toArray();

            // error_log( "player_tabla_posiciones=>>". $rol->id."====".$category_id."===". $semana_query."||||====".$week );   

            $rol_tournaments_recalcular_handycap = RolTournament::where("rol_id", $rol->id)->where("type", "!=", "ronda_posiciones_virtual")->where("category_id", $category_id)->where("iterator_week", $semana_query, $week)->where("suspended", 0)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->get();

            //puntaje mayor sin hc
            $rol_tournament_ses_comp = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->max("sum_puntaje");//->get();
            $rol_tournament_ses_comp_hc = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->max("sum_puntaje_handycap");//->get();            
                        
            $team_each = TeamEachCategory::where("team_category_id", $category_id)->where("team_id", $team_id)->first();            
            //$team = Team::find($team_each->team_id);

            $rol_tournaments_team = RolTournament::where("rol_id", $rol->id)->where("category_id", $category_id)->where("type", "!=", "ronda_posiciones_virtual")->where("suspended", 0)->where("iterator_week", $semana_query, $week)->whereRaw("(first_team_id = " . $team_id . " OR second_team_id = " . $team_id . ")")->get();

                $serie_equipo        = array();
                $serie_equipo_hc     = array();
                $juego_equipo        = array();
                $juego_equipo_all    = array();
                $juegos_jugados_week = array();
                
                
                foreach ($rol_tournaments_team as $key => $puntaje) {

                    $rol_t_session = RolTournamentSession::select("id","rol_tournament_id","sum_puntaje", "sum_puntaje_handycap")
                                        ->where("rol_tournament_id", $puntaje->id)                                        
                                        ->where("team_id", $team_id)->get();
                    

                    array_push( $juego_equipo, array( "week" => $puntaje->iterator_week, "puntajes" => $rol_t_session, "session_points" => $this->juegos_individuales( $rol_t_session ) ) );

                    
                    // $rol_t_s = RolTournamentSession::where("rol_tournament_id", $puntaje->id)->where("team_id", $team_id)->get();
                    $series_equipo = $this->series_equipo( $rol_t_session, $team_id );
                    array_push($serie_equipo, $series_equipo["puntaje_serie"]);
                    array_push($serie_equipo_hc, $series_equipo["puntaje_serie_hc"]);
                    
                }                
            
            $team = Team::find($team_id);
            
            $players = TeamEachCategory::where("team_category_id", $category_id)->where("team_id", $team_id)->with("get_player_in_team_each_category.get_player")->first();
            
            $data = [];
                        
            foreach ($players["get_player_in_team_each_category"] as $key => $value) {

                $player_in_team = PlayerInTeamEachCategory::where("team_each_categories_id", $team_each->id)->where("player_id", $value->player_id)->first();                

                $genero   = $value->get_player->gender;
                $nickname = $value->get_player->nickname;
                $name     = $value->get_player->name;
                $lastname = $value->get_player->last_name;
                //juegos jugados
                $rol_tournament_session_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $value->player_id)->count();
                $rol_t_s_p_max_points          = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $value->player_id)->get();

                $juego_perfecto_status = false;

                // Juegos perfectos
                $juegos_perfectos  = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sessions)->where("player_id", $value->player_id)->where("points", 300)->get();

                if( count($juegos_perfectos) > 0 ){
                    $juego_perfecto_status = true;
                }

                error_log( "juegos perfect =>". json_encode($juegos_perfectos));

                $puntajes = $this->suma_puntaje( $rol_t_s_p_max_points, $rol_tournament_session_points, $value, $week, $league_id, $category_id, $team, $rol_tournament );                
                $p_serie  = $this->suma_serires_player_web($rol_tournaments, $team_id,  $value->player_id);//$this->suma_serires_player( $rol_tournaments, $team_id,  $value->player_id );
                
                // error_log( "p_serie ===>". json_encode($p_serie) );

                $hc = $this->get_handycap_by_player( $player_in_team, $week, $league_id, $category_id, $team_id, "partido", $rol_tournament, true);

                array_push( $data, array(   "team_name" => $team->name, 
                    "genero"    => $genero, 
                    "name"      => $name, 
                    "nickname"  => $nickname ? $nickname: $name. " ".$lastname, 
                    "val_mas_alto"    => $puntajes["val_mas_alto"], 
                    "val_mas_alto_hc" => $puntajes["val_mas_alto_hc"], 
                    "lastname"        => $lastname,
                    "promedio"        => $puntajes["promedio"], 
                    "promedio_hc"     => $puntajes["promedio_hc"],
                    "player_id"       => $value->get_player->id, 
                    "serie_max"       => $p_serie["serie"],
                    "serie_max_hc"    => $p_serie["serie_hc"], 
                    "juegos_jugados"  => $puntajes["juegos_jugados"],
                    "total" => $puntajes["total"], 
                    "handicap" => $hc,
                    "player_id" => $value->get_player->id, 
                    "juego_equipo" => $rol_tournament_ses_comp,
                    "juego_equipo_hc" => $rol_tournament_ses_comp_hc, 
                    "team_id" => $team_id,
                    "serie_equipo" => count($serie_equipo) > 0 ? max($serie_equipo): 0, 
                    "serie_equipo_hc" => count($serie_equipo_hc) > 0 ? max($serie_equipo_hc):0,
                    "juegos_jugados" => count($rol_t_s_p_max_points), 
                    "serie_equipo_all" => $serie_equipo,
                    "serie_quipo_hc_all" => $serie_equipo_hc,                                              
                    "juegos_equipo_all" => $juego_equipo, 
                    "juegos_individuales_all" => $rol_t_s_p_max_points,
                    "juego_perfecto" => $juego_perfecto_status
                    // "juegos_jugados_week" => $juegos_jugados_week
                ));

            }
            
            return $data;
            
    }

    private function juegos_individuales( $rol_t_session ){
        $push = array();
        foreach ($rol_t_session as $key => $value) {
            $rol_tournament_session = RolTournamentSessionPoint::where("rol_tournament_session_id", $value->id )->get();            
            array_push( $push, array( "key"=> $key, "data_session_point" => $rol_tournament_session ) );
        }        

        return $push;

    }

    private function series_equipo( $rol_t, $team_id ){        

        $puntaje_serie_arr = array();
        $puntaje_serie = 0;
        $puntaje_serie_hc = 0;        
        
        foreach ($rol_t as $key => $value) {           
            $puntaje_serie = $puntaje_serie + $value->sum_puntaje;
            $puntaje_serie_hc = $puntaje_serie_hc + $value->sum_puntaje_handycap;
          
        }
        
        return array( "puntaje_serie" => $puntaje_serie, "puntaje_serie_hc" => $puntaje_serie_hc );
        
    }   

    // private function toFixed($number, $decimals) {
    //     return number_format($number, $decimals, '.', "");
    // }

    private function suma_puntaje( $puntajes, $rol_torunament, $player ){
        
        $num = 0;
        $num_hp = 0;
        $num_puntaje = 0;
        $valor_max = [];
        $valor_max_hc = [];
        $handicap_arr = [];
        $handicap = 0;
        $puntaje_hc = 0;
        $total = 0;        
        // error_log( $player );
        if( $player->id == 87 ){
            // error_log( "PUNTOS => ". json_encode( $puntajes ) );
            // error_log( "rol_torunament => ". $rol_torunament );
        }

        // $this->get_handycap_by_player( $player_in_team_each_category, $week, $league_id, $category_id, $team_id, $type, $rol_tournament);

        if( count( $puntajes ) == 0 ){              
            $result = array( "handicap" => 0, "total" => $total, "juegos_jugados" => 0, "val_mas_alto" => 0, "val_mas_alto_hc" => 0, "promedio" => 0, "promedio_hc" => 0 );
            return $result;
        }
        // error_log( "score => ". $puntajes);
        
        foreach ($puntajes as $key => $value) {
            
            $num_puntaje++;
            $num   = $num + $value->points;
            $total = $num;
            $puntaje_hc = $puntaje_hc + $value->points + $value->handycap_calculated;
            $handicap = $value->handycap_calculated;
            array_push( $handicap_arr, $value->handycap_calculated );
            array_push( $valor_max, $value->points ); //valor maximo sin handicap
            array_push( $valor_max_hc, $value->points + $value->handycap_calculated ); //valor maximo con handicap   
            if( $player->id == 87 ){
                // error_log( "handycap_calculated =>". $value->handycap_calculated );
            }         
        
        }        

        // error_log( "num puntaje =>". print_r( $puntaje_hc, true ) );

        $max    = max($valor_max); //max valor sin hc
        $max_hc = max($valor_max_hc); // max valor con hc

        $promedio    = $num / $num_puntaje;
        $promedio_hc = $puntaje_hc / $num_puntaje;            

        if( $player->id == 87 ){
            // round( number_format( $promedio, 2 ), 1, PHP_ROUND_HALF_UP)
            // error_log("handicap_arr ==== >". print_r( $handicap_arr, true ) );
        }

        // error_log( round( $promedio, 1, PHP_ROUND_HALF_UP) );
        // error_log( "promedio =>". $this->redondeo($promedio) );
        // error_log( "promedio_hc =>". $this->redondeo($promedio_hc) );
        //round( number_format( $promedio, 2 ), 1, PHP_ROUND_HALF_UP)
        $result = array( "handicap" => $handicap, "total" => $total, "juegos_jugados" => count( $puntajes ), "val_mas_alto" => $max, "val_mas_alto_hc" => $max_hc, "promedio" => round( number_format( $promedio, 2 ), 1, PHP_ROUND_HALF_UP), "promedio_hc" => $promedio_hc );
        return $result;
    }

    private function suma_serires_player_web( $rol_tournaments, $team_id, $player_id ){

        $series = 0;
        
        $sum = 0;
        $sum_hc = 0;
        $max_serie = [0];
        $max_serie_hc = [0];
        $sum_serires_total = [];
        $suma_puntaje = array();

        if( $team_id == 9 ){
            // error_log( print_r( $rol_tournaments, true ) );
            // error_log( print_r($juegos_equipo, true) );
        }
        
        // $rol_tournament_ses_comp = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->max("sum_puntaje");//->get();

        if( $team_id == 9 ){
            // error_log( print_r( $rol_tournament_ses_comp, true ) );
            // error_log( print_r($juegos_equipo, true) );
        }

        foreach( $rol_tournaments as $key => $value ){                    
            
            $rol_tournament_ses = RolTournamentSession::where("rol_tournament_id", $value)->where("team_id", $team_id)->pluck("id")->toArray();
            $rol_t_s_p_max_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_ses)->where("player_id", $player_id )->get();            
            
            $puntos = $this->suma_puntaje_semana( $rol_t_s_p_max_points, $player_id );
            // $juegos_equipo = $this->suma_puntaje_semana_juego( $rol_tournament_ses_comp, $team_id ); 

            // error_log("juegos_equipo => ". print_r($juegos_equipo, true));
            
            // $puntaje_juegos = $this->puntaje_juegos( $juegos_equipo );
                        
            // array_push( $suma_puntaje, $puntaje_juegos );

            // error_log( print_r( $puntaje_juegos, true ) );

            // error_log("value => ". print_r( $value, true ));
            
            array_push($max_serie, $puntos["puntos"]);
            array_push($max_serie_hc, $puntos["sum_hc"]); 
                                                      

        }

        if( $team_id == 9 ){
            // $juegos_equipo = $this->suma_puntaje_semana_juego( $rol_tournament_ses_comp, $team_id );
            // error_log( "SUMA PUNTAJE =>". json_encode($suma_puntaje) );
            // error_log( print_r( array_merge( $suma_puntaje), true ) );
            // error_log(print_r($juegos_equipo, true));
        }

        $max_serie_num    = max($max_serie);
        $max_serie_num_hc = max($max_serie_hc);   
        

        $result = array( "serie" => $max_serie_num, "serie_hc" => $max_serie_num_hc );
        // error_log("================================");
        return $result;


    }

    private function suma_serires_player( $rol_tournaments, $team_id, $player_id ){

        $series = 0;
        
        
        $rol_tournament_sesion = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->max("sum_puntaje");
        $rol_tournament_sesion_hc = RolTournamentSession::whereIn("rol_tournament_id", $rol_tournaments)->where("team_id", $team_id)->max("sum_puntaje_handycap");

        if( $rol_tournament_sesion >= 799 ){
            error_log(json_encode($rol_tournament_sesion). "team ==>". $team_id);
        }
        // $rol_t_s_p_max_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_sesion)->where("player_id", $player_id )->get();            
        error_log( $rol_tournament_sesion );
        
        

        // foreach( $rol_tournaments as $key => $value ){
            // $sum = 0;
            // $sum_hc = 0;
            // $max_serie = [0];
            // $max_serie_hc = [0];
            // // error_log( $value );
            // $rol_tournament_ses = RolTournamentSession::where("rol_tournament_id", $value)->where("team_id", $team_id)->pluck("id")->toArray();
            // $rol_t_s_p_max_points = RolTournamentSessionPoint::whereIn("rol_tournament_session_id", $rol_tournament_ses)->where("player_id", $player_id )->get();            

            // $puntos = $this->suma_puntaje_semana( $rol_t_s_p_max_points, $player_id );
            // error_log( json_encode( $puntos ) );
            // if( $rol_t_s_p_max_points ){

            //     foreach( $rol_t_s_p_max_points as $key_point => $points ){
                    
            //         $sum    = $sum + $points->points;
            //         $sum_hc = $sum_hc + $points->points + $points->handycap_calculated;

            //         array_push($max_serie, $sum);
            //         array_push($max_serie_hc, $sum_hc);
            //     }  
                                
            // }

            // array_push($max_serie, $puntos["puntos"]);
            // array_push($max_serie_hc, $puntos["sum_hc"]);
            
        // }

        // error_log("MAX SERIE =>". json_encode( $max_serie ) );

        $max_serie_num    = $rol_tournament_sesion;//max($max_serie);
        $max_serie_num_hc = $rol_tournament_sesion_hc;//max($max_serie_hc);

        $result = array( "serie" => $max_serie_num, "serie_hc" => $max_serie_num_hc );
        return $result;


    }

    private function suma_puntaje_semana( $semana, $player_id ){

        $sum    = 0;
        $sum_hc = 0;

        foreach( $semana as $key_point => $points ){
            // error_log( $points->points );
            $sum    = $sum + $points->points;
            $sum_hc = $sum_hc + $points->points + $points->handycap_calculated;

            if( $player_id == 56 ){
                // error_log( "suma_puntaje_semana". $points );
            }

        }

        return array( "puntos" => $sum, "sum_hc" => $sum_hc );

    }

    private function is_negative_number($number=0){

        if( is_numeric($number) AND ($number<0) ){
            return true;
        }else{
            return false;
        }
    
    }

    public function ver_puntaje( Request $request ){

        // error_log( print_r( $request->all(), true ) );

        $response["status"]  = false;   
        // $rol_tournaments = RolTournament::where("id",$request->rol_tournament_id)->where("category_id", $request->category_id)->with(["get_first_team", "get_second_team"])->orderBy("id", "ASC")->get();  
        $rol_tournaments = RolTournament::where("id",$request->rol_tournament_id)->where("category_id", $request->category_id)->first();                

        /******** Primer  - segundo equipo ********/        
        $players_first  = TeamEachCategory::where("team_category_id", $request->category_id)->where("team_id", $rol_tournaments->first_team_id)->with("get_player_in_team_each_category.get_player")->first();
        $players_second = TeamEachCategory::where("team_category_id", $request->category_id)->where("team_id", $rol_tournaments->second_team_id)->with("get_player_in_team_each_category.get_player")->first();
                
        //Obtenemos el puntaje

        $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $rol_tournaments->first_team_id)->orderBy("session_number")->get();
        $second_rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)->where("team_id", $rol_tournaments->second_team_id)->orderBy("session_number")->get();
        
        $rol_tournament_sessions_session = RolTournamentSession::where("rol_tournament_id", $request->rol_tournament_id)
                                                                ->where("validate", 1)->groupBy("session_number")->get()->toArray();
                                                                   

        $first_team_data  = Team::find( $rol_tournaments->first_team_id );
        $second_team_data = Team::find( $rol_tournaments->second_team_id );

        $ver_editar_jugadores = true;

        //se revisa cuantos juegos lleva en la serie
        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        // $league["get_player_types"]->number_game_by_session
        
        if( count( $rol_tournament_sessions_session ) >= $league["get_player_types"]->number_game_by_session ){
            $ver_editar_jugadores = false;
        }

        // if(  $rol_tournaments->first_team_id == $request->team_id ){                    
        //     if( count( $rol_tournament_sessions ) >= $league["get_player_types"]->number_game_by_session ){
        //         $ver_editar_jugadores = false;
        //     }
        // }

        // if(  $rol_tournaments->second_team_id == $request->team_id ){            
        //     if( count( $second_rol_tournament_sessions ) >= $league["get_player_types"]->number_game_by_session ){
        //         $ver_editar_jugadores = false;
        //     }
        // }       

        // $row_team["logo_team"] = ( $team->image_path != null ) ? url($team->image_path) : url("assets/images/team.svg");

        // Cuantos juegos permites por equipo
        $total_dummy_players_available = Handicap::where("league_id", $request->league_id)->first();
        $total_dummy_players_available = $total_dummy_players_available->dummy_by_team;
    
        $first_team = [];
        $primer_equipo = [];
        $ids_first  = array();
        $ids_second = array();
                    

        foreach ($players_first["get_player_in_team_each_category"] as $key_player => $player) {
            
            array_push( $primer_equipo, array("player" => $player) );
            
            $primer_equipo[$key_player]["player"] = $player;
            $puntaje_por_session = array();
            foreach ($rol_tournament_sessions as $rol_tournament_session) {

                // array_push($ids_first, array( "id" => $rol_tournament_session->id, 
                //                               "session" => $rol_tournament_session->session_number ) );

                $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", $player->player_id)->first();                
                
                if ($rol_tournament_session_points != null) {                    
                    $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
                }else{
                    $puntaje_por_session[$rol_tournament_session->session_number] = null;
                    
                }

            }
            
            // VERIFICAR EL HC INICIAL POR QUE NO TIENE VALIDACIONES
            $primer_equipo[$key_player]["puntaje_por_session"] = $puntaje_por_session;
            $primer_equipo[$key_player]["handicap"] = $this->get_handycap_by_player($player, $rol_tournaments->iterator_week, $request->league_id, $request->category_id, $rol_tournaments->first_team_id, $rol_tournaments->type, $rol_tournaments);                    
            
            // error_log($primer_equipo[$key_player]["player"]["nickname"]);

            array_push( $first_team, array( "player" => $primer_equipo[$key_player]["player"], "puntaje_por_session" => $puntaje_por_session, "hc" => $primer_equipo[$key_player]["handicap"] ) );
        }

        
        for ($i = 1; $i <= $total_dummy_players_available; $i++) {

            $total_dummys_first = 0;

            foreach ($rol_tournament_sessions as $rol_tournament_session) {
                
                $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", ($i * -1))->first();                
    
                if ($rol_tournament_session_points != null) {                    
                    $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
                }else{
                    $puntaje_por_session[$rol_tournament_session->session_number] = null;
                    
                }               
                
                $total_dummys_first = $total_dummys_first + $puntaje_por_session[$rol_tournament_session->session_number];
                $primer_equipo[($i * -1)]["puntaje_por_session"] = $puntaje_por_session;
    
            }
            
            array_push($ids_first, array( "total_dummys_first" => $total_dummys_first, "player" => "DUMMY", "puntaje_session" => $puntaje_por_session, "hc" => 0 ) );
        }
        

        $second_team = [];
        $segundo_equipo = [];
                
        foreach ($players_second["get_player_in_team_each_category"] as $key_player => $player) {
                    

            array_push( $segundo_equipo, array("player" => $player) );
            
            $segundo_equipo[$key_player]["player"] = $player;
            $puntaje_por_session = array();
            foreach ($second_rol_tournament_sessions as $rol_tournament_session) {

                // array_push($ids_second, $rol_tournament_session->id);
                $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", $player->player_id)->first();                

                if ($rol_tournament_session_points != null) {                    
                    $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
                }else{
                    $puntaje_por_session[$rol_tournament_session->session_number] = null;
                    
                }

            }            

            $segundo_equipo[$key_player]["puntaje_por_session"] = $puntaje_por_session;
            $segundo_equipo[$key_player]["handicap"] = $this->get_handycap_by_player($player, $rol_tournaments->iterator_week, $request->league_id, $request->category_id, $rol_tournaments->second_team_id, $rol_tournaments->type, $rol_tournaments);                            
    
            array_push( $second_team, array( "total_dummys_first" => $total_dummys_first, "player" => $segundo_equipo[$key_player]["player"], "puntaje_por_session" => $puntaje_por_session, "hc" => $segundo_equipo[$key_player]["handicap"] ) );
        }

        
        for ($i = 1; $i <= $total_dummy_players_available; $i++) {
            
            $total_dummys_second = 0;

            foreach ($second_rol_tournament_sessions as $rol_tournament_session) {
                
                $rol_tournament_session_points = RolTournamentSessionPoint::where("rol_tournament_session_id", $rol_tournament_session->id)->where("player_id", ($i * -1))->first();                

                if ($rol_tournament_session_points != null) {                    
                    $puntaje_por_session[$rol_tournament_session->session_number] = $rol_tournament_session_points->points;
                }else{
                    $puntaje_por_session[$rol_tournament_session->session_number] = null;
                    
                }               

                $segundo_equipo[($i * -1)]["puntaje_por_session"] = $puntaje_por_session;
                $total_dummys_second = $total_dummys_second + $puntaje_por_session[$rol_tournament_session->session_number];
                
            }
            
            array_push($ids_second, array( "total_dummys_second" => $total_dummys_second, "player" => "DUMMY", "puntaje_session" => $puntaje_por_session, "hc" => 0 ) );
        }
        
        
        $league = League::with("get_player_types")->where("id", $request->league_id)->first();
        $player = Player::find( $request->player_id );

        // $rol_tournament_sessions = RolTournamentSession::where("rol_tournament_id", $row["team_1"]["rol_tournament_id"])
        // ->where("validate", 1)->groupBy("session_number")->get()->toArray();

        // if (count($rol_tournament_sessions) >= $league["get_player_types"]->number_game_by_session) {
        
        
        $response["valida_asiganr_jugadores"]  = $ver_editar_jugadores;
        $response["first_team_dummys"]  = $ids_first;
        $response["second_team_dummys"] = $ids_second;
        $response["player"] = $player;
        $response["first_team"] = $first_team;
        $response["second_team"] = $second_team;
        $response["first_rol_tournament_sessions"]  = $rol_tournament_sessions;
        $response["second_rol_tournament_sessions"] = $second_rol_tournament_sessions;
        $response["number_game_by_session"] = $league->get_player_types->number_game_by_session;
        $response["rol_tournament"]   = array( "first_team_win_serie" => $rol_tournaments->first_team_win_serie, "second_team_win_serie" => $rol_tournaments->second_team_win_serie, "suspended" => $rol_tournaments->suspended, "first_team_suspended" => $rol_tournaments->first_team_suspended, "second_team_suspended" => $rol_tournaments->second_team_suspended);
        $response["first_team_data"]  = array( "name" => $first_team_data->name, "logo_team" => ( $first_team_data->image_path != null ) ? url($first_team_data->image_path) : url("assets/images/team.svg")  );
        $response["second_team_data"] = array( "name" => $second_team_data->name, "logo_team" => ( $second_team_data->image_path != null ) ? url($second_team_data->image_path) : url("assets/images/team.svg")  );
        $response["rol_tournaments"] = $rol_tournaments;

        return response()->json( $response );
    }    


    public function get_alerts_player( Request $request ){
        
        $response["status"] = false;

        $alertas = AlertsMovil::where("player_id", $request->id)->get();

        if( $alertas ){
            $response["status"] = true;
        }

        $response["data"] = $alertas;

        return response()->json( $response );

    }

    public function update_token( Request $request ){

        $response["status"] = false;        

        $update = Player::where("id", $request->id)->update(["token_notification" => $request->token_notification]);

        if( $update ){
            $response["status"] = true;
        }

        return response()->json( $response );


    }

    public function handicap( Request $request ){

        $response = array();
        $response["status"] = true;

        $league_id = $request->id_liga;

        $team_categories = TeamCategory::find( $league_id );
        //team_and_category_specifications_id
        error_log( "team_categories => ". $league_id );

        $handicap_each_categories =  HandicapEachCategory::where("team_category_id", $team_categories->team_and_category_specifications_id)->first();
        $response["data"] = $handicap_each_categories;

        return response()->json($response, 200); 
    }

    public function player_nip( Request $request ){

        $response = array();

        // $player = Player::where( "id", $request->id )->where("nip", $request->nip)->get();
        $player = Player::find( $request->id );        
        // error_log( $player );
        if( $player ){

            if( $player->nip === 0 ){
                $response["status"]  = false;                
                $response["mensaje"] = "El jugador no ha asigando un nip";
            }else{
                
                if( $player->nip == $request->nip ){
                    $response["status"]  = true; 
                }else{
                    $response["status"]  = false;                
                    $response["mensaje"] = "El nip es incorrecto, favor de verificar.";
                }

            }

            
        }else{
            $response["status"] = false;
        }

        return response()->json($response, 200); 
    }


    public function premiacion(Request $request)
    {

        $response = array();
        $response["message"] = "";        
        
        $premiacion_each_categories = PremiacionEachCategory::where("league_id", $request->id)->first();
        $premiacion_lugares         = PremiacionLugares::where( "premiacion_each_categories_id", $premiacion_each_categories->id )
                                                        ->where("category_id", $premiacion_each_categories->team_category_id)->get();

        $response["data"] = $premiacion_each_categories;
        $response["premiacion_lugares"] = $premiacion_lugares;

        return response()->json($response, 200);
        
    }

    public function get_tabla_posiciones_individual( Request $request ){
        
        $response["status"]  = true;     

        $team_each = TeamEachCategory::where("team_category_id", $request->category_id)->get();  
        $rol = Rol::where("league_id", $request->id_liga)->first();

        $data_result = [];
        foreach($team_each as $key => $team_id){        
            $result = $this->player_tabla_posiciones(null, $request->id_liga, $request->category_id, $request->week_number, $team_id->team_id, $rol, $request->juegos_jugados_req, $request->juegos_jugados );            
            array_push($data_result, $result);            
        }
    
        // error_log( json_encode( $data_result ) );

        $response["data"] = $data_result;
        $response["num"]  = count( $team_each );

        return response()->json( $response );
    }

    public function upload_images( Request $request ){

        $target_path = "uploads/".$request->id."/";

        //   $target_path = "uploads/";

        if (!file_exists($target_path)) {
            mkdir($target_path, 0777, true);
        }

        $target_path = $target_path . basename( $_FILES['file']['name']);
        
        if(move_uploaded_file($_FILES['file']['tmp_name'], $target_path)) {
            // header('Content-type: application/json');
            // $data = ['success' => true, 'message' => 'Se actualizo correctamente', 'url' => $target_path, "id" => $id];
            // echo json_encode( $data );
            $response["success"] = true;
        } else{
            // header('Content-type: application/json');
            // $data = ['success' => false, 'message' => 'There was an error uploading the file, please try again!', 'data' => target_path ];
            // echo json_encode( $data );
            $response["success"] = false;
        }

        $response["success"] = true;
        $response["message"] = 'Se actualizo correctamente';
        $response["url"] =  $target_path;
        $response["id"]  = $id;

        return response()->json( $response );

    }

}


// $response["RESPONSE"]["UUID"] = "dzGZyeoMQ72wWUyOhnP6wO:APA91bEL2ffZ0BCKjmM6HRM72PHrkFakvM1gDoa3BGm6nURO_LEH21KHYoB2frA8SibZ9fZpKkDgYovVPT6-7s8BOUofg9QokzFUnJT8QJwb2lagUOx8UTK1F0srgYcclsBPIvUwVEjb";
// $response["RESPONSE"]["TEXTO_NOTIFICACION"] = "ejemplo";
// $response["RESPONSE"]["TITLE"] = "beonline";

// Pastora::sendPushNotification($response);




